@@ -25,11 +25,19 @@ extern "C" {
 typedef struct {
     lv_draw_dsc_base_t base;
 
-    lv_opa_t bg_opa;
-    lv_color_t bg_color;
-    lv_grad_dsc_t bg_grad;
-
+    /**Points of the triangle. If `LV_USE_FLOAT` is enabled floats can be used here*/
     lv_point_precise_t p[3];
+
+    /**Color of the triangle*/
+    lv_color_t color;
+
+    /**Opacity of the arc in 0...255 range.
+     * LV_OPA_TRANSP, LV_OPA_10, LV_OPA_20, .. LV_OPA_COVER can be used as well*/
+    lv_opa_t opa;
+
+    /**Describe a gradient. If `grad.dir` is not `LV_GRAD_DIR_NONE` `color` will be ignored*/
+    lv_grad_dsc_t grad;
+
 } lv_draw_triangle_dsc_t;
 
 /**********************
@@ -38,7 +46,7 @@ typedef struct {
 
 /**
  * Initialize a triangle draw descriptor
- * @param dsc       pointer to a draw descriptor
+ * @param draw_dsc  pointer to a draw descriptor
  */
 void lv_draw_triangle_dsc_init(lv_draw_triangle_dsc_t * draw_dsc);
 
@@ -52,7 +60,7 @@ lv_draw_triangle_dsc_t * lv_draw_task_get_triangle_dsc(lv_draw_task_t * task);
 /**
  * Create a triangle draw task
  * @param layer     pointer to a layer
- * @param dsc       pointer to an initialized `lv_draw_triangle_dsc_t` variable
+ * @param draw_dsc  pointer to an initialized `lv_draw_triangle_dsc_t` object
  */
 void lv_draw_triangle(lv_layer_t * layer, const lv_draw_triangle_dsc_t * draw_dsc);
 
