@@ -8,12 +8,21 @@
  *********************/
 #include "lv_draw_dave2d.h"
 #if LV_USE_DRAW_DAVE2D
+#include "../../lv_draw_buf_private.h"
+#include "../../../misc/lv_area_private.h"
 
 /*********************
  *      DEFINES
  *********************/
-#define DRAW_UNIT_ID_DAVE2D     4
+#define DRAW_UNIT_ID_DAVE2D         4
+/* The amount of tasks exercising pressure to the currrent to get finished
+ * This one is used as the main signal to start to render a block of tasks.
+ */
+#define DAVE2D_MAX_DRAW_PRESSURE    256
 
+#if (DAVE2D_MAX_DRAW_PRESSURE < 256)
+    #error "DRAW Pressure should be at least 256 otherwise the Dave engine may crash!"
+#endif
 /**********************
  *      TYPEDEFS
  **********************/
@@ -21,12 +30,8 @@
 /**********************
  *  STATIC PROTOTYPES
  **********************/
-#if LV_USE_OS
-    static void _dave2d_render_thread_cb(void * ptr);
-#endif
 
 static void execute_drawing(lv_draw_dave2d_unit_t * u);
-
 #if defined(RENESAS_CORTEX_M85)
     #if (BSP_CFG_DCACHE_ENABLED)
         static void _dave2d_buf_invalidate_cache_cb(const lv_draw_buf_t * draw_buf, const lv_area_t * area);
@@ -34,13 +39,12 @@ static void execute_drawing(lv_draw_dave2d_unit_t * u);
 #endif
 
 static int32_t _dave2d_evaluate(lv_draw_unit_t * draw_unit, lv_draw_task_t * task);
-
+static int32_t _dave2d_wait_finish(lv_draw_unit_t * draw_unit);
 static int32_t lv_draw_dave2d_dispatch(lv_draw_unit_t * draw_unit, lv_layer_t * layer);
 
 static d2_s32 lv_dave2d_init(void);
-
 static void lv_draw_buf_dave2d_init_handlers(void);
-
+static bool lv_draw_dave2d_image_color_format_supported(lv_color_format_t color_format);
 void dave2d_execute_dlist_and_flush(void);
 
 /**********************
@@ -51,11 +55,16 @@ void dave2d_execute_dlist_and_flush(void);
  *  STATIC VARIABLES
  **********************/
 
-d2_device * _d2_handle;
-d2_renderbuffer * _renderbuffer;
-d2_renderbuffer * _blit_renderbuffer;
+static d2_device * _d2_handle;
 
-lv_ll_t  _ll_Dave2D_Tasks;
+/* Main render buffer, used to carry the block of dave commands for any shape */
+static d2_renderbuffer * _renderbuffer;
+
+/* Label dedicated render buffer, used to carry only label related dave commands */
+static d2_renderbuffer * _label_renderbuffer;
+
+static lv_ll_t  draw_tasks_on_dlist;
+static uint32_t draw_pressure = 0;
 
 #if LV_USE_OS
     lv_mutex_t xd2Semaphore;
@@ -78,6 +87,8 @@ void lv_draw_dave2d_init(void)
     lv_draw_dave2d_unit_t * draw_dave2d_unit = lv_draw_create_unit(sizeof(lv_draw_dave2d_unit_t));
     draw_dave2d_unit->base_unit.dispatch_cb = lv_draw_dave2d_dispatch;
     draw_dave2d_unit->base_unit.evaluate_cb = _dave2d_evaluate;
+    draw_dave2d_unit->base_unit.wait_for_finish_cb = _dave2d_wait_finish;
+    draw_dave2d_unit->base_unit.name = "DAVE2D";
     draw_dave2d_unit->idx = DRAW_UNIT_ID_DAVE2D;
 
     result = lv_dave2d_init();
@@ -87,18 +98,13 @@ void lv_draw_dave2d_init(void)
     lv_result_t res;
     res =  lv_mutex_init(&xd2Semaphore);
     LV_ASSERT(LV_RESULT_OK == res);
-
     draw_dave2d_unit->pd2Mutex    = &xd2Semaphore;
 #endif
 
     draw_dave2d_unit->d2_handle = _d2_handle;
     draw_dave2d_unit->renderbuffer = _renderbuffer;
-    _lv_ll_init(&_ll_Dave2D_Tasks, 4);
-
-#if LV_USE_OS
-    lv_thread_init(&draw_dave2d_unit->thread, LV_THREAD_PRIO_HIGH, _dave2d_render_thread_cb, 8 * 1024, draw_dave2d_unit);
-#endif
-
+    draw_dave2d_unit->label_renderbuffer = _label_renderbuffer;
+    lv_ll_init(&draw_tasks_on_dlist, sizeof(uintptr_t));
 }
 
 /**********************
@@ -207,6 +213,26 @@ static void _dave2d_buf_copy(void * dest_buf, uint32_t dest_w, uint32_t dest_h,
 }
 #endif
 
+static bool lv_draw_dave2d_image_color_format_supported(lv_color_format_t color_format)
+{
+    bool supported = false;
+
+    switch(color_format) {
+        case LV_COLOR_FORMAT_A8:
+        case LV_COLOR_FORMAT_RGB565:
+        case LV_COLOR_FORMAT_ARGB1555:
+        case LV_COLOR_FORMAT_ARGB4444:
+        case LV_COLOR_FORMAT_ARGB8888:
+        case LV_COLOR_FORMAT_XRGB8888:
+            supported = true;
+            break;
+        default:
+            break;
+    }
+
+    return supported;
+}
+
 #define USE_D2 (1)
 
 static int32_t _dave2d_evaluate(lv_draw_unit_t * u, lv_draw_task_t * t)
@@ -214,6 +240,11 @@ static int32_t _dave2d_evaluate(lv_draw_unit_t * u, lv_draw_task_t * t)
     LV_UNUSED(u);
     int32_t ret = 0;
 
+    lv_draw_dsc_base_t * draw_dsc_base = (lv_draw_dsc_base_t *) t->draw_dsc;
+
+    if(!lv_draw_dave2d_is_dest_cf_supported(draw_dsc_base->layer->color_format))
+        return 0;
+
     switch(t->type) {
         case LV_DRAW_TASK_TYPE_FILL: {
 #if USE_D2
@@ -241,6 +272,11 @@ static int32_t _dave2d_evaluate(lv_draw_unit_t * u, lv_draw_task_t * t)
             }
 
         case LV_DRAW_TASK_TYPE_IMAGE: {
+                lv_draw_image_dsc_t * dsc = t->draw_dsc;
+                if(!lv_draw_dave2d_image_color_format_supported(dsc->header.cf)) {
+                    ret = 0;
+                    break;
+                }
 #if USE_D2
                 t->preferred_draw_unit_id = DRAW_UNIT_ID_DAVE2D;
                 t->preference_score = 0;
@@ -330,118 +366,125 @@ static int32_t _dave2d_evaluate(lv_draw_unit_t * u, lv_draw_task_t * t)
     return ret;
 }
 
-#define DAVE2D_REFERRING_WATERMARK  10
 
 static int32_t lv_draw_dave2d_dispatch(lv_draw_unit_t * draw_unit, lv_layer_t * layer)
 {
     lv_draw_dave2d_unit_t * draw_dave2d_unit = (lv_draw_dave2d_unit_t *) draw_unit;
-#if  (0 == D2_RENDER_EACH_OPERATION)
-    static uint32_t ref_count = 0;
-#endif
+    uint32_t deps = 0;
 
-    /*Return immediately if it's busy with draw task*/
-    if(draw_dave2d_unit->task_act) return 0;
+    if(draw_dave2d_unit->task_act) {
+        /* Return immediately if it's busy with draw task */
+        return LV_DRAW_UNIT_IDLE;
+    }
 
     lv_draw_task_t * t = NULL;
     t = lv_draw_get_next_available_task(layer, NULL, DRAW_UNIT_ID_DAVE2D);
-
-    /* Return 0 is no selection, some tasks can be supported by other units. */
     if(t == NULL) {
-#if  (0 == D2_RENDER_EACH_OPERATION)
-        if(false == _lv_ll_is_empty(&_ll_Dave2D_Tasks)) {
-            ref_count = 0;
+        /* No valid task, but there are tasks waiting to be rendered,
+         * start to draw then immediatelly.
+         */
+        if(false == lv_ll_is_empty(&draw_tasks_on_dlist)) {
+            draw_pressure = 0;
             dave2d_execute_dlist_and_flush();
         }
-#endif
-        return 0;
+        return LV_DRAW_UNIT_IDLE;  /* This task is not for us. */
     }
 
-    if(t->preferred_draw_unit_id != DRAW_UNIT_ID_DAVE2D) {
-        return 0;
-    }
+    if((t->preferred_draw_unit_id != DRAW_UNIT_ID_DAVE2D)) return LV_DRAW_UNIT_IDLE;
+    if(!lv_draw_dave2d_is_dest_cf_supported(layer->color_format)) return LV_DRAW_UNIT_IDLE;
 
     void * buf = lv_draw_layer_alloc_buf(layer);
-    if(buf == NULL) {
-        return -1;
-    }
+    if(buf == NULL) return LV_DRAW_UNIT_IDLE;
 
-#if  (0 == D2_RENDER_EACH_OPERATION)
-    ref_count += lv_draw_get_dependent_count(t);
+    deps = lv_draw_get_dependent_count(t);
+    if(deps > 0 || draw_pressure > 0) {
+        draw_pressure += deps;
+        if(draw_pressure < DAVE2D_MAX_DRAW_PRESSURE) {
+            /* No other tasks are pressuring to get the current block
+             * of tasks including the latest one, just accumulate it
+             * and tells the drawing pipeline to send a new one if there is any
+             */
+            lv_draw_task_t ** p_new_list_entry;
+            p_new_list_entry = lv_ll_ins_tail(&draw_tasks_on_dlist);
+            *p_new_list_entry = t;
 
-    if(DAVE2D_REFERRING_WATERMARK < ref_count) {
-        ref_count = 0;
-        dave2d_execute_dlist_and_flush();
-    }
+            draw_dave2d_unit->task_act = t;
+            draw_dave2d_unit->task_act->state = LV_DRAW_TASK_STATE_IN_PROGRESS;
+            execute_drawing(draw_dave2d_unit);
 
-    lv_draw_task_t ** p_new_list_entry;
-    p_new_list_entry = _lv_ll_ins_tail(&_ll_Dave2D_Tasks);
-    *p_new_list_entry = t;
-#endif
+            draw_dave2d_unit->task_act = NULL;
+            lv_draw_dispatch_request();
 
-    t->state = LV_DRAW_TASK_STATE_IN_PROGRESS;
-    draw_dave2d_unit->base_unit.target_layer = layer;
-    draw_dave2d_unit->base_unit.clip_area = &t->clip_area;
-    draw_dave2d_unit->task_act = t;
-
-#if LV_USE_OS
-    /*Let the render thread work*/
-    lv_thread_sync_signal(&draw_dave2d_unit->sync);
-#else
-    execute_drawing(draw_dave2d_unit);
-#if  (D2_RENDER_EACH_OPERATION)
-    draw_dave2d_unit->task_act->state = LV_DRAW_TASK_STATE_READY;
-#endif
-    draw_dave2d_unit->task_act = NULL;
-
-    /*The draw unit is free now. Request a new dispatching as it can get a new task*/
-    lv_draw_dispatch_request();
-
-#endif
-    return 1;
-}
-
-#if LV_USE_OS
-static void _dave2d_render_thread_cb(void * ptr)
-{
-    lv_draw_dave2d_unit_t * u = ptr;
-
-    lv_thread_sync_init(&u->sync);
-
-    while(1) {
-        while(u->task_act == NULL) {
-            lv_thread_sync_wait(&u->sync);
+            return 1;
         }
+        /* If the pressure for drawing value was maxed-out, it is time to render
+         * return IDLE to force the drawing pipeline to wait while the Dave
+         * draw the block of tasks in a single row
+         */
+        return LV_DRAW_UNIT_IDLE;
+    }
+    else {
+        /* Handles a special case when there is no sufficient draw pressure
+         * But the actual task did not carry any extra pressure to get drew
+         * in this case, the drawing pipeline have a few set of tasks that
+         * don't make sense to accumulate them, just do a run to completion
+         * here, that is it, render the incoming task immediately.
+         */
+        draw_dave2d_unit->task_act = t;
+        draw_dave2d_unit->task_act->state = LV_DRAW_TASK_STATE_IN_PROGRESS;
+        d2_selectrenderbuffer(_d2_handle, _renderbuffer);
+        execute_drawing(draw_dave2d_unit);
+        d2_executerenderbuffer(_d2_handle, _renderbuffer, 0);
+        d2_flushframe(_d2_handle);
 
-        execute_drawing(u);
-
-        /*Cleanup*/
-#if  (D2_RENDER_EACH_OPERATION)
-        u->task_act->state = LV_DRAW_TASK_STATE_READY;
-#endif
-        u->task_act = NULL;
-
-        /*The draw unit is free now. Request a new dispatching as it can get a new task*/
+        draw_dave2d_unit->task_act->state = LV_DRAW_TASK_STATE_FINISHED;
+        draw_dave2d_unit->task_act = NULL;
         lv_draw_dispatch_request();
+
+        return 1;
     }
 }
-#endif
+
+static int32_t _dave2d_wait_finish(lv_draw_unit_t * draw_unit)
+{
+    /* If the drawing pipeline is waiting it means the pressure for drawing
+     * has been maxed out, defer the block of tasks to be rendered by the
+     * Dave and wait for its interrupt. (Dave2D driver is RTOS aware, no need for semaphores);
+     */
+    lv_draw_dave2d_unit_t * draw_dave2d_unit = (lv_draw_dave2d_unit_t *) draw_unit;
+
+    if(!draw_pressure) {
+        /* It reached here because Dave2D Draw Unit was not suitable to take a task
+         * While there is nothing being rendered, prevent the dead lock
+         * by flushing the GPU command buffer empty and just return.
+         */
+        return 0;
+    }
+    dave2d_execute_dlist_and_flush();
+    draw_pressure = 0;
+
+    return 0;
+}
 
 static void execute_drawing(lv_draw_dave2d_unit_t * u)
 {
     /*Render the draw task*/
     lv_draw_task_t * t = u->task_act;
 
+    /* remember draw unit for access to unit's context */
+    t->draw_unit = (lv_draw_unit_t *)u;
+
 #if defined(RENESAS_CORTEX_M85)
 #if (BSP_CFG_DCACHE_ENABLED)
-    lv_layer_t * layer = u->base_unit.target_layer;
+    lv_layer_t * layer = t->target_layer;
     lv_area_t clipped_area;
     int32_t x;
     int32_t y;
 
-    _lv_area_intersect(&clipped_area,  &t->area, u->base_unit.clip_area);
+    lv_area_intersect(&clipped_area,  &t->area, &t->clip_area);
 
-    x = 0 - u->base_unit.target_layer->buf_area.x1;
-    y = 0 - u->base_unit.target_layer->buf_area.y1;
+    x = 0 - t->target_layer->buf_area.x1;
+    y = 0 - t->target_layer->buf_area.y1;
 
     lv_area_move(&clipped_area, x, y);
 
@@ -452,44 +495,49 @@ static void execute_drawing(lv_draw_dave2d_unit_t * u)
 
     switch(t->type) {
         case LV_DRAW_TASK_TYPE_FILL:
-            lv_draw_dave2d_fill(u, t->draw_dsc, &t->area);
+            lv_draw_dave2d_fill(t, t->draw_dsc, &t->area);
             break;
         case LV_DRAW_TASK_TYPE_BORDER:
-            lv_draw_dave2d_border(u, t->draw_dsc, &t->area);
+            lv_draw_dave2d_border(t, t->draw_dsc, &t->area);
             break;
         case LV_DRAW_TASK_TYPE_BOX_SHADOW:
-            //lv_draw_dave2d_box_shadow(u, t->draw_dsc, &t->area);
+            //lv_draw_dave2d_box_shadow(t, t->draw_dsc, &t->area);
             break;
 #if 0
         case LV_DRAW_TASK_TYPE_BG_IMG:
-            //lv_draw_dave2d_bg_image(u, t->draw_dsc, &t->area);
+            //lv_draw_dave2d_bg_image(t, t->draw_dsc, &t->area);
             break;
 #endif
         case LV_DRAW_TASK_TYPE_LABEL:
-            lv_draw_dave2d_label(u, t->draw_dsc, &t->area);
+            lv_draw_dave2d_label(t, t->draw_dsc, &t->area);
             break;
         case LV_DRAW_TASK_TYPE_IMAGE:
-            lv_draw_dave2d_image(u, t->draw_dsc, &t->area);
+            lv_draw_dave2d_image(t, t->draw_dsc, &t->area);
             break;
         case LV_DRAW_TASK_TYPE_LINE:
-            lv_draw_dave2d_line(u, t->draw_dsc);
+            lv_draw_dave2d_line(t, t->draw_dsc);
             break;
         case LV_DRAW_TASK_TYPE_ARC:
-            lv_draw_dave2d_arc(u, t->draw_dsc, &t->area);
+            lv_draw_dave2d_arc(t, t->draw_dsc, &t->area);
             break;
         case LV_DRAW_TASK_TYPE_TRIANGLE:
-            lv_draw_dave2d_triangle(u, t->draw_dsc);
+            lv_draw_dave2d_triangle(t, t->draw_dsc);
             break;
         case LV_DRAW_TASK_TYPE_LAYER:
-            //lv_draw_dave2d_layer(u, t->draw_dsc, &t->area);
+            //lv_draw_dave2d_layer(t, t->draw_dsc, &t->area);
             break;
         case LV_DRAW_TASK_TYPE_MASK_RECTANGLE:
-            //lv_draw_dave2d_mask_rect(u, t->draw_dsc, &t->area);
+            //lv_draw_dave2d_mask_rect(t, t->draw_dsc, &t->area);
             break;
         default:
             break;
     }
 
+#if defined(RENESAS_CORTEX_M85)
+#if (BSP_CFG_DCACHE_ENABLED)
+    lv_draw_buf_invalidate_cache(layer->draw_buf, &clipped_area);
+#endif
+#endif
 }
 
 static d2_s32 lv_dave2d_init(void)
@@ -524,23 +572,23 @@ static d2_s32 lv_dave2d_init(void)
     result = d2_setlinejoin(_d2_handle, d2_lj_miter);
 
     /* set blocksize for default displaylist */
-    result = d2_setdlistblocksize(_d2_handle, 25);
+    result = d2_setdlistblocksize(_d2_handle, 20);
     if(D2_OK != result) {
         LV_LOG_ERROR("Could NOT d2_setdlistblocksize");
         d2_closedevice(_d2_handle);
         return result;
     }
 
-    _blit_renderbuffer = d2_newrenderbuffer(_d2_handle, 20, 20);
-    if(!_blit_renderbuffer) {
+    _renderbuffer = d2_newrenderbuffer(_d2_handle, 250, 25);
+    if(!_renderbuffer) {
         LV_LOG_ERROR("NO renderbuffer");
         d2_closedevice(_d2_handle);
 
         return D2_NOMEMORY;
     }
 
-    _renderbuffer = d2_newrenderbuffer(_d2_handle, 20, 20);
-    if(!_renderbuffer) {
+    _label_renderbuffer = d2_newrenderbuffer(_d2_handle, 250, 25);
+    if(!_label_renderbuffer) {
         LV_LOG_ERROR("NO renderbuffer");
         d2_closedevice(_d2_handle);
 
@@ -558,6 +606,10 @@ static d2_s32 lv_dave2d_init(void)
 
 void dave2d_execute_dlist_and_flush(void)
 {
+    d2_s32 result;
+    lv_draw_task_t ** p_list_entry;
+    lv_draw_task_t * p_list_entry1;
+
 #if LV_USE_OS
     lv_result_t  status;
 
@@ -565,11 +617,6 @@ void dave2d_execute_dlist_and_flush(void)
     LV_ASSERT(LV_RESULT_OK == status);
 #endif
 
-    d2_s32     result;
-    lv_draw_task_t ** p_list_entry;
-    lv_draw_task_t * p_list_entry1;
-
-    // Execute render operations
     result = d2_executerenderbuffer(_d2_handle, _renderbuffer, 0);
     LV_ASSERT(D2_OK == result);
 
@@ -579,11 +626,11 @@ void dave2d_execute_dlist_and_flush(void)
     result = d2_selectrenderbuffer(_d2_handle, _renderbuffer);
     LV_ASSERT(D2_OK == result);
 
-    while(false == _lv_ll_is_empty(&_ll_Dave2D_Tasks)) {
-        p_list_entry = _lv_ll_get_tail(&_ll_Dave2D_Tasks);
+    while(false == lv_ll_is_empty(&draw_tasks_on_dlist)) {
+        p_list_entry = lv_ll_get_tail(&draw_tasks_on_dlist);
         p_list_entry1 = *p_list_entry;
-        p_list_entry1->state = LV_DRAW_TASK_STATE_READY;
-        _lv_ll_remove(&_ll_Dave2D_Tasks, p_list_entry);
+        p_list_entry1->state = LV_DRAW_TASK_STATE_FINISHED;
+        lv_ll_remove(&draw_tasks_on_dlist, p_list_entry);
         lv_free(p_list_entry);
     }
 
