@@ -28,12 +28,12 @@ extern "C" {
 
 d2_color lv_draw_dave2d_lv_colour_to_d2_colour(lv_color_t color);
 
-d2_s32 lv_draw_dave2d_cf_fb_get(void);
-
 d2_u32 lv_draw_dave2d_lv_colour_fmt_to_d2_fmt(lv_color_format_t colour_format);
 
 void d2_framebuffer_from_layer(d2_device * handle, lv_layer_t * layer);
 
+bool lv_draw_dave2d_is_dest_cf_supported(lv_color_format_t cf);
+
 /**********************
  *      MACROS
  **********************/
