@@ -1,4 +1,4 @@
-﻿/**
+/**
  * @file lv_draw_sw_transform.c
  *
  */
@@ -51,29 +51,49 @@ typedef struct {
 static void transform_point_upscaled(point_transform_dsc_t * t, int32_t xin, int32_t yin, int32_t * xout,
                                      int32_t * yout);
 
+#if LV_DRAW_SW_SUPPORT_RGB888 || LV_DRAW_SW_SUPPORT_XRGB8888
 static void transform_rgb888(const uint8_t * src, int32_t src_w, int32_t src_h, int32_t src_stride,
                              int32_t xs_ups, int32_t ys_ups, int32_t xs_step, int32_t ys_step,
                              int32_t x_end, uint8_t * dest_buf, bool aa, uint32_t px_size);
+#endif
 
+#if LV_DRAW_SW_SUPPORT_ARGB8888
 static void transform_argb8888(const uint8_t * src, int32_t src_w, int32_t src_h, int32_t src_stride,
                                int32_t xs_ups, int32_t ys_ups, int32_t xs_step, int32_t ys_step,
                                int32_t x_end, uint8_t * dest_buf, bool aa);
+#endif
 
+#if LV_DRAW_SW_SUPPORT_ARGB8888_PREMULTIPLIED
+static void transform_argb8888_premultiplied(const uint8_t * src, int32_t src_w, int32_t src_h, int32_t src_stride,
+                                             int32_t xs_ups, int32_t ys_ups, int32_t xs_step, int32_t ys_step,
+                                             int32_t x_end, uint8_t * dest_buf, bool aa);
+#endif
+
+#if LV_DRAW_SW_SUPPORT_RGB565A8
 static void transform_rgb565a8(const uint8_t * src, int32_t src_w, int32_t src_h, int32_t src_stride,
                                int32_t xs_ups, int32_t ys_ups, int32_t xs_step, int32_t ys_step,
                                int32_t x_end, uint16_t * cbuf, uint8_t * abuf, bool src_has_a8, bool aa);
+#endif
 
+#if LV_DRAW_SW_SUPPORT_RGB565_SWAPPED
+static void transform_rgb565a8_swapped(const uint8_t * src, int32_t src_w, int32_t src_h, int32_t src_stride,
+                                       int32_t xs_ups, int32_t ys_ups, int32_t xs_step, int32_t ys_step,
+                                       int32_t x_end, uint16_t * cbuf, uint8_t * abuf, bool src_has_a8, bool aa);
+#endif
+
+#if LV_DRAW_SW_SUPPORT_A8
 static void transform_a8(const uint8_t * src, int32_t src_w, int32_t src_h, int32_t src_stride,
                          int32_t xs_ups, int32_t ys_ups, int32_t xs_step, int32_t ys_step,
                          int32_t x_end, uint8_t * abuf, bool aa);
+#endif
 
+#if LV_DRAW_SW_SUPPORT_L8
+#if LV_DRAW_SW_SUPPORT_AL88
 static void transform_l8_to_al88(const uint8_t * src, int32_t src_w, int32_t src_h, int32_t src_stride,
                                  int32_t xs_ups, int32_t ys_ups, int32_t xs_step, int32_t ys_step,
                                  int32_t x_end, uint8_t * abuf, bool aa);
-
-static void transform_l8_to_argb8888(const uint8_t * src, int32_t src_w, int32_t src_h, int32_t src_stride,
-                                     int32_t xs_ups, int32_t ys_ups, int32_t xs_step, int32_t ys_step,
-                                     int32_t x_end, uint8_t * abuf, bool aa);
+#endif
+#endif /*LV_DRAW_SW_SUPPORT_L8*/
 
 /**********************
  *  STATIC VARIABLES
@@ -87,11 +107,10 @@ static void transform_l8_to_argb8888(const uint8_t * src, int32_t src_w, int32_t
  *   GLOBAL FUNCTIONS
  **********************/
 
-void lv_draw_sw_transform(lv_draw_unit_t * draw_unit, const lv_area_t * dest_area, const void * src_buf,
+void lv_draw_sw_transform(const lv_area_t * dest_area, const void * src_buf,
                           int32_t src_w, int32_t src_h, int32_t src_stride,
                           const lv_draw_image_dsc_t * draw_dsc, const lv_draw_image_sup_t * sup, lv_color_format_t src_cf, void * dest_buf)
 {
-    LV_UNUSED(draw_unit);
     LV_UNUSED(sup);
 
     point_transform_dsc_t tr_dsc;
@@ -122,10 +141,7 @@ void lv_draw_sw_transform(lv_draw_unit_t * draw_unit, const lv_area_t * dest_are
 
     int32_t dest_stride_a8 = dest_w;
     int32_t dest_stride;
-    if(src_cf == LV_COLOR_FORMAT_L8) {
-        dest_stride = dest_w * ((draw_dsc->recolor_opa >= LV_OPA_MIN) ? 4 : 2);
-    }
-    else if(src_cf == LV_COLOR_FORMAT_RGB888) {
+    if(src_cf == LV_COLOR_FORMAT_RGB888) {
         dest_stride = dest_w * lv_color_format_get_size(LV_COLOR_FORMAT_ARGB8888);
     }
     else if((src_cf == LV_COLOR_FORMAT_RGB565A8) || (src_cf == LV_COLOR_FORMAT_L8)) {
@@ -136,7 +152,7 @@ void lv_draw_sw_transform(lv_draw_unit_t * draw_unit, const lv_area_t * dest_are
     }
 
     uint8_t * alpha_buf;
-    if(src_cf == LV_COLOR_FORMAT_RGB565 || src_cf == LV_COLOR_FORMAT_RGB565A8) {
+    if(src_cf == LV_COLOR_FORMAT_RGB565 || src_cf == LV_COLOR_FORMAT_RGB565_SWAPPED || src_cf == LV_COLOR_FORMAT_RGB565A8) {
         alpha_buf = dest_buf;
         alpha_buf += dest_stride * dest_h;
     }
@@ -150,6 +166,13 @@ void lv_draw_sw_transform(lv_draw_unit_t * draw_unit, const lv_area_t * dest_are
     int32_t xs_ups = 0, ys_ups = 0, ys_ups_start = 0, ys_step_256_original = 0;
     int32_t xs_step_256 = 0, ys_step_256 = 0;
 
+    /*When some of the color formats are disabled, these variables could be unused, avoid warning here*/
+    LV_UNUSED(aa);
+    LV_UNUSED(xs_ups);
+    LV_UNUSED(ys_ups);
+    LV_UNUSED(xs_step_256);
+    LV_UNUSED(ys_step_256);
+
     /*If scaled only make some simplification to avoid rounding errors.
      *For example if there is a 100x100 image zoomed to 300%
      *The destination area in X will be x1=0; x2=299
@@ -215,39 +238,67 @@ void lv_draw_sw_transform(lv_draw_unit_t * draw_unit, const lv_area_t * dest_are
         }
 
         switch(src_cf) {
+#if LV_DRAW_SW_SUPPORT_XRGB8888
             case LV_COLOR_FORMAT_XRGB8888:
                 transform_rgb888(src_buf, src_w, src_h, src_stride, xs_ups, ys_ups, xs_step_256, ys_step_256, dest_w, dest_buf, aa,
                                  4);
                 break;
+#endif
+#if LV_DRAW_SW_SUPPORT_RGB888
             case LV_COLOR_FORMAT_RGB888:
                 transform_rgb888(src_buf, src_w, src_h, src_stride, xs_ups, ys_ups, xs_step_256, ys_step_256, dest_w, dest_buf, aa,
                                  3);
                 break;
+#endif
+#if LV_DRAW_SW_SUPPORT_A8
             case LV_COLOR_FORMAT_A8:
                 transform_a8(src_buf, src_w, src_h, src_stride, xs_ups, ys_ups, xs_step_256, ys_step_256, dest_w, dest_buf, aa);
                 break;
+#endif
+#if LV_DRAW_SW_SUPPORT_ARGB8888
             case LV_COLOR_FORMAT_ARGB8888:
                 transform_argb8888(src_buf, src_w, src_h, src_stride, xs_ups, ys_ups, xs_step_256, ys_step_256, dest_w, dest_buf,
                                    aa);
                 break;
+#endif
+#if LV_DRAW_SW_SUPPORT_ARGB8888_PREMULTIPLIED
+            case LV_COLOR_FORMAT_ARGB8888_PREMULTIPLIED:
+                transform_argb8888_premultiplied(src_buf, src_w, src_h, src_stride, xs_ups, ys_ups, xs_step_256, ys_step_256, dest_w,
+                                                 dest_buf,
+                                                 aa);
+                break;
+#endif
+#if LV_DRAW_SW_SUPPORT_RGB565 && LV_DRAW_SW_SUPPORT_RGB565A8
             case LV_COLOR_FORMAT_RGB565:
                 transform_rgb565a8(src_buf, src_w, src_h, src_stride, xs_ups, ys_ups, xs_step_256, ys_step_256, dest_w, dest_buf,
                                    alpha_buf, false, aa);
                 break;
+#endif
+#if LV_DRAW_SW_SUPPORT_RGB565_SWAPPED
+            case LV_COLOR_FORMAT_RGB565_SWAPPED:
+                transform_rgb565a8_swapped(src_buf, src_w, src_h, src_stride, xs_ups, ys_ups, xs_step_256, ys_step_256, dest_w,
+                                           dest_buf, alpha_buf, false, aa);
+                break;
+#endif
+#if LV_DRAW_SW_SUPPORT_RGB565A8
             case LV_COLOR_FORMAT_RGB565A8:
                 transform_rgb565a8(src_buf, src_w, src_h, src_stride, xs_ups, ys_ups, xs_step_256, ys_step_256, dest_w,
                                    (uint16_t *)dest_buf,
                                    alpha_buf, true, aa);
                 break;
+#endif
+
+#if LV_DRAW_SW_SUPPORT_L8 && LV_DRAW_SW_SUPPORT_AL88
             case LV_COLOR_FORMAT_L8:
-                if(draw_dsc->recolor_opa >= LV_OPA_MIN)
-                    transform_l8_to_argb8888(src_buf, src_w, src_h, src_stride, xs_ups, ys_ups, xs_step_256, ys_step_256, dest_w, dest_buf,
-                                             aa);
-                else
-                    transform_l8_to_al88(src_buf, src_w, src_h, src_stride, xs_ups, ys_ups, xs_step_256, ys_step_256, dest_w, dest_buf, aa);
+                transform_l8_to_al88(src_buf, src_w, src_h, src_stride, xs_ups, ys_ups, xs_step_256, ys_step_256, dest_w, dest_buf, aa);
                 break;
+#endif /*LV_DRAW_SW_SUPPORT_L8 && (LV_DRAW_SW_SUPPORT_ARGB8888 || LV_DRAW_SW_SUPPORT_AL88)*/
             default:
-                break;
+                LV_LOG_WARN("Color format 0x%02X is not enabled. "
+                            "See lv_color.h to find the name of the color formats and "
+                            "enable the related LV_DRAW_SW_SUPPORT_* in lv_conf.h.",
+                            src_cf);
+                return;
         }
 
         dest_buf = (uint8_t *)dest_buf + dest_stride;
@@ -259,6 +310,8 @@ void lv_draw_sw_transform(lv_draw_unit_t * draw_unit, const lv_area_t * dest_are
  *   STATIC FUNCTIONS
  **********************/
 
+#if LV_DRAW_SW_SUPPORT_RGB888 || LV_DRAW_SW_SUPPORT_XRGB8888
+
 static void transform_rgb888(const uint8_t * src, int32_t src_w, int32_t src_h, int32_t src_stride,
                              int32_t xs_ups, int32_t ys_ups, int32_t xs_step, int32_t ys_step,
                              int32_t x_end, uint8_t * dest_buf, bool aa, uint32_t px_size)
@@ -355,6 +408,10 @@ static void transform_rgb888(const uint8_t * src, int32_t src_w, int32_t src_h,
     }
 }
 
+#endif
+
+#if LV_DRAW_SW_SUPPORT_ARGB8888
+
 static void transform_argb8888(const uint8_t * src, int32_t src_w, int32_t src_h, int32_t src_stride,
                                int32_t xs_ups, int32_t ys_ups, int32_t xs_step, int32_t ys_step,
                                int32_t x_end, uint8_t * dest_buf, bool aa)
@@ -418,7 +475,7 @@ static void transform_argb8888(const uint8_t * src, int32_t src_w, int32_t src_h
                 dest_c32[x].alpha = (dest_c32[x].alpha * (0xFF - ys_fract)) >> 8;
             }
             else if(!lv_color32_eq(dest_c32[x], px_ver)) {
-                dest_c32[x].alpha = ((px_ver.alpha * ys_fract) + (dest_c32[x].alpha * (0xFF - ys_fract))) >> 8;
+                if(dest_c32[x].alpha) dest_c32[x].alpha = ((px_ver.alpha * ys_fract) + (dest_c32[x].alpha * (0xFF - ys_fract))) >> 8;
                 px_ver.alpha = ys_fract;
                 dest_c32[x] = lv_color_mix32(px_ver, dest_c32[x]);
             }
@@ -427,7 +484,7 @@ static void transform_argb8888(const uint8_t * src, int32_t src_w, int32_t src_h
                 dest_c32[x].alpha = (dest_c32[x].alpha * (0xFF - xs_fract)) >> 8;
             }
             else if(!lv_color32_eq(dest_c32[x], px_hor)) {
-                dest_c32[x].alpha = ((px_hor.alpha * xs_fract) + (dest_c32[x].alpha * (0xFF - xs_fract))) >> 8;
+                if(dest_c32[x].alpha) dest_c32[x].alpha = ((px_hor.alpha * xs_fract) + (dest_c32[x].alpha * (0xFF - xs_fract))) >> 8;
                 px_hor.alpha = xs_fract;
                 dest_c32[x] = lv_color_mix32(px_hor, dest_c32[x]);
             }
@@ -444,6 +501,142 @@ static void transform_argb8888(const uint8_t * src, int32_t src_w, int32_t src_h
     }
 }
 
+#endif
+
+#if LV_DRAW_SW_SUPPORT_ARGB8888_PREMULTIPLIED
+
+static lv_color32_t unpremultiply(lv_color32_t c)
+{
+    if(c.alpha == 0) {
+        c.red = 0;
+        c.green = 0;
+        c.blue = 0;
+    }
+    else {
+        uint16_t reciprocal_alpha = (255 * 256) / c.alpha;
+        c.red = (c.red * reciprocal_alpha) >> 8;
+        c.green = (c.green * reciprocal_alpha) >> 8;
+        c.blue = (c.blue  * reciprocal_alpha) >> 8;
+    }
+
+    return c;
+}
+
+static void transform_argb8888_premultiplied(const uint8_t * src, int32_t src_w, int32_t src_h, int32_t src_stride,
+                                             int32_t xs_ups, int32_t ys_ups, int32_t xs_step, int32_t ys_step,
+                                             int32_t x_end, uint8_t * dest_buf, bool aa)
+{
+    int32_t xs_ups_start = xs_ups;
+    int32_t ys_ups_start = ys_ups;
+    lv_color32_t * dest_c32 = (lv_color32_t *) dest_buf;
+
+    int32_t x;
+    for(x = 0; x < x_end; x++) {
+        xs_ups = xs_ups_start + ((xs_step * x) >> 8);
+        ys_ups = ys_ups_start + ((ys_step * x) >> 8);
+
+        int32_t xs_int = xs_ups >> 8;
+        int32_t ys_int = ys_ups >> 8;
+
+        /*Fully out of the image*/
+        if(xs_int < 0 || xs_int >= src_w || ys_int < 0 || ys_int >= src_h) {
+            ((uint32_t *)dest_buf)[x] = 0x00000000;
+            continue;
+        }
+
+        /*Get the direction the hor and ver neighbor
+         *`fract` will be in range of 0x00..0xFF and `next` (+/-1) indicates the direction*/
+        int32_t xs_fract = xs_ups & 0xFF;
+        int32_t ys_fract = ys_ups & 0xFF;
+
+        int32_t x_next;
+        int32_t y_next;
+        if(xs_fract < 0x80) {
+            x_next = -1;
+            xs_fract = 0x7F - xs_fract;
+        }
+        else {
+            x_next = 1;
+            xs_fract = xs_fract - 0x80;
+        }
+        if(ys_fract < 0x80) {
+            y_next = -1;
+            ys_fract = 0x7F - ys_fract;
+        }
+        else {
+            y_next = 1;
+            ys_fract = ys_fract - 0x80;
+        }
+
+        const lv_color32_t * src_c32 = (const lv_color32_t *)(src + ys_int * src_stride + xs_int * 4);
+
+        dest_c32[x] = src_c32[0];
+
+        if(aa &&
+           xs_int + x_next >= 0 &&
+           xs_int + x_next <= src_w - 1 &&
+           ys_int + y_next >= 0 &&
+           ys_int + y_next <= src_h - 1) {
+
+            lv_color32_t px_hor = src_c32[x_next];
+            lv_color32_t px_ver = *(const lv_color32_t *)((uint8_t *)src_c32 + y_next * src_stride);
+
+            /*Have the non-premultiplied colors first, mix them as needed,
+             *and premultiply again*/
+            dest_c32[x] = unpremultiply(dest_c32[x]);
+            px_hor = unpremultiply(px_hor);
+            px_ver = unpremultiply(px_ver);
+
+            if(px_ver.alpha == 0) {
+                dest_c32[x].alpha = (dest_c32[x].alpha * (0xFF - ys_fract)) >> 8;
+
+            }
+            else if(!lv_color32_eq(dest_c32[x], px_ver)) {
+                if(dest_c32[x].alpha) dest_c32[x].alpha = ((px_ver.alpha * ys_fract) + (dest_c32[x].alpha * (0xFF - ys_fract))) >> 8;
+                px_ver.alpha = ys_fract;
+                dest_c32[x] = lv_color_mix32(px_ver, dest_c32[x]);
+            }
+
+            if(px_hor.alpha == 0) {
+                dest_c32[x].alpha = (dest_c32[x].alpha * (0xFF - xs_fract)) >> 8;
+            }
+            else if(!lv_color32_eq(dest_c32[x], px_hor)) {
+                if(dest_c32[x].alpha) dest_c32[x].alpha = ((px_hor.alpha * xs_fract) + (dest_c32[x].alpha * (0xFF - xs_fract))) >> 8;
+                px_hor.alpha = xs_fract;
+                dest_c32[x] = lv_color_mix32(px_hor, dest_c32[x]);
+            }
+
+            dest_c32[x].red = (dest_c32[x].red * dest_c32[x].alpha) >> 8;
+            dest_c32[x].green = (dest_c32[x].green * dest_c32[x].alpha) >> 8;
+            dest_c32[x].blue = (dest_c32[x].blue * dest_c32[x].alpha) >> 8;
+
+        }
+        /*Partially out of the image*/
+        else {
+            if((xs_int == 0 && x_next < 0) || (xs_int == src_w - 1 && x_next > 0))  {
+                dest_c32[x] = unpremultiply(dest_c32[x]);
+                lv_opa_t alpha = (dest_c32[x].alpha * (0x7F - xs_fract)) >> 7;
+                dest_c32[x].alpha = alpha;
+                dest_c32[x].red = (dest_c32[x].red * dest_c32[x].alpha) >> 8;
+                dest_c32[x].green = (dest_c32[x].green * dest_c32[x].alpha) >> 8;
+                dest_c32[x].blue = (dest_c32[x].blue * dest_c32[x].alpha) >> 8;
+
+            }
+            else if((ys_int == 0 && y_next < 0) || (ys_int == src_h - 1 && y_next > 0))  {
+                dest_c32[x] = unpremultiply(dest_c32[x]);
+                lv_opa_t alpha = (dest_c32[x].alpha * (0x7F - ys_fract)) >> 7;
+                dest_c32[x].alpha = alpha;
+                dest_c32[x].red = (dest_c32[x].red * dest_c32[x].alpha) >> 8;
+                dest_c32[x].green = (dest_c32[x].green * dest_c32[x].alpha) >> 8;
+                dest_c32[x].blue = (dest_c32[x].blue * dest_c32[x].alpha) >> 8;
+            }
+        }
+    }
+}
+#endif
+
+#if LV_DRAW_SW_SUPPORT_RGB565A8
+
 static void transform_rgb565a8(const uint8_t * src, int32_t src_w, int32_t src_h, int32_t src_stride,
                                int32_t xs_ups, int32_t ys_ups, int32_t xs_step, int32_t ys_step,
                                int32_t x_end, uint16_t * cbuf, uint8_t * abuf, bool src_has_a8, bool aa)
@@ -555,6 +748,126 @@ static void transform_rgb565a8(const uint8_t * src, int32_t src_w, int32_t src_h
     }
 }
 
+#endif
+
+#if LV_DRAW_SW_SUPPORT_RGB565_SWAPPED
+
+static void transform_rgb565a8_swapped(const uint8_t * src, int32_t src_w, int32_t src_h, int32_t src_stride,
+                                       int32_t xs_ups, int32_t ys_ups, int32_t xs_step, int32_t ys_step,
+                                       int32_t x_end, uint16_t * cbuf, uint8_t * abuf, bool src_has_a8, bool aa)
+{
+    int32_t xs_ups_start = xs_ups;
+    int32_t ys_ups_start = ys_ups;
+
+    const lv_opa_t * src_alpha = src + src_stride * src_h;
+
+    /*Must be signed type, because we would use negative array index calculated from stride*/
+    int32_t alpha_stride = src_stride / 2; /*alpha map stride is always half of RGB map stride*/
+
+    int32_t x;
+    for(x = 0; x < x_end; x++) {
+        xs_ups = xs_ups_start + ((xs_step * x) >> 8);
+        ys_ups = ys_ups_start + ((ys_step * x) >> 8);
+
+        int32_t xs_int = xs_ups >> 8;
+        int32_t ys_int = ys_ups >> 8;
+
+        /*Fully out of the image*/
+        if(xs_int < 0 || xs_int >= src_w || ys_int < 0 || ys_int >= src_h) {
+            abuf[x] = 0x00;
+            continue;
+        }
+
+        /*Get the direction the hor and ver neighbor
+         *`fract` will be in range of 0x00..0xFF and `next` (+/-1) indicates the direction*/
+        int32_t xs_fract = xs_ups & 0xFF;
+        int32_t ys_fract = ys_ups & 0xFF;
+
+        int32_t x_next;
+        int32_t y_next;
+        if(xs_fract < 0x80) {
+            x_next = -1;
+            xs_fract = (0x7F - xs_fract) * 2;
+        }
+        else {
+            x_next = 1;
+            xs_fract = (xs_fract - 0x80) * 2;
+        }
+        if(ys_fract < 0x80) {
+            y_next = -1;
+            ys_fract = (0x7F - ys_fract) * 2;
+        }
+        else {
+            y_next = 1;
+            ys_fract = (ys_fract - 0x80) * 2;
+        }
+
+        const uint16_t * src_tmp_u16 = (const uint16_t *)(src + (ys_int * src_stride) + xs_int * 2);
+        cbuf[x] = lv_color_swap_16(src_tmp_u16[0]); /* swap the src pixels */
+
+        if(aa &&
+           xs_int + x_next >= 0 &&
+           xs_int + x_next <= src_w - 1 &&
+           ys_int + y_next >= 0 &&
+           ys_int + y_next <= src_h - 1) {
+
+            /* swap the src pixels */
+            uint16_t px_hor = lv_color_swap_16(src_tmp_u16[x_next]);
+            uint16_t px_ver = lv_color_swap_16(*(const uint16_t *)((uint8_t *)src_tmp_u16 + (y_next * src_stride)));
+
+            if(src_has_a8) {
+                const lv_opa_t * src_alpha_tmp = src_alpha;
+                src_alpha_tmp += (ys_int * alpha_stride) + xs_int;
+                abuf[x] = src_alpha_tmp[0];
+
+                lv_opa_t a_hor = src_alpha_tmp[x_next];
+                lv_opa_t a_ver = src_alpha_tmp[y_next * alpha_stride];
+
+                if(a_ver != abuf[x]) a_ver = ((a_ver * ys_fract) + (abuf[x] * (0x100 - ys_fract))) >> 8;
+                if(a_hor != abuf[x]) a_hor = ((a_hor * xs_fract) + (abuf[x] * (0x100 - xs_fract))) >> 8;
+                abuf[x] = (a_ver + a_hor) >> 1;
+
+                if(abuf[x] == 0x00) continue;
+            }
+            else {
+                abuf[x] = 0xff;
+            }
+
+            if(cbuf[x] != px_ver || cbuf[x] != px_hor) {
+                uint16_t v = lv_color_16_16_mix(px_ver, cbuf[x], ys_fract);
+                uint16_t h = lv_color_16_16_mix(px_hor, cbuf[x], xs_fract);
+                cbuf[x] =  lv_color_16_16_mix(h, v, LV_OPA_50);
+            }
+        }
+        /*Partially out of the image*/
+        else {
+            lv_opa_t a;
+            if(src_has_a8) {
+                const lv_opa_t * src_alpha_tmp = src_alpha;
+                src_alpha_tmp += (ys_int * alpha_stride) + xs_int;
+                a = src_alpha_tmp[0];
+            }
+            else {
+                a = 0xff;
+            }
+
+            if((xs_int == 0 && x_next < 0) || (xs_int == src_w - 1 && x_next > 0))  {
+                abuf[x] = (a * (0xFF - xs_fract)) >> 8;
+            }
+            else if((ys_int == 0 && y_next < 0) || (ys_int == src_h - 1 && y_next > 0))  {
+                abuf[x] = (a * (0xFF - ys_fract)) >> 8;
+            }
+            else {
+                abuf[x] = a;
+            }
+        }
+    }
+}
+
+#endif
+
+#if LV_DRAW_SW_SUPPORT_A8
+
 static void transform_a8(const uint8_t * src, int32_t src_w, int32_t src_h, int32_t src_stride,
                          int32_t xs_ups, int32_t ys_ups, int32_t xs_step, int32_t ys_step,
                          int32_t x_end, uint8_t * abuf, bool aa)
@@ -629,7 +942,10 @@ static void transform_a8(const uint8_t * src, int32_t src_w, int32_t src_h, int3
     }
 }
 
-/* L8 will be transformed into an AL88 buffer, because it will not be recolored */
+#endif
+
+#if LV_DRAW_SW_SUPPORT_L8 && LV_DRAW_SW_SUPPORT_AL88
+
 static void transform_l8_to_al88(const uint8_t * src, int32_t src_w, int32_t src_h, int32_t src_stride,
                                  int32_t xs_ups, int32_t ys_ups, int32_t xs_step, int32_t ys_step,
                                  int32_t x_end, uint8_t * dest_buf, bool aa)
@@ -706,81 +1022,7 @@ static void transform_l8_to_al88(const uint8_t * src, int32_t src_w, int32_t src
     }
 }
 
-/* L8 has to be transformed into an ARGB8888 buffer, because it will be recolored as well */
-static void transform_l8_to_argb8888(const uint8_t * src, int32_t src_w, int32_t src_h, int32_t src_stride,
-                                     int32_t xs_ups, int32_t ys_ups, int32_t xs_step, int32_t ys_step,
-                                     int32_t x_end, uint8_t * dest_buf, bool aa)
-{
-    int32_t xs_ups_start = xs_ups;
-    int32_t ys_ups_start = ys_ups;
-    lv_color32_t * dest_c32 = (lv_color32_t *)dest_buf;
-
-    int32_t x;
-    for(x = 0; x < x_end; x++) {
-        xs_ups = xs_ups_start + ((xs_step * x) >> 8);
-        ys_ups = ys_ups_start + ((ys_step * x) >> 8);
-
-        int32_t xs_int = xs_ups >> 8;
-        int32_t ys_int = ys_ups >> 8;
-
-        /*Fully out of the image*/
-        if(xs_int < 0 || xs_int >= src_w || ys_int < 0 || ys_int >= src_h) {
-            *((uint32_t *)&dest_c32[x]) = 0L;
-            continue;
-        }
-
-        /*Get the direction the hor and ver neighbor
-         *`fract` will be in range of 0x00..0xFF and `next` (+/-1) indicates the direction*/
-        int32_t xs_fract = xs_ups & 0xFF;
-        int32_t ys_fract = ys_ups & 0xFF;
-
-        int32_t x_next;
-        int32_t y_next;
-        if(xs_fract < 0x80) {
-            x_next = -1;
-            xs_fract = (0x7F - xs_fract) * 2;
-        }
-        else {
-            x_next = 1;
-            xs_fract = (xs_fract - 0x80) * 2;
-        }
-        if(ys_fract < 0x80) {
-            y_next = -1;
-            ys_fract = (0x7F - ys_fract) * 2;
-        }
-        else {
-            y_next = 1;
-            ys_fract = (ys_fract - 0x80) * 2;
-        }
-
-        const uint8_t * src_tmp = src;
-        src_tmp += ys_int * src_stride + xs_int;
-        dest_c32[x].red = dest_c32[x].green = dest_c32[x].blue = src_tmp[0];
-        dest_c32[x].alpha = 255;
-        if(aa &&
-           xs_int + x_next >= 0 &&
-           xs_int + x_next <= src_w - 1 &&
-           ys_int + y_next >= 0 &&
-           ys_int + y_next <= src_h - 1) {
-
-            lv_opa_t a_ver = src_tmp[x_next];
-            lv_opa_t a_hor = src_tmp[y_next * src_stride];
-
-            if(a_ver != src_tmp[0]) a_ver = ((a_ver * ys_fract) + (src_tmp[0] * (0x100 - ys_fract))) >> 8;
-            if(a_hor != src_tmp[0]) a_hor = ((a_hor * xs_fract) + (src_tmp[0] * (0x100 - xs_fract))) >> 8;
-            dest_c32[x].red = dest_c32[x].green = dest_c32[x].blue = (a_ver + a_hor) >> 1;
-        }
-        else {
-            /*Partially out of the image*/
-            if((xs_int == 0 && x_next < 0) || (xs_int == src_w - 1 && x_next > 0)) {
-                dest_c32[x].alpha = (src_tmp[0] * (0xFF - xs_fract)) >> 8;
-            }
-            else if((ys_int == 0 && y_next < 0) || (ys_int == src_h - 1 && y_next > 0)) {
-                dest_c32[x].alpha = (src_tmp[0] * (0xFF - ys_fract)) >> 8;
-            }
-        }
-    }
-}
+#endif /*LV_DRAW_SW_SUPPORT_L8 && LV_DRAW_SW_SUPPORT_AL88*/
 
 static void transform_point_upscaled(point_transform_dsc_t * t, int32_t xin, int32_t yin, int32_t * xout,
                                      int32_t * yout)
