@@ -11,6 +11,7 @@
 
 #if LV_USE_DRAW_VG_LITE
 
+#include "../lv_draw_buf_private.h"
 #include "lv_vg_lite_utils.h"
 
 /*********************
@@ -25,7 +26,7 @@
  *  STATIC PROTOTYPES
  **********************/
 
-static uint32_t width_to_stride(uint32_t w, lv_color_format_t color_format);
+static void init_handlers(lv_draw_buf_handlers_t * handlers);
 
 /**********************
  *  STATIC VARIABLES
@@ -41,8 +42,9 @@ static uint32_t width_to_stride(uint32_t w, lv_color_format_t color_format);
 
 void lv_draw_buf_vg_lite_init_handlers(void)
 {
-    lv_draw_buf_handlers_t * handlers = lv_draw_buf_get_handlers();
-    handlers->width_to_stride_cb = width_to_stride;
+    init_handlers(lv_draw_buf_get_handlers());
+    init_handlers(lv_draw_buf_get_font_handlers());
+    init_handlers(lv_draw_buf_get_image_handlers());
 }
 
 /**********************
@@ -54,4 +56,10 @@ static uint32_t width_to_stride(uint32_t w, lv_color_format_t color_format)
     return lv_vg_lite_width_to_stride(w, lv_vg_lite_vg_fmt(color_format));
 }
 
+static void init_handlers(lv_draw_buf_handlers_t * handlers)
+{
+    LV_ASSERT_NULL(handlers);
+    handlers->width_to_stride_cb = width_to_stride;
+}
+
 #endif /*LV_USE_DRAW_VG_LITE*/
