@@ -18,7 +18,7 @@ extern "C" {
 
 #if LV_USE_DRAW_VG_LITE
 
-#include "../lv_draw.h"
+#include "../lv_draw_private.h"
 #include "../../misc/lv_array.h"
 
 #if LV_USE_VG_LITE_THORVG
@@ -36,20 +36,23 @@ extern "C" {
  **********************/
 
 struct _lv_vg_lite_pending_t;
+struct _lv_vg_lite_grad_ctx_t;
 
 struct _lv_draw_vg_lite_unit_t {
     lv_draw_unit_t base_unit;
     lv_draw_task_t * task_act;
+    lv_area_t current_scissor_area;
 
     struct _lv_vg_lite_pending_t * image_dsc_pending;
 
-    lv_cache_t * linear_grad_cache;
-    struct _lv_vg_lite_pending_t * linear_grad_pending;
+    struct _lv_vg_lite_grad_ctx_t * grad_ctx;
 
-    lv_cache_t * radial_grad_cache;
-    struct _lv_vg_lite_pending_t * radial_grad_pending;
+    lv_cache_t * stroke_cache;
+
+    struct _lv_vg_lite_pending_t * bitmap_font_pending;
 
     uint16_t flush_count;
+    uint16_t letter_count;
     vg_lite_buffer_t target_buffer;
     vg_lite_matrix_t global_matrix;
     struct _lv_vg_lite_path_t * global_path;
@@ -70,4 +73,4 @@ struct _lv_draw_vg_lite_unit_t {
 } /*extern "C"*/
 #endif
 
-#endif /*LV_VG_LITE_DRAW_H*/
+#endif /*LV_DRAW_VG_LITE_TYPE_H*/
