@@ -18,6 +18,7 @@ extern "C" {
 
 #if defined(LV_LIBINPUT_XKB) && LV_LIBINPUT_XKB
 
+#include "../../misc/lv_types.h"
 #include <stdbool.h>
 #include <xkbcommon/xkbcommon.h>
 
@@ -25,15 +26,6 @@ extern "C" {
  *      DEFINES
  *********************/
 
-/**********************
- *      TYPEDEFS
- **********************/
-
-typedef struct {
-    struct xkb_keymap * keymap;
-    struct xkb_state * state;
-} lv_xkb_t;
-
 /**********************
  * GLOBAL PROTOTYPES
  **********************/
@@ -63,10 +55,10 @@ uint32_t lv_xkb_process_key(lv_xkb_t * dsc, uint32_t scancode, bool down);
  *      MACROS
  **********************/
 
-#endif /* LV_LIBINPUT_XKB */
+#endif /* defined(LV_LIBINPUT_XKB) && LV_LIBINPUT_XKB */
 
 #ifdef __cplusplus
 } /* extern "C" */
 #endif
 
-#endif /* defined(LV_LIBINPUT_XKB) && LV_LIBINPUT_XKB */
+#endif /* LV_XKB_H */
