@@ -18,10 +18,11 @@
 #include "lv_windows_display.h"
 #include "lv_windows_input_private.h"
 #include "../../misc/lv_text_private.h"
+#include "../../core/lv_obj_private.h"
 
 #include <windowsx.h>
 
-#include "../../widgets/textarea/lv_textarea.h"
+#include "../../widgets/textarea/lv_textarea_private.h"
 #include "../../widgets/keyboard/lv_keyboard.h"
 
 /*********************
@@ -128,8 +129,7 @@ lv_indev_t * lv_windows_acquire_keypad_indev(lv_display_t * display)
     }
 
     if(!context->keypad.indev) {
-        InitializeCriticalSection(&context->keypad.mutex);
-        _lv_ll_init(
+        lv_ll_init(
             &context->keypad.queue,
             sizeof(lv_windows_keypad_queue_item_t));
         context->keypad.utf16_high_surrogate = 0;
@@ -421,21 +421,17 @@ static void lv_windows_keypad_driver_read_callback(
         return;
     }
 
-    EnterCriticalSection(&context->keypad.mutex);
-
     lv_windows_keypad_queue_item_t * current = (lv_windows_keypad_queue_item_t *)(
-                                                   _lv_ll_get_head(&context->keypad.queue));
+                                                   lv_ll_get_head(&context->keypad.queue));
     if(current) {
         data->key = current->key;
         data->state = current->state;
 
-        _lv_ll_remove(&context->keypad.queue, current);
+        lv_ll_remove(&context->keypad.queue, current);
         lv_free(current);
 
         data->continue_reading = true;
     }
-
-    LeaveCriticalSection(&context->keypad.mutex);
 }
 
 static void lv_windows_release_keypad_device_event_callback(lv_event_t * e)
@@ -456,8 +452,7 @@ static void lv_windows_release_keypad_device_event_callback(lv_event_t * e)
         return;
     }
 
-    DeleteCriticalSection(&context->keypad.mutex);
-    _lv_ll_clear(&context->keypad.queue);
+    lv_ll_clear(&context->keypad.queue);
     context->keypad.utf16_high_surrogate = 0;
     context->keypad.utf16_low_surrogate = 0;
 
@@ -470,7 +465,7 @@ static void lv_windows_push_key_to_keyboard_queue(
     lv_indev_state_t state)
 {
     lv_windows_keypad_queue_item_t * current = (lv_windows_keypad_queue_item_t *)(
-                                                   _lv_ll_ins_tail(&context->keypad.queue));
+                                                   lv_ll_ins_tail(&context->keypad.queue));
     if(current) {
         current->key = key;
         current->state = state;
@@ -571,8 +566,6 @@ bool lv_windows_keypad_device_window_message_handler(
                 lv_windows_window_context_t * context = (lv_windows_window_context_t *)(
                                                             lv_windows_get_window_context(hWnd));
                 if(context) {
-                    EnterCriticalSection(&context->keypad.mutex);
-
                     bool skip_translation = false;
                     uint32_t translated_key = 0;
 
@@ -627,8 +620,6 @@ bool lv_windows_keypad_device_window_message_handler(
                              ? LV_INDEV_STATE_RELEASED
                              : LV_INDEV_STATE_PRESSED));
                     }
-
-                    LeaveCriticalSection(&context->keypad.mutex);
                 }
 
                 break;
@@ -637,8 +628,6 @@ bool lv_windows_keypad_device_window_message_handler(
                 lv_windows_window_context_t * context = (lv_windows_window_context_t *)(
                                                             lv_windows_get_window_context(hWnd));
                 if(context) {
-                    EnterCriticalSection(&context->keypad.mutex);
-
                     uint16_t raw_code_point = (uint16_t)(wParam);
 
                     if(raw_code_point >= 0x20 && raw_code_point != 0x7F) {
@@ -677,8 +666,6 @@ bool lv_windows_keypad_device_window_message_handler(
                             lvgl_code_point,
                             LV_INDEV_STATE_RELEASED);
                     }
-
-                    LeaveCriticalSection(&context->keypad.mutex);
                 }
 
                 break;
