@@ -1,10 +1,10 @@
 /**
- * @file lv_layouts.h
+ * @file lv_layout.h
  *
  */
 
-#ifndef LV_LAYOUTS_H
-#define LV_LAYOUTS_H
+#ifndef LV_LAYOUT_H
+#define LV_LAYOUT_H
 
 #ifdef __cplusplus
 extern "C" {
@@ -25,10 +25,6 @@ extern "C" {
  **********************/
 
 typedef void (*lv_layout_update_cb_t)(lv_obj_t *, void * user_data);
-typedef struct {
-    lv_layout_update_cb_t cb;
-    void * user_data;
-} lv_layout_dsc_t;
 
 typedef enum {
     LV_LAYOUT_NONE = 0,
@@ -41,17 +37,9 @@ typedef enum {
     LV_LAYOUT_GRID,
 #endif
 
-    _LV_LAYOUT_LAST
+    LV_LAYOUT_LAST
 } lv_layout_t;
 
-/**********************
- * GLOBAL PROTOTYPES
- **********************/
-
-void _lv_layout_init(void);
-
-void _lv_layout_deinit(void);
-
 /**
  * Register a new layout
  * @param cb        the layout update callback
@@ -60,12 +48,6 @@ void _lv_layout_deinit(void);
  */
 uint32_t lv_layout_register(lv_layout_update_cb_t cb, void * user_data);
 
-/**
- * Update the layout of a widget
- * @param obj   pointer to a widget
- */
-void _lv_layout_apply(lv_obj_t * obj);
-
 /**********************
  *      MACROS
  **********************/
@@ -82,4 +64,4 @@ void _lv_layout_apply(lv_obj_t * obj);
 } /*extern "C"*/
 #endif
 
-#endif /*LV_LAYOUTS_H*/
+#endif /*LV_LAYOUT_H*/
