@@ -1,11 +1,12 @@
 /**
- * @file lv_image_decoder.c
+ * @file lv_bin_decoder.c
  *
  */
 
 /*********************
  *      INCLUDES
  *********************/
+#include "../../draw/lv_image_decoder_private.h"
 #include "lv_bin_decoder.h"
 #include "../../draw/lv_draw_image.h"
 #include "../../draw/lv_draw_buf.h"
@@ -110,11 +111,13 @@ void lv_bin_decoder_init(void)
     decoder->name = DECODER_NAME;
 }
 
-lv_result_t lv_bin_decoder_info(lv_image_decoder_t * decoder, const void * src, lv_image_header_t * header)
+lv_result_t lv_bin_decoder_info(lv_image_decoder_t * decoder, lv_image_decoder_dsc_t * dsc, lv_image_header_t * header)
 {
     LV_UNUSED(decoder); /*Unused*/
 
-    lv_image_src_t src_type = lv_image_src_get_type(src);
+    const void * src = dsc->src;
+    lv_image_src_t src_type = dsc->src_type;
+
     if(src_type == LV_IMAGE_SRC_VARIABLE) {
         lv_image_dsc_t * image = (lv_image_dsc_t *)src;
         lv_memcpy(header, &image->header, sizeof(lv_image_header_t));
@@ -123,31 +126,28 @@ lv_result_t lv_bin_decoder_info(lv_image_decoder_t * decoder, const void * src,
         /*Support only "*.bin" files*/
         if(lv_strcmp(lv_fs_get_ext(src), "bin")) return LV_RESULT_INVALID;
 
-        lv_fs_file_t f;
-        lv_fs_res_t res = lv_fs_open(&f, src, LV_FS_MODE_RD);
-        if(res == LV_FS_RES_OK) {
-            uint32_t rn;
-            res = lv_fs_read(&f, header, sizeof(lv_image_header_t), &rn);
-            lv_fs_close(&f);
-            if(res != LV_FS_RES_OK || rn != sizeof(lv_image_header_t)) {
-                LV_LOG_WARN("Read file header failed: %d", res);
-                return LV_RESULT_INVALID;
-            }
+        lv_fs_res_t res;
+        uint32_t rn;
+        res = lv_fs_read(&dsc->file, header, sizeof(lv_image_header_t), &rn);
 
-            /**
-             * @todo
-             * This is a temp backward compatibility solution after adding
-             * magic in image header.
-             */
-            if(header->magic != LV_IMAGE_HEADER_MAGIC) {
-                LV_LOG_WARN("Legacy bin image detected: %s", (char *)src);
-                header->cf = header->magic;
-                header->magic = LV_IMAGE_HEADER_MAGIC;
-            }
-
-            /*File is always read to buf, thus data can be modified.*/
-            header->flags |= LV_IMAGE_FLAGS_MODIFIABLE;
+        if(res != LV_FS_RES_OK || rn != sizeof(lv_image_header_t)) {
+            LV_LOG_WARN("Read file header failed: %d with len: %" LV_PRIu32 ", expected: %zu", res, rn, sizeof(lv_image_header_t));
+            return LV_RESULT_INVALID;
         }
+
+        /**
+         * @todo
+         * This is a temp backward compatibility solution after adding
+         * magic in image header.
+         */
+        if(header->magic != LV_IMAGE_HEADER_MAGIC) {
+            LV_LOG_WARN("Legacy bin image detected: %s", (char *)src);
+            header->cf = header->magic;
+            header->magic = LV_IMAGE_HEADER_MAGIC;
+        }
+
+        /*File is always read to buf, thus data can be modified.*/
+        header->flags |= LV_IMAGE_FLAGS_MODIFIABLE;
     }
     else if(src_type == LV_IMAGE_SRC_SYMBOL) {
         /*The size depend on the font but it is unknown here. It should be handled outside of the
@@ -163,6 +163,11 @@ lv_result_t lv_bin_decoder_info(lv_image_decoder_t * decoder, const void * src,
         return LV_RESULT_INVALID;
     }
 
+    if(header->cf == LV_COLOR_FORMAT_UNKNOWN) {
+        LV_LOG_WARN("Image color format is unknown");
+        return LV_RESULT_INVALID;
+    }
+
     /*For backward compatibility, all images are not premultiplied for now.*/
     if(header->magic != LV_IMAGE_HEADER_MAGIC) {
         header->flags &= ~LV_IMAGE_FLAGS_PREMULTIPLIED;
@@ -181,7 +186,8 @@ lv_result_t lv_bin_decoder_open(lv_image_decoder_t * decoder, lv_image_decoder_d
 {
     LV_UNUSED(decoder);
 
-    lv_fs_res_t res = LV_RESULT_INVALID;
+    lv_result_t res = LV_RESULT_INVALID;
+    lv_fs_res_t fs_res = LV_FS_RES_UNKNOWN;
     bool use_directly = false; /*If the image is already decoded and can be used directly*/
 
     /*Open the file if it's a file*/
@@ -202,9 +208,9 @@ lv_result_t lv_bin_decoder_open(lv_image_decoder_t * decoder, lv_image_decoder_d
             return LV_RESULT_INVALID;
         }
 
-        res = lv_fs_open(f, dsc->src, LV_FS_MODE_RD);
-        if(res != LV_FS_RES_OK) {
-            LV_LOG_WARN("Open file failed: %d", res);
+        fs_res = lv_fs_open(f, dsc->src, LV_FS_MODE_RD);
+        if(fs_res != LV_FS_RES_OK) {
+            LV_LOG_WARN("Open file failed: %d", fs_res);
             lv_free(f);
             free_decoder_data(dsc);
             return LV_RESULT_INVALID;
@@ -234,6 +240,7 @@ lv_result_t lv_bin_decoder_open(lv_image_decoder_t * decoder, lv_image_decoder_d
                 || cf == LV_COLOR_FORMAT_XRGB8888   \
                 || cf == LV_COLOR_FORMAT_RGB888     \
                 || cf == LV_COLOR_FORMAT_RGB565     \
+                || cf == LV_COLOR_FORMAT_RGB565_SWAPPED     \
                 || cf == LV_COLOR_FORMAT_RGB565A8   \
                 || cf == LV_COLOR_FORMAT_ARGB8565) {
             res = decode_rgb(decoder, dsc);
@@ -273,7 +280,7 @@ lv_result_t lv_bin_decoder_open(lv_image_decoder_t * decoder, lv_image_decoder_d
                 res = decode_indexed(decoder, dsc);
             }
         }
-        else if(LV_COLOR_FORMAT_IS_ALPHA_ONLY(cf)) {
+        else if(LV_COLOR_FORMAT_IS_ALPHA_ONLY(cf) && cf != LV_COLOR_FORMAT_A8) {
             /*Alpha only image will need decoder data to store pointer to decoded image, to free it when decoder closes*/
             decoder_data_t * decoder_data = get_decoder_data(dsc);
             if(decoder_data == NULL) {
@@ -290,21 +297,30 @@ lv_result_t lv_bin_decoder_open(lv_image_decoder_t * decoder, lv_image_decoder_d
             lv_draw_buf_t * decoded;
             if(image->header.flags & LV_IMAGE_FLAGS_ALLOCATED) {
                 decoded = (lv_draw_buf_t *)image;
+                res = LV_RESULT_OK;
             }
             else {
                 decoded = &decoder_data->c_array;
-                lv_draw_buf_from_image(decoded, image);
+                if(image->header.stride == 0) {
+                    /*If image doesn't have stride, treat it as lvgl v8 legacy image format*/
+                    lv_image_dsc_t tmp = *image;
+                    tmp.header.stride = (tmp.header.w * lv_color_format_get_bpp(cf) + 7) >> 3;
+                    res = lv_draw_buf_from_image(decoded, &tmp);
+                }
+                else
+                    res = lv_draw_buf_from_image(decoded, image);
             }
 
-            dsc->decoded = decoded;
+            if(res == LV_RESULT_OK) {
+                dsc->decoded = decoded;
 
-            if(decoded->header.stride == 0) {
-                /*Use the auto calculated value from decoder_info callback*/
-                decoded->header.stride = dsc->header.stride;
+                if(decoded->header.stride == 0) {
+                    /*Use the auto calculated value from decoder_info callback*/
+                    decoded->header.stride = dsc->header.stride;
+                }
+
+                use_directly = true; /*A variable image that can be used directly.*/
             }
-
-            res = LV_RESULT_OK;
-            use_directly = true; /*A variable image that can be used directly.*/
         }
     }
 
@@ -316,6 +332,10 @@ lv_result_t lv_bin_decoder_open(lv_image_decoder_t * decoder, lv_image_decoder_d
     if(dsc->decoded == NULL) return LV_RESULT_OK; /*Need to read via get_area_cb*/
 
     lv_draw_buf_t * decoded = (lv_draw_buf_t *)dsc->decoded;
+    if(dsc->header.flags & LV_IMAGE_FLAGS_PREMULTIPLIED) {
+        lv_draw_buf_set_flag(decoded, LV_IMAGE_FLAGS_PREMULTIPLIED);
+    }
+
     lv_draw_buf_t * adjusted = lv_image_decoder_post_process(dsc, decoded);
     if(adjusted == NULL) {
         free_decoder_data(dsc);
@@ -331,7 +351,18 @@ lv_result_t lv_bin_decoder_open(lv_image_decoder_t * decoder, lv_image_decoder_d
     }
     dsc->decoded = adjusted;
 
-    if(use_directly || dsc->args.no_cache) return LV_RESULT_OK; /*Do not put image to cache if it can be used directly.*/
+    /* Copy user flags to the decoded image */
+    if(dsc->header.flags & LV_IMAGE_FLAGS_USER_MASK) {
+        lv_draw_buf_set_flag((lv_draw_buf_t *)dsc->decoded, dsc->header.flags & LV_IMAGE_FLAGS_USER_MASK);
+    }
+
+    /*Do not put image to cache if it can be used directly.*/
+    if(use_directly || dsc->args.no_cache) {
+        if(dsc->args.flush_cache && use_directly) {
+            dsc->args.flush_cache = false;
+        }
+        return LV_RESULT_OK;
+    }
 
     /*If the image cache is disabled, just return the decoded image*/
     if(!lv_image_cache_is_enabled()) return LV_RESULT_OK;
@@ -380,6 +411,7 @@ lv_result_t lv_bin_decoder_get_area(lv_image_decoder_t * decoder, lv_image_decod
                      || cf == LV_COLOR_FORMAT_XRGB8888  \
                      || cf == LV_COLOR_FORMAT_RGB888    \
                      || cf == LV_COLOR_FORMAT_RGB565    \
+                     || cf == LV_COLOR_FORMAT_RGB565_SWAPPED    \
                      || cf == LV_COLOR_FORMAT_ARGB8565  \
                      || cf == LV_COLOR_FORMAT_RGB565A8;
     if(!supported) {
@@ -387,7 +419,7 @@ lv_result_t lv_bin_decoder_get_area(lv_image_decoder_t * decoder, lv_image_decod
         return LV_RESULT_INVALID;
     }
 
-    lv_result_t res = LV_RESULT_INVALID;
+    lv_fs_res_t res = LV_FS_RES_UNKNOWN;
     decoder_data_t * decoder_data = dsc->user_data;
     if(decoder_data == NULL) {
         LV_LOG_ERROR("Unexpected null decoder data");
@@ -412,7 +444,7 @@ lv_result_t lv_bin_decoder_get_area(lv_image_decoder_t * decoder, lv_image_decod
                 lv_draw_buf_destroy(decoder_data->decoded_partial);
                 decoder_data->decoded_partial = NULL;
             }
-            decoded = lv_draw_buf_create_user(image_cache_draw_buf_handlers, w_px, 1, cf_decoded, LV_STRIDE_AUTO);
+            decoded = lv_draw_buf_create_ex(image_cache_draw_buf_handlers, w_px, 1, cf_decoded, LV_STRIDE_AUTO);
             if(decoded == NULL) return LV_RESULT_INVALID;
             decoder_data->decoded_partial = decoded; /*Free on decoder close*/
         }
@@ -422,7 +454,7 @@ lv_result_t lv_bin_decoder_get_area(lv_image_decoder_t * decoder, lv_image_decod
     else {
         decoded_area->y1++;
         decoded_area->y2++;
-        decoded = decoder_data->decoded_partial; /*Already alloced*/
+        decoded = decoder_data->decoded_partial; /*Already allocated*/
     }
 
     img_data = decoded->data; /*Get the buffer to operate on*/
@@ -465,7 +497,7 @@ lv_result_t lv_bin_decoder_get_area(lv_image_decoder_t * decoder, lv_image_decod
     }
 
     if(cf == LV_COLOR_FORMAT_ARGB8888 || cf == LV_COLOR_FORMAT_XRGB8888 || cf == LV_COLOR_FORMAT_RGB888
-       || cf == LV_COLOR_FORMAT_RGB565 || cf == LV_COLOR_FORMAT_ARGB8565) {
+       || cf == LV_COLOR_FORMAT_RGB565 || cf == LV_COLOR_FORMAT_RGB565_SWAPPED || cf == LV_COLOR_FORMAT_ARGB8565) {
         uint32_t len = (w_px * bpp) / 8;
         offset += decoded_area->y1 * dsc->header.stride;
         offset += decoded_area->x1 * bpp / 8; /*Move to x1*/
@@ -546,7 +578,7 @@ static void free_decoder_data(lv_image_decoder_dsc_t * dsc)
 static lv_result_t decode_indexed(lv_image_decoder_t * decoder, lv_image_decoder_dsc_t * dsc)
 {
     LV_UNUSED(decoder); /*Unused*/
-    lv_result_t res;
+    lv_fs_res_t res;
     uint32_t rn;
     decoder_data_t * decoder_data = dsc->user_data;
     lv_fs_file_t * f = decoder_data->f;
@@ -574,7 +606,7 @@ static lv_result_t decode_indexed(lv_image_decoder_t * decoder, lv_image_decoder
 
         res = fs_read_file_at(f, sizeof(lv_image_header_t), (uint8_t *)palette, palette_len, &rn);
         if(res != LV_FS_RES_OK || rn != palette_len) {
-            LV_LOG_WARN("Read palette failed: %d", res);
+            LV_LOG_WARN("Read palette failed: %d, with len: %" LV_PRIu32 ", expected: %" LV_PRIu32, res, rn, palette_len);
             lv_free((void *)palette);
             return LV_RESULT_INVALID;
         }
@@ -582,8 +614,8 @@ static lv_result_t decode_indexed(lv_image_decoder_t * decoder, lv_image_decoder
         decoder_data->palette = (void *)palette; /*Need to free when decoder closes*/
 
 #if LV_BIN_DECODER_RAM_LOAD
-        draw_buf_indexed = lv_draw_buf_create_user(image_cache_draw_buf_handlers, dsc->header.w, dsc->header.h, cf,
-                                                   dsc->header.stride);
+        draw_buf_indexed = lv_draw_buf_create_ex(image_cache_draw_buf_handlers, dsc->header.w, dsc->header.h, cf,
+                                                 dsc->header.stride);
         if(draw_buf_indexed == NULL) {
             LV_LOG_ERROR("Draw buffer alloc failed");
             goto exit_with_buf;
@@ -602,7 +634,7 @@ static lv_result_t decode_indexed(lv_image_decoder_t * decoder, lv_image_decoder
         data_len -= data_offset;
         res = fs_read_file_at(f, data_offset, (uint8_t *)indexed_data, data_len, &rn);
         if(res != LV_FS_RES_OK || rn != data_len) {
-            LV_LOG_WARN("Read indexed image failed: %d", res);
+            LV_LOG_WARN("Read indexed image failed: %d, with len: %" LV_PRIu32 ", expected: %" LV_PRIu32, res, rn, data_len);
             goto exit_with_buf;
         }
 #endif
@@ -621,9 +653,9 @@ static lv_result_t decode_indexed(lv_image_decoder_t * decoder, lv_image_decoder
 
 #if LV_BIN_DECODER_RAM_LOAD
     /*Convert to ARGB8888, since sw renderer cannot render it directly even it's in RAM*/
-    lv_draw_buf_t * decoded = lv_draw_buf_create_user(image_cache_draw_buf_handlers, dsc->header.w, dsc->header.h,
-                                                      LV_COLOR_FORMAT_ARGB8888,
-                                                      0);
+    lv_draw_buf_t * decoded = lv_draw_buf_create_ex(image_cache_draw_buf_handlers, dsc->header.w, dsc->header.h,
+                                                    LV_COLOR_FORMAT_ARGB8888,
+                                                    0);
     if(decoded == NULL) {
         LV_LOG_ERROR("No memory for indexed image");
         goto exit_with_buf;
@@ -676,7 +708,7 @@ static lv_result_t load_indexed(lv_image_decoder_t * decoder, lv_image_decoder_d
 
     LV_UNUSED(decoder); /*Unused*/
 
-    lv_result_t res;
+    lv_fs_res_t res;
     uint32_t rn;
     decoder_data_t * decoder_data = dsc->user_data;
 
@@ -698,7 +730,10 @@ static lv_result_t load_indexed(lv_image_decoder_t * decoder, lv_image_decoder_d
         }
         else {
             decoded = &decoder_data->c_array;
-            lv_draw_buf_from_image(decoded, image);
+            lv_result_t result = lv_draw_buf_from_image(decoded, image);
+            if(result != LV_RESULT_OK) {
+                return result;
+            }
         }
 
         dsc->decoded = decoded;
@@ -714,8 +749,8 @@ static lv_result_t load_indexed(lv_image_decoder_t * decoder, lv_image_decoder_d
     if(dsc->src_type == LV_IMAGE_SRC_FILE) {
         lv_color_format_t cf = dsc->header.cf;
         lv_fs_file_t * f = decoder_data->f;
-        lv_draw_buf_t * decoded = lv_draw_buf_create_user(image_cache_draw_buf_handlers, dsc->header.w, dsc->header.h, cf,
-                                                          dsc->header.stride);
+        lv_draw_buf_t * decoded = lv_draw_buf_create_ex(image_cache_draw_buf_handlers, dsc->header.w, dsc->header.h, cf,
+                                                        dsc->header.stride);
         if(decoded == NULL) {
             LV_LOG_ERROR("Draw buffer alloc failed");
             return LV_RESULT_INVALID;
@@ -725,7 +760,7 @@ static lv_result_t load_indexed(lv_image_decoder_t * decoder, lv_image_decoder_d
         uint32_t palette_len = sizeof(lv_color32_t) * LV_COLOR_INDEXED_PALETTE_SIZE(cf);
         res = fs_read_file_at(f, sizeof(lv_image_header_t), data, palette_len, &rn);
         if(res != LV_FS_RES_OK || rn != palette_len) {
-            LV_LOG_WARN("Read palette failed: %d", res);
+            LV_LOG_WARN("Read palette failed: %d, with len: %" LV_PRIu32 ", expected: %" LV_PRIu32, res, rn, palette_len);
             lv_draw_buf_destroy(decoded);
             return LV_RESULT_INVALID;
         }
@@ -743,7 +778,7 @@ static lv_result_t load_indexed(lv_image_decoder_t * decoder, lv_image_decoder_d
         data += palette_len;
         res = fs_read_file_at(f, data_offset, data, data_len, &rn);
         if(res != LV_FS_RES_OK || rn != data_len) {
-            LV_LOG_WARN("Read indexed image failed: %d", res);
+            LV_LOG_WARN("Read indexed image failed: %d, with len: %" LV_PRIu32 ", expected: %" LV_PRIu32, res, rn, data_len);
             lv_draw_buf_destroy(decoded);
             return LV_RESULT_INVALID;
         }
@@ -762,7 +797,7 @@ static lv_result_t load_indexed(lv_image_decoder_t * decoder, lv_image_decoder_d
 static lv_result_t decode_rgb(lv_image_decoder_t * decoder, lv_image_decoder_dsc_t * dsc)
 {
     LV_UNUSED(decoder);
-    lv_result_t res;
+    lv_fs_res_t res;
     decoder_data_t * decoder_data = dsc->user_data;
     lv_fs_file_t * f = decoder_data->f;
     lv_color_format_t cf = dsc->header.cf;
@@ -772,8 +807,8 @@ static lv_result_t decode_rgb(lv_image_decoder_t * decoder, lv_image_decoder_dsc
         len += (dsc->header.stride / 2) * dsc->header.h; /*A8 mask*/
     }
 
-    lv_draw_buf_t * decoded = lv_draw_buf_create_user(image_cache_draw_buf_handlers, dsc->header.w, dsc->header.h, cf,
-                                                      dsc->header.stride);
+    lv_draw_buf_t * decoded = lv_draw_buf_create_ex(image_cache_draw_buf_handlers, dsc->header.w, dsc->header.h, cf,
+                                                    dsc->header.stride);
     if(decoded == NULL) {
         LV_LOG_ERROR("No memory for rgb file read");
         return LV_RESULT_INVALID;
@@ -784,7 +819,7 @@ static lv_result_t decode_rgb(lv_image_decoder_t * decoder, lv_image_decoder_dsc
     uint32_t rn;
     res = fs_read_file_at(f, sizeof(lv_image_header_t), img_data, len, &rn);
     if(res != LV_FS_RES_OK || rn != len) {
-        LV_LOG_WARN("Read rgb file failed: %d", res);
+        LV_LOG_WARN("Read rgb file failed: %d, with len: %" LV_PRIu32 ", expected: %" LV_PRIu32, res, rn, len);
         lv_draw_buf_destroy(decoded);
         return LV_RESULT_INVALID;
     }
@@ -815,18 +850,24 @@ static inline uint8_t bit_extend(uint8_t value, uint8_t bpp)
 static lv_result_t decode_alpha_only(lv_image_decoder_t * decoder, lv_image_decoder_dsc_t * dsc)
 {
     LV_UNUSED(decoder);
-    lv_result_t res;
+    lv_fs_res_t res;
     uint32_t rn;
     decoder_data_t * decoder_data = dsc->user_data;
     uint8_t bpp = lv_color_format_get_bpp(dsc->header.cf);
+
+    if(bpp == 0) {
+        LV_LOG_ERROR("Error color format: %d", dsc->header.cf);
+        return LV_RESULT_INVALID;
+    }
+
     uint32_t w = (dsc->header.stride * 8) / bpp;
     uint32_t buf_stride = (w * 8 + 7) >> 3; /*stride for img_data*/
     uint32_t buf_len = w * dsc->header.h; /*always decode to A8 format*/
     lv_draw_buf_t * decoded;
     uint32_t file_len = (uint32_t)dsc->header.stride * dsc->header.h;
 
-    decoded = lv_draw_buf_create_user(image_cache_draw_buf_handlers, dsc->header.w, dsc->header.h, LV_COLOR_FORMAT_A8,
-                                      buf_stride);
+    decoded = lv_draw_buf_create_ex(image_cache_draw_buf_handlers, dsc->header.w, dsc->header.h, LV_COLOR_FORMAT_A8,
+                                    buf_stride);
     if(decoded == NULL) {
         LV_LOG_ERROR("Out of memory");
         return LV_RESULT_INVALID;
@@ -841,7 +882,7 @@ static lv_result_t decode_alpha_only(lv_image_decoder_t * decoder, lv_image_deco
     else if(dsc->src_type == LV_IMAGE_SRC_FILE) {
         res = fs_read_file_at(decoder_data->f, sizeof(lv_image_header_t), img_data, file_len, &rn);
         if(res != LV_FS_RES_OK || rn != file_len) {
-            LV_LOG_WARN("Read header failed: %d", res);
+            LV_LOG_WARN("Read header failed: %d, with len: %" LV_PRIu32 ", expected: %" LV_PRIu32, res, rn, file_len);
             lv_draw_buf_destroy(decoded);
             return LV_RESULT_INVALID;
         }
@@ -891,6 +932,7 @@ static lv_result_t decode_compressed(lv_image_decoder_t * decoder, lv_image_deco
     uint32_t compressed_len;
     decoder_data_t * decoder_data = get_decoder_data(dsc);
     lv_result_t res;
+    lv_fs_res_t fs_res;
     uint8_t * file_buf = NULL;
     lv_image_compressed_t * compressed = &decoder_data->compressed;
 
@@ -910,9 +952,9 @@ static lv_result_t decode_compressed(lv_image_decoder_t * decoder, lv_image_deco
 
         /*Read compress header*/
         len = 12;
-        res = fs_read_file_at(f, sizeof(lv_image_header_t), compressed, len, &rn);
-        if(res != LV_FS_RES_OK || rn != len) {
-            LV_LOG_WARN("Read compressed header failed: %d", res);
+        fs_res = fs_read_file_at(f, sizeof(lv_image_header_t), compressed, len, &rn);
+        if(fs_res != LV_FS_RES_OK || rn != len) {
+            LV_LOG_WARN("Read compressed header failed: %d, with len: %" LV_PRIu32 ", expected: %" LV_PRIu32, fs_res, rn, len);
             return LV_RESULT_INVALID;
         }
 
@@ -929,9 +971,10 @@ static lv_result_t decode_compressed(lv_image_decoder_t * decoder, lv_image_deco
         }
 
         /*Continue to read the compressed data following compression header*/
-        res = lv_fs_read(f, file_buf, compressed_len, &rn);
-        if(res != LV_FS_RES_OK || rn != compressed_len) {
-            LV_LOG_WARN("Read compressed file failed: %d", res);
+        fs_res = lv_fs_read(f, file_buf, compressed_len, &rn);
+        if(fs_res != LV_FS_RES_OK || rn != compressed_len) {
+            LV_LOG_WARN("Read compressed file failed: %d, with len: %" LV_PRIu32 ", expected: %" LV_PRIu32, fs_res, rn,
+                        compressed_len);
             lv_free(file_buf);
             return LV_RESULT_INVALID;
         }
@@ -968,7 +1011,7 @@ static lv_result_t decode_compressed(lv_image_decoder_t * decoder, lv_image_deco
 
     /*Depends on the cf, need to further decode image like an C-array image*/
     lv_image_dsc_t * image = (lv_image_dsc_t *)dsc->src;
-    if(image->data == NULL) {
+    if(dsc->src_type == LV_IMAGE_SRC_VARIABLE && image->data == NULL) {
         return LV_RESULT_INVALID;
     }
 
@@ -1068,6 +1111,26 @@ static lv_fs_res_t fs_read_file_at(lv_fs_file_t * f, uint32_t pos, void * buff,
 
 static lv_result_t decompress_image(lv_image_decoder_dsc_t * dsc, const lv_image_compressed_t * compressed)
 {
+    /* At least one compression method must be enabled */
+#if (LV_USE_LZ4 || LV_USE_RLE)
+    /* Check if the decompression method is enabled and valid */
+    if(compressed->method == LV_IMAGE_COMPRESS_RLE) {
+#if !LV_USE_RLE
+        LV_LOG_WARN("RLE decompression is not enabled");
+        return LV_RESULT_INVALID;
+#endif
+    }
+    else if(compressed->method == LV_IMAGE_COMPRESS_LZ4) {
+#if !LV_USE_LZ4
+        LV_LOG_WARN("LZ4 decompression is not enabled");
+        return LV_RESULT_INVALID;
+#endif
+    }
+    else {
+        LV_LOG_WARN("Unknown compression method: %" LV_PRIu32, compressed->method);
+        return LV_RESULT_INVALID;
+    }
+
     /*Need to store decompressed data to decoder to free on close*/
     decoder_data_t * decoder_data = get_decoder_data(dsc);
     if(decoder_data == NULL) {
@@ -1077,17 +1140,22 @@ static lv_result_t decompress_image(lv_image_decoder_dsc_t * dsc, const lv_image
     uint8_t * img_data;
     uint32_t out_len = compressed->decompressed_size;
     uint32_t input_len = compressed->compressed_size;
-    LV_UNUSED(input_len);
-    LV_UNUSED(out_len);
+    uint32_t len = 0;
 
-    lv_draw_buf_t * decompressed = lv_draw_buf_create_user(image_cache_draw_buf_handlers, dsc->header.w, dsc->header.h,
-                                                           dsc->header.cf,
-                                                           dsc->header.stride);
+    lv_draw_buf_t * decompressed = lv_draw_buf_create_ex(image_cache_draw_buf_handlers, dsc->header.w, dsc->header.h,
+                                                         dsc->header.cf,
+                                                         dsc->header.stride);
     if(decompressed == NULL) {
         LV_LOG_WARN("No memory for decompressed image, input: %" LV_PRIu32 ", output: %" LV_PRIu32, input_len, out_len);
         return LV_RESULT_INVALID;
     }
 
+    if(out_len > decompressed->data_size) {
+        LV_LOG_ERROR("Decompressed size mismatch: %" LV_PRIu32 " > %" LV_PRIu32, out_len, decompressed->data_size);
+        lv_draw_buf_destroy(decompressed);
+        return LV_RESULT_INVALID;
+    }
+
     img_data = decompressed->data;
 
     if(compressed->method == LV_IMAGE_COMPRESS_RLE) {
@@ -1098,45 +1166,38 @@ static lv_result_t decompress_image(lv_image_decoder_dsc_t * dsc, const lv_image
             pixel_byte = 2;
         else
             pixel_byte = (lv_color_format_get_bpp(dsc->header.cf) + 7) >> 3;
+
         const uint8_t * input = compressed->data;
         uint8_t * output = img_data;
-        uint32_t len;
+
         len = lv_rle_decompress(input, input_len, output, out_len, pixel_byte);
-        if(len != compressed->decompressed_size) {
-            LV_LOG_WARN("Decompress failed: %" LV_PRIu32 ", got: %" LV_PRIu32, out_len, len);
-            lv_draw_buf_destroy(decompressed);
-            return LV_RESULT_INVALID;
-        }
-#else
-        LV_LOG_WARN("RLE decompress is not enabled");
-        lv_draw_buf_destroy(decompressed);
-        return LV_RESULT_INVALID;
-#endif
+#endif /* LV_USE_RLE */
     }
     else if(compressed->method == LV_IMAGE_COMPRESS_LZ4) {
 #if LV_USE_LZ4
         const char * input = (const char *)compressed->data;
         char * output = (char *)img_data;
-        int len;
-        len = LZ4_decompress_safe(input, output, input_len, out_len);
-        if(len < 0 || (uint32_t)len != compressed->decompressed_size) {
-            LV_LOG_WARN("Decompress failed: %" LV_PRId32 ", got: %" LV_PRId32, out_len, len);
-            lv_draw_buf_destroy(decompressed);
-            return LV_RESULT_INVALID;
+
+        int ret = LZ4_decompress_safe(input, output, (int)input_len, (int)out_len);
+        if(ret >= 0) {
+            /* Cast is safe because of the above check */
+            len = (uint32_t)ret;
         }
-#else
-        LV_LOG_WARN("LZ4 decompress is not enabled");
-        lv_draw_buf_destroy(decompressed);
-        return LV_RESULT_INVALID;
-#endif
+#endif /* LV_USE_LZ4 */
     }
-    else {
-        LV_UNUSED(img_data);
-        LV_LOG_WARN("Unknown compression method: %d", compressed->method);
+
+    if(len != compressed->decompressed_size) {
+        LV_LOG_WARN("Decompress failed: %" LV_PRIu32 ", got: %" LV_PRIu32, out_len, len);
         lv_draw_buf_destroy(decompressed);
         return LV_RESULT_INVALID;
     }
 
     decoder_data->decompressed = decompressed; /*Free on decoder close*/
     return LV_RESULT_OK;
+#else
+    LV_UNUSED(dsc);
+    LV_UNUSED(compressed);
+    LV_LOG_WARN("At least one compression method must be enabled");
+    return LV_RESULT_INVALID;
+#endif /* (LV_USE_LZ4 || LV_USE_RLE) */
 }
