@@ -6,8 +6,11 @@
 /*********************
  *      INCLUDES
  *********************/
-#include "lv_ffmpeg.h"
+#include "lv_ffmpeg_private.h"
 #if LV_USE_FFMPEG != 0
+#include "../../draw/lv_image_decoder_private.h"
+#include "../../draw/lv_draw_buf_private.h"
+#include "../../core/lv_obj_class_private.h"
 
 #include <libavcodec/avcodec.h>
 #include <libavformat/avformat.h>
@@ -36,10 +39,14 @@
 
 #define FRAME_DEF_REFR_PERIOD   33  /*[ms]*/
 
+#define DECODER_BUFFER_SIZE (8 * 1024)
+
 /**********************
  *      TYPEDEFS
  **********************/
 struct ffmpeg_context_s {
+    AVIOContext * io_ctx;
+    lv_fs_file_t lv_file;
     AVFormatContext * fmt_ctx;
     AVCodecContext * video_dec_ctx;
     AVStream * video_stream;
@@ -54,11 +61,12 @@ struct ffmpeg_context_s {
     enum AVPixelFormat video_dst_pix_fmt;
     bool has_alpha;
     lv_draw_buf_t draw_buf;
+    lv_draw_buf_handlers_t draw_buf_handlers;
 };
 
 #pragma pack(1)
 
-struct lv_image_pixel_color_s {
+struct _lv_image_pixel_color_s {
     lv_color_t c;
     uint8_t alpha;
 };
@@ -69,16 +77,19 @@ struct lv_image_pixel_color_s {
  *  STATIC PROTOTYPES
  **********************/
 
-static lv_result_t decoder_info(lv_image_decoder_t * decoder, const void * src, lv_image_header_t * header);
+static lv_result_t decoder_info(lv_image_decoder_t * decoder, lv_image_decoder_dsc_t * src, lv_image_header_t * header);
 static lv_result_t decoder_open(lv_image_decoder_t * decoder, lv_image_decoder_dsc_t * dsc);
 static void decoder_close(lv_image_decoder_t * dec, lv_image_decoder_dsc_t * dsc);
 
-static struct ffmpeg_context_s * ffmpeg_open_file(const char * path);
+static int ffmpeg_lvfs_read(void * ptr, uint8_t * buf, int buf_size);
+static int64_t ffmpeg_lvfs_seek(void * ptr, int64_t pos, int whence);
+static AVIOContext * ffmpeg_open_io_context(lv_fs_file_t * file);
+static struct ffmpeg_context_s * ffmpeg_open_file(const char * path, bool is_lv_fs_path);
 static void ffmpeg_close(struct ffmpeg_context_s * ffmpeg_ctx);
 static void ffmpeg_close_src_ctx(struct ffmpeg_context_s * ffmpeg_ctx);
 static void ffmpeg_close_dst_ctx(struct ffmpeg_context_s * ffmpeg_ctx);
 static int ffmpeg_image_allocate(struct ffmpeg_context_s * ffmpeg_ctx);
-static int ffmpeg_get_image_header(const char * path, lv_image_header_t * header);
+static int ffmpeg_get_image_header(lv_image_decoder_dsc_t * dsc, lv_image_header_t * header);
 static int ffmpeg_get_frame_refr_period(struct ffmpeg_context_s * ffmpeg_ctx);
 static uint8_t * ffmpeg_get_image_data(struct ffmpeg_context_s * ffmpeg_ctx);
 static int ffmpeg_update_next_frame(struct ffmpeg_context_s * ffmpeg_ctx);
@@ -98,7 +109,7 @@ const lv_obj_class_t lv_ffmpeg_player_class = {
     .destructor_cb = lv_ffmpeg_player_destructor,
     .instance_size = sizeof(lv_ffmpeg_player_t),
     .base_class = &lv_image_class,
-    .name = "ffmpeg-player",
+    .name = "lv_ffmpeg_player",
 };
 
 /**********************
@@ -118,15 +129,26 @@ void lv_ffmpeg_init(void)
 
     dec->name = DECODER_NAME;
 
-#if LV_FFMPEG_AV_DUMP_FORMAT == 0
+#if LV_FFMPEG_DUMP_FORMAT == 0
     av_log_set_level(AV_LOG_QUIET);
 #endif
 }
 
+void lv_ffmpeg_deinit(void)
+{
+    lv_image_decoder_t * dec = NULL;
+    while((dec = lv_image_decoder_get_next(dec)) != NULL) {
+        if(dec->info_cb == decoder_info) {
+            lv_image_decoder_delete(dec);
+            break;
+        }
+    }
+}
+
 int lv_ffmpeg_get_frame_num(const char * path)
 {
     int ret = -1;
-    struct ffmpeg_context_s * ffmpeg_ctx = ffmpeg_open_file(path);
+    struct ffmpeg_context_s * ffmpeg_ctx = ffmpeg_open_file(path, LV_FFMPEG_PLAYER_USE_LV_FS);
 
     if(ffmpeg_ctx) {
         ret = ffmpeg_ctx->video_stream->nb_frames;
@@ -157,32 +179,35 @@ lv_result_t lv_ffmpeg_player_set_src(lv_obj_t * obj, const char * path)
 
     lv_timer_pause(player->timer);
 
-    player->ffmpeg_ctx = ffmpeg_open_file(path);
+    player->ffmpeg_ctx = ffmpeg_open_file(path, LV_FFMPEG_PLAYER_USE_LV_FS);
 
     if(!player->ffmpeg_ctx) {
-        LV_LOG_ERROR("ffmpeg file open failed: %s", path);
         goto failed;
     }
 
     if(ffmpeg_image_allocate(player->ffmpeg_ctx) < 0) {
         LV_LOG_ERROR("ffmpeg image allocate failed");
         ffmpeg_close(player->ffmpeg_ctx);
+        player->ffmpeg_ctx = NULL;
         goto failed;
     }
 
     bool has_alpha = player->ffmpeg_ctx->has_alpha;
     int width = player->ffmpeg_ctx->video_dec_ctx->width;
     int height = player->ffmpeg_ctx->video_dec_ctx->height;
-    uint32_t data_size = 0;
 
-    data_size = width * height * 4;
+    uint8_t * data = ffmpeg_get_image_data(player->ffmpeg_ctx);
+    lv_color_format_t cf = has_alpha ? LV_COLOR_FORMAT_ARGB8888 : LV_COLOR_FORMAT_NATIVE;
+    uint32_t stride = width * lv_color_format_get_size(cf);
+    uint32_t data_size = stride * height;
+    lv_memzero(data, data_size);
 
     player->imgdsc.header.w = width;
     player->imgdsc.header.h = height;
     player->imgdsc.data_size = data_size;
-    player->imgdsc.header.cf = has_alpha ? LV_COLOR_FORMAT_ARGB8888 : LV_COLOR_FORMAT_NATIVE;
-    player->imgdsc.header.stride = width * lv_color_format_get_size(player->imgdsc.header.cf);
-    player->imgdsc.data = ffmpeg_get_image_data(player->ffmpeg_ctx);
+    player->imgdsc.header.cf = cf;
+    player->imgdsc.header.stride = stride;
+    player->imgdsc.data = data;
 
     lv_image_set_src(&player->img.obj, &(player->imgdsc));
 
@@ -253,17 +278,15 @@ void lv_ffmpeg_player_set_auto_restart(lv_obj_t * obj, bool en)
  *   STATIC FUNCTIONS
  **********************/
 
-static lv_result_t decoder_info(lv_image_decoder_t * decoder, const void * src, lv_image_header_t * header)
+static lv_result_t decoder_info(lv_image_decoder_t * decoder, lv_image_decoder_dsc_t * dsc, lv_image_header_t * header)
 {
     LV_UNUSED(decoder);
 
     /* Get the source type */
-    lv_image_src_t src_type = lv_image_src_get_type(src);
+    lv_image_src_t src_type = dsc->src_type;
 
     if(src_type == LV_IMAGE_SRC_FILE) {
-        const char * fn = src;
-
-        if(ffmpeg_get_image_header(fn, header) < 0) {
+        if(ffmpeg_get_image_header(dsc, header) < 0) {
             LV_LOG_ERROR("ffmpeg can't get image header");
             return LV_RESULT_INVALID;
         }
@@ -288,7 +311,7 @@ static lv_result_t decoder_open(lv_image_decoder_t * decoder, lv_image_decoder_d
     if(dsc->src_type == LV_IMAGE_SRC_FILE) {
         const char * path = dsc->src;
 
-        struct ffmpeg_context_s * ffmpeg_ctx = ffmpeg_open_file(path);
+        struct ffmpeg_context_s * ffmpeg_ctx = ffmpeg_open_file(path, true);
 
         if(ffmpeg_ctx == NULL) {
             return LV_RESULT_INVALID;
@@ -311,11 +334,24 @@ static lv_result_t decoder_open(lv_image_decoder_t * decoder, lv_image_decoder_d
 
         dsc->user_data = ffmpeg_ctx;
         lv_draw_buf_t * decoded = &ffmpeg_ctx->draw_buf;
-        decoded->header = dsc->header;
-        decoded->header.flags |= LV_IMAGE_FLAGS_MODIFIABLE;
-        decoded->data = img_data;
-        decoded->data_size = (uint32_t)dsc->header.stride * dsc->header.h;
-        decoded->unaligned_data = NULL;
+        lv_draw_buf_init(
+            decoded,
+            dsc->header.w,
+            dsc->header.h,
+            dsc->header.cf,
+            dsc->header.stride,
+            img_data,
+            dsc->header.stride * dsc->header.h);
+        lv_draw_buf_set_flag(decoded, LV_IMAGE_FLAGS_MODIFIABLE);
+
+        /* Empty handlers to avoid decoder asserts */
+        lv_draw_buf_handlers_init(&ffmpeg_ctx->draw_buf_handlers, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+        decoded->handlers = &ffmpeg_ctx->draw_buf_handlers;
+
+        if(dsc->args.premultiply && ffmpeg_ctx->has_alpha) {
+            lv_draw_buf_premultiply(decoded);
+        }
+
         dsc->decoded = decoded;
 
         /* The image is fully decoded. Return with its pointer */
@@ -398,8 +434,6 @@ static int ffmpeg_output_video_frame(struct ffmpeg_context_s * ffmpeg_ctx)
         goto failed;
     }
 
-    LV_LOG_TRACE("video_frame coded_n:%d", frame->coded_picture_number);
-
     /* copy decoded frame to destination buffer:
      * this is required since rawvideo expects non aligned data
      */
@@ -561,18 +595,33 @@ static int ffmpeg_open_codec_context(int * stream_idx,
     return 0;
 }
 
-static int ffmpeg_get_image_header(const char * filepath,
+static int ffmpeg_get_image_header(lv_image_decoder_dsc_t * dsc,
                                    lv_image_header_t * header)
 {
     int ret = -1;
 
     AVFormatContext * fmt_ctx = NULL;
     AVCodecContext * video_dec_ctx = NULL;
+    AVIOContext * io_ctx = NULL;
     int video_stream_idx;
 
+    io_ctx = ffmpeg_open_io_context(&dsc->file);
+    if(io_ctx == NULL) {
+        LV_LOG_ERROR("io_ctx malloc failed");
+        return ret;
+    }
+
+    fmt_ctx = avformat_alloc_context();
+    if(fmt_ctx == NULL) {
+        LV_LOG_ERROR("fmt_ctx malloc failed");
+        goto failed;
+    }
+    fmt_ctx->pb = io_ctx;
+    fmt_ctx->flags |= AVFMT_FLAG_CUSTOM_IO;
+
     /* open input file, and allocate format context */
-    if(avformat_open_input(&fmt_ctx, filepath, NULL, NULL) < 0) {
-        LV_LOG_ERROR("Could not open source file %s", filepath);
+    if(avformat_open_input(&fmt_ctx, (const char *)dsc->src, NULL, NULL) < 0) {
+        LV_LOG_ERROR("Could not open source file %s", (const char *)dsc->src);
         goto failed;
     }
 
@@ -599,7 +648,10 @@ static int ffmpeg_get_image_header(const char * filepath,
 failed:
     avcodec_free_context(&video_dec_ctx);
     avformat_close_input(&fmt_ctx);
-
+    if(io_ctx != NULL) {
+        av_free(io_ctx->buffer);
+        av_free(io_ctx);
+    }
     return ret;
 }
 
@@ -655,20 +707,85 @@ static int ffmpeg_update_next_frame(struct ffmpeg_context_s * ffmpeg_ctx)
     return ret;
 }
 
-struct ffmpeg_context_s * ffmpeg_open_file(const char * path)
+static int ffmpeg_lvfs_read(void * ptr, uint8_t * buf, int buf_size)
+{
+    lv_fs_file_t * file = ptr;
+    uint32_t bytesRead = 0;
+    lv_fs_res_t res = lv_fs_read(file, buf, buf_size, &bytesRead);
+    if(bytesRead == 0)
+        return AVERROR_EOF;  /* Let FFmpeg know that we have reached eof */
+    if(res != LV_FS_RES_OK)
+        return AVERROR_EOF;
+    return bytesRead;
+}
+
+static int64_t ffmpeg_lvfs_seek(void * ptr, int64_t pos, int whence)
+{
+    lv_fs_file_t * file = ptr;
+    if(whence == SEEK_SET && lv_fs_seek(file, pos, SEEK_SET) == LV_FS_RES_OK) {
+        return pos;
+    }
+    return -1;
+}
+
+static AVIOContext * ffmpeg_open_io_context(lv_fs_file_t * file)
+{
+    uint8_t * iBuffer = av_malloc(DECODER_BUFFER_SIZE);
+    if(iBuffer == NULL) {
+        LV_LOG_ERROR("iBuffer malloc failed");
+        return NULL;
+    }
+    AVIOContext * pIOCtx = avio_alloc_context(iBuffer, DECODER_BUFFER_SIZE,   /* internal Buffer and its size */
+                                              0,                                   /* bWriteable (1=true,0=false) */
+                                              file,                                /* user data ; will be passed to our callback functions */
+                                              ffmpeg_lvfs_read,                    /* Read callback function */
+                                              0,                                   /* Write callback function */
+                                              ffmpeg_lvfs_seek);                   /* Seek callback function */
+    if(pIOCtx == NULL) {
+        av_free(iBuffer);
+        return NULL;
+    }
+    return pIOCtx;
+}
+
+static struct ffmpeg_context_s * ffmpeg_open_file(const char * path, bool is_lv_fs_path)
 {
     if(path == NULL || lv_strlen(path) == 0) {
         LV_LOG_ERROR("file path is empty");
         return NULL;
     }
 
-    struct ffmpeg_context_s * ffmpeg_ctx = calloc(1, sizeof(struct ffmpeg_context_s));
-
+    struct ffmpeg_context_s * ffmpeg_ctx = lv_malloc_zeroed(sizeof(struct ffmpeg_context_s));
+    LV_ASSERT_MALLOC(ffmpeg_ctx);
     if(ffmpeg_ctx == NULL) {
         LV_LOG_ERROR("ffmpeg_ctx malloc failed");
         goto failed;
     }
 
+    if(is_lv_fs_path) {
+        const lv_fs_res_t fs_res = lv_fs_open(&(ffmpeg_ctx->lv_file), path, LV_FS_MODE_RD);
+        if(fs_res != LV_FS_RES_OK) {
+            LV_LOG_WARN("Could not open file: %s, res: %d", path, fs_res);
+            lv_free(ffmpeg_ctx);
+            return NULL;
+        }
+
+        ffmpeg_ctx->io_ctx = ffmpeg_open_io_context(&(ffmpeg_ctx->lv_file));     /* Save the buffer pointer to free it later */
+
+        if(ffmpeg_ctx->io_ctx == NULL) {
+            LV_LOG_ERROR("io_ctx malloc failed");
+            goto failed;
+        }
+
+        ffmpeg_ctx->fmt_ctx = avformat_alloc_context();
+        if(ffmpeg_ctx->fmt_ctx == NULL) {
+            LV_LOG_ERROR("fmt_ctx malloc failed");
+            goto failed;
+        }
+        ffmpeg_ctx->fmt_ctx->pb = ffmpeg_ctx->io_ctx;
+        ffmpeg_ctx->fmt_ctx->flags |= AVFMT_FLAG_CUSTOM_IO;
+    }
+
     /* open input file, and allocate format context */
 
     if(avformat_open_input(&(ffmpeg_ctx->fmt_ctx), path, NULL, NULL) < 0) {
@@ -695,7 +812,7 @@ struct ffmpeg_context_s * ffmpeg_open_file(const char * path)
         ffmpeg_ctx->video_dst_pix_fmt = (ffmpeg_ctx->has_alpha ? AV_PIX_FMT_BGRA : AV_PIX_FMT_TRUE_COLOR);
     }
 
-#if LV_FFMPEG_AV_DUMP_FORMAT != 0
+#if LV_FFMPEG_DUMP_FORMAT
     /* dump input information to stderr */
     av_dump_format(ffmpeg_ctx->fmt_ctx, 0, path, 0);
 #endif
@@ -771,6 +888,7 @@ static void ffmpeg_close_src_ctx(struct ffmpeg_context_s * ffmpeg_ctx)
 {
     avcodec_free_context(&(ffmpeg_ctx->video_dec_ctx));
     avformat_close_input(&(ffmpeg_ctx->fmt_ctx));
+    av_packet_free(&ffmpeg_ctx->pkt);
     av_frame_free(&(ffmpeg_ctx->frame));
     if(ffmpeg_ctx->video_src_data[0] != NULL) {
         av_free(ffmpeg_ctx->video_src_data[0]);
@@ -796,14 +914,19 @@ static void ffmpeg_close(struct ffmpeg_context_s * ffmpeg_ctx)
     sws_freeContext(ffmpeg_ctx->sws_ctx);
     ffmpeg_close_src_ctx(ffmpeg_ctx);
     ffmpeg_close_dst_ctx(ffmpeg_ctx);
-    free(ffmpeg_ctx);
+    if(ffmpeg_ctx->io_ctx != NULL) {
+        av_free(ffmpeg_ctx->io_ctx->buffer);
+        av_free(ffmpeg_ctx->io_ctx);
+        lv_fs_close(&(ffmpeg_ctx->lv_file));
+    }
+    lv_free(ffmpeg_ctx);
 
     LV_LOG_INFO("ffmpeg_ctx closed");
 }
 
 static void lv_ffmpeg_player_frame_update_cb(lv_timer_t * timer)
 {
-    lv_obj_t * obj = (lv_obj_t *)timer->user_data;
+    lv_obj_t * obj = (lv_obj_t *)lv_timer_get_user_data(timer);
     lv_ffmpeg_player_t * player = (lv_ffmpeg_player_t *)obj;
 
     if(!player->ffmpeg_ctx) {
@@ -814,6 +937,9 @@ static void lv_ffmpeg_player_frame_update_cb(lv_timer_t * timer)
 
     if(has_next < 0) {
         lv_ffmpeg_player_set_cmd(obj, player->auto_restart ? LV_FFMPEG_PLAYER_CMD_START : LV_FFMPEG_PLAYER_CMD_STOP);
+        if(!player->auto_restart) {
+            lv_obj_send_event((lv_obj_t *)player, LV_EVENT_READY, NULL);
+        }
         return;
     }
 
