@@ -23,7 +23,9 @@
 #include "../../lv_conf_internal.h"
 #if LV_USE_THORVG_INTERNAL
 
+
 #include "tvgMath.h"
+#include "tvgCompressor.h"
 #include "tvgLottieModel.h"
 #include "tvgLottieExpressions.h"
 
@@ -35,12 +37,14 @@
 
 struct ExpContent
 {
+    LottieExpression* exp;
     LottieObject* obj;
     float frameNo;
 };
 
+static jerry_value_t _content(const jerry_call_info_t* info, const jerry_value_t args[], const jerry_length_t argsCnt);
 
-//reserved expressions speicifiers
+//reserved expressions specifiers
 static const char* EXP_NAME = "name";
 static const char* EXP_CONTENT = "content";
 static const char* EXP_WIDTH = "width";
@@ -57,9 +61,20 @@ static const char* EXP_EFFECT= "effect";
 static LottieExpressions* exps = nullptr;   //singleton instance engine
 
 
+static ExpContent* _expcontent(LottieExpression* exp, float frameNo, LottieObject* obj)
+{
+    auto data = (ExpContent*)lv_malloc(sizeof(ExpContent));
+    LV_ASSERT_MALLOC(ndata);
+    data->exp = exp;
+    data->frameNo = frameNo;
+    data->obj = obj;
+    return data;
+}
+
+
 static void contentFree(void *native_p, struct jerry_object_native_info_t *info_p)
 {
-    free(native_p);
+    lv_free(native_p);
 }
 
 static jerry_object_native_info_t freeCb {contentFree, 0, 0};
@@ -70,7 +85,8 @@ static char* _name(jerry_value_t args)
 {
     auto arg0 = jerry_value_to_string(args);
     auto len = jerry_string_length(arg0);
-    auto name = (jerry_char_t*)malloc(len * sizeof(jerry_char_t) + 1);
+    auto name = (jerry_char_t*)lv_malloc(len * sizeof(jerry_char_t) + 1);
+    LV_ASSERT_MALLOC(name);
     jerry_string_to_buffer(arg0, JERRY_ENCODING_UTF8, name, len);
     name[len] = '\0';
     jerry_value_free(arg0);
@@ -78,43 +94,91 @@ static char* _name(jerry_value_t args)
 }
 
 
+static unsigned long _idByName(jerry_value_t args)
+{
+    auto name = _name(args);
+    auto id = djb2Encode(name);
+    lv_free(name);
+    return id;
+}
+
+
 static jerry_value_t _toComp(const jerry_call_info_t* info, const jerry_value_t args[], const jerry_length_t argsCnt)
 {
-    TVGERR("LOTTIE", "toComp is not supported in expressions!");
+    TVGLOG("LOTTIE", "toComp is not supported in expressions!");
 
     return jerry_undefined();
 }
 
 
-static void _buildTransform(jerry_value_t context, LottieTransform* transform)
+static jerry_value_t _value(float frameNo, LottieProperty* property)
+{
+    switch (property->type) {
+        case LottieProperty::Type::Point: {
+            auto value = jerry_object();
+            auto pos = (*static_cast<LottiePoint*>(property))(frameNo);
+            auto val1 = jerry_number(pos.x);
+            auto val2 = jerry_number(pos.y);
+            jerry_object_set_index(value, 0, val1);
+            jerry_object_set_index(value, 1, val2);
+            jerry_value_free(val1);
+            jerry_value_free(val2);
+            return value;
+        }
+        case LottieProperty::Type::Float: {
+            return jerry_number((*static_cast<LottieFloat*>(property))(frameNo));
+        }
+        case LottieProperty::Type::Opacity: {
+            return jerry_number((*static_cast<LottieOpacity*>(property))(frameNo));
+        }
+        case LottieProperty::Type::PathSet: {
+            auto value = jerry_object();
+            jerry_object_set_native_ptr(value, nullptr, property);
+            return value;
+        }
+        case LottieProperty::Type::Position: {
+            auto value = jerry_object();
+            auto pos = (*static_cast<LottiePosition*>(property))(frameNo);
+            auto val1 = jerry_number(pos.x);
+            auto val2 = jerry_number(pos.y);
+            jerry_object_set_index(value, 0, val1);
+            jerry_object_set_index(value, 1, val2);
+            jerry_value_free(val1);
+            jerry_value_free(val2);
+            return value;
+        }
+        default: {
+            TVGERR("LOTTIE", "Non supported type for value? = %d", (int) property->type);
+        }
+    }
+    return jerry_undefined();
+}
+
+
+static void _buildTransform(jerry_value_t context, float frameNo, LottieTransform* transform)
 {
     if (!transform) return;
 
     auto obj = jerry_object();
     jerry_object_set_sz(context, "transform", obj);
 
-    auto anchorPoint = jerry_object();
-    jerry_object_set_native_ptr(anchorPoint, nullptr, &transform->anchor);
+    auto anchorPoint = _value(frameNo, &transform->anchor);
     jerry_object_set_sz(obj, "anchorPoint", anchorPoint);
     jerry_value_free(anchorPoint);
 
-    auto position = jerry_object();
-    jerry_object_set_native_ptr(position, nullptr, &transform->position);
+    auto position = _value(frameNo, &transform->position);
     jerry_object_set_sz(obj, "position", position);
     jerry_value_free(position);
 
-    auto scale = jerry_object();
-    jerry_object_set_native_ptr(scale, nullptr, &transform->scale);
+    auto scale = _value(frameNo, &transform->scale);
     jerry_object_set_sz(obj, "scale", scale);
     jerry_value_free(scale);
 
-    auto rotation = jerry_object();
-    jerry_object_set_native_ptr(rotation, nullptr, &transform->rotation);
+    auto rotation = _value(frameNo, &transform->rotation);
     jerry_object_set_sz(obj, "rotation", rotation);
     jerry_value_free(rotation);
 
-    auto opacity = jerry_object();
-    jerry_object_set_native_ptr(opacity, nullptr, &transform->opacity);
+    auto opacity = _value(frameNo, &transform->opacity);
     jerry_object_set_sz(obj, "opacity", opacity);
     jerry_value_free(opacity);
 
@@ -122,7 +186,71 @@ static void _buildTransform(jerry_value_t context, LottieTransform* transform)
 }
 
 
-static void _buildLayer(jerry_value_t context, LottieLayer* layer, LottieComposition* comp)
+static jerry_value_t _buildGroup(LottieGroup* group, float frameNo)
+{
+    auto obj = jerry_function_external(_content);
+
+    //attach a transform
+    for (auto c = group->children.begin(); c < group->children.end(); ++c) {
+        if ((*c)->type == LottieObject::Type::Transform) {
+            _buildTransform(obj, frameNo, static_cast<LottieTransform*>(*c));
+            break;
+        }
+    }
+    jerry_object_set_native_ptr(obj, &freeCb, _expcontent(nullptr, frameNo, group));
+    jerry_object_set_sz(obj, EXP_CONTENT, obj);
+    return obj;
+}
+
+
+static jerry_value_t _buildPolystar(LottiePolyStar* polystar, float frameNo)
+{
+    auto obj = jerry_object();
+    auto position = jerry_object();
+    jerry_object_set_native_ptr(position, nullptr, &polystar->position);
+    jerry_object_set_sz(obj, "position", position);
+    jerry_value_free(position);
+    auto innerRadius = jerry_number(polystar->innerRadius(frameNo));
+    jerry_object_set_sz(obj, "innerRadius", innerRadius);
+    jerry_value_free(innerRadius);
+    auto outerRadius = jerry_number(polystar->outerRadius(frameNo));
+    jerry_object_set_sz(obj, "outerRadius", outerRadius);
+    jerry_value_free(outerRadius);
+    auto innerRoundness = jerry_number(polystar->innerRoundness(frameNo));
+    jerry_object_set_sz(obj, "innerRoundness", innerRoundness);
+    jerry_value_free(innerRoundness);
+    auto outerRoundness = jerry_number(polystar->outerRoundness(frameNo));
+    jerry_object_set_sz(obj, "outerRoundness", outerRoundness);
+    jerry_value_free(outerRoundness);
+    auto rotation = jerry_number(polystar->rotation(frameNo));
+    jerry_object_set_sz(obj, "rotation", rotation);
+    jerry_value_free(rotation);
+    auto ptsCnt = jerry_number(polystar->ptsCnt(frameNo));
+    jerry_object_set_sz(obj, "points", ptsCnt);
+    jerry_value_free(ptsCnt);
+
+    return obj;
+}
+
+
+static jerry_value_t _buildTrimpath(LottieTrimpath* trimpath, float frameNo)
+{
+    jerry_value_t obj = jerry_object();
+    auto start = jerry_number(trimpath->start(frameNo));
+    jerry_object_set_sz(obj, "start", start);
+    jerry_value_free(start);
+    auto end = jerry_number(trimpath->end(frameNo));
+    jerry_object_set_sz(obj, "end", end);
+    jerry_value_free(end);
+    auto offset = jerry_number(trimpath->offset(frameNo));
+    jerry_object_set_sz(obj, "offset", end);
+    jerry_value_free(offset);
+
+    return obj;
+}
+
+
+static void _buildLayer(jerry_value_t context, float frameNo, LottieLayer* layer, LottieLayer* comp, LottieExpression* exp)
 {
     auto width = jerry_number(layer->w);
     jerry_object_set_sz(context, EXP_WIDTH, width);
@@ -132,7 +260,7 @@ static void _buildLayer(jerry_value_t context, LottieLayer* layer, LottieComposi
     jerry_object_set_sz(context, EXP_HEIGHT, height);
     jerry_value_free(height);
 
-    auto index = jerry_number(layer->id);
+    auto index = jerry_number(layer->idx);
     jerry_object_set_sz(context, EXP_INDEX, index);
     jerry_value_free(index);
 
@@ -153,7 +281,8 @@ static void _buildLayer(jerry_value_t context, LottieLayer* layer, LottieComposi
     jerry_object_set_sz(context, "outPoint", outPoint);
     jerry_value_free(outPoint);
 
-    auto startTime = jerry_number(comp->timeAtFrame(layer->startFrame));
+    //TODO: Confirm exp->layer->comp->timeAtFrame() ?
+    auto startTime = jerry_number(exp->comp->timeAtFrame(layer->startFrame));
     jerry_object_set_sz(context, "startTime", startTime);
     jerry_value_free(startTime);
 
@@ -177,7 +306,7 @@ static void _buildLayer(jerry_value_t context, LottieLayer* layer, LottieComposi
 
     //sampleImage(point, radius = [.5, .5], postEffect=true, t=time)
 
-    _buildTransform(context, layer->transform);
+    _buildTransform(context, frameNo, layer->transform);
 
     //audioLevels, #the value of the Audio Levels property of the layer in decibels
 
@@ -199,70 +328,45 @@ static void _buildLayer(jerry_value_t context, LottieLayer* layer, LottieComposi
     jerry_object_set_sz(context, "toComp", toComp);
     jerry_object_set_native_ptr(toComp, nullptr, comp);
     jerry_value_free(toComp);
-}
 
-
-static jerry_value_t _value(float frameNo, LottieExpression* exp)
-{
-    switch (exp->type) {
-        case LottieProperty::Type::Point: {
-            auto value = jerry_object();
-            auto pos = (*static_cast<LottiePoint*>(exp->property))(frameNo);
-            auto val1 = jerry_number(pos.x);
-            auto val2 = jerry_number(pos.y);
-            jerry_object_set_index(value, 0, val1);
-            jerry_object_set_index(value, 1, val2);
-            jerry_value_free(val1);
-            jerry_value_free(val2);
-            return value;
-        }
-        case LottieProperty::Type::Float: {
-            return jerry_number((*static_cast<LottieFloat*>(exp->property))(frameNo));
-        }
-        case LottieProperty::Type::Opacity: {
-            return jerry_number((*static_cast<LottieOpacity*>(exp->property))(frameNo));
-        }
-        case LottieProperty::Type::PathSet: {
-            auto value = jerry_object();
-            jerry_object_set_native_ptr(value, nullptr, exp->property);
-            return value;
-        }
-        case LottieProperty::Type::Position: {
-            auto value = jerry_object();
-            auto pos = (*static_cast<LottiePosition*>(exp->property))(frameNo);
-            auto val1 = jerry_number(pos.x);
-            auto val2 = jerry_number(pos.y);
-            jerry_object_set_index(value, 0, val1);
-            jerry_object_set_index(value, 1, val2);
-            jerry_value_free(val1);
-            jerry_value_free(val2);
-            return value;
-        }
-        default: {
-            TVGERR("LOTTIE", "Non supported type for value? = %d", (int) exp->type);
-        }
-    }
-    return jerry_undefined();
+    //content("name"), #look for the named property from a layer
+    auto content = jerry_function_external(_content);
+    jerry_object_set_sz(context, EXP_CONTENT, content);
+    jerry_object_set_native_ptr(content, &freeCb, _expcontent(exp, frameNo, layer));
+    jerry_value_free(content);
 }
 
 
 static jerry_value_t _addsub(const jerry_value_t args[], float addsub)
 {
-    //1d
-    if (jerry_value_is_number(args[0])) return jerry_number(jerry_value_as_number(args[0]) + addsub * jerry_value_as_number(args[1]));
+    auto n1 = jerry_value_is_number(args[0]);
+    auto n2 = jerry_value_is_number(args[1]);
 
-    //2d
-    auto val1 = jerry_object_get_index(args[0], 0);
-    auto val2 = jerry_object_get_index(args[0], 1);
-    auto val3 = jerry_object_get_index(args[1], 0);
-    auto val4 = jerry_object_get_index(args[1], 1);
-    auto x = jerry_value_as_number(val1) + addsub * jerry_value_as_number(val3);
-    auto y = jerry_value_as_number(val2) + addsub * jerry_value_as_number(val4);
+    //1d + 1d
+    if (n1 && n2) return jerry_number(jerry_value_as_number(args[0]) + addsub * jerry_value_as_number(args[1]));
 
+    auto val1 = jerry_object_get_index(args[n1 ? 1 : 0], 0);
+    auto val2 = jerry_object_get_index(args[n1 ? 1 : 0], 1);
+    auto x = jerry_value_as_number(val1);
+    auto y = jerry_value_as_number(val2);
     jerry_value_free(val1);
     jerry_value_free(val2);
-    jerry_value_free(val3);
-    jerry_value_free(val4);
+
+    //2d + 1d
+    if (n1 || n2) {
+        auto secondary = n1 ? 0 : 1;
+        auto val3 = jerry_value_as_number(args[secondary]);
+        if (secondary == 0) x = (x * addsub) + val3;
+        else x += (addsub * val3);
+    //2d + 2d
+    } else {
+        auto val3 = jerry_object_get_index(args[1], 0);
+        auto val4 = jerry_object_get_index(args[1], 1);
+        x += (addsub * jerry_value_as_number(val3));
+        y += (addsub * jerry_value_as_number(val4));
+        jerry_value_free(val3);
+        jerry_value_free(val4);
+    }
 
     auto obj = jerry_object();
     val1 = jerry_number(x);
@@ -332,25 +436,25 @@ static jerry_value_t _interp(float t, const jerry_value_t args[], int argsCnt)
     auto tMax = 1.0f;
     int idx = 0;
 
-    if (argsCnt > 3) {
-        tMin = jerry_value_as_number(args[1]);
-        tMax = jerry_value_as_number(args[2]);
-        idx += 2;
-    }
+    tMin = jerry_value_as_number(args[1]);
+    tMax = jerry_value_as_number(args[2]);
+    idx += 2;
+
+    t = (t - tMin) / (tMax - tMin);
+    if (t < 0) t = 0.0f;
+    else if (t > 1) t = 1.0f;
 
     //2d
     if (jerry_value_is_object(args[idx + 1]) && jerry_value_is_object(args[idx + 2])) {
-        auto val1 = jerry_object_get_index(args[0], 0);
-        auto val2 = jerry_object_get_index(args[0], 1);
-        auto val3 = jerry_object_get_index(args[1], 0);
-        auto val4 = jerry_object_get_index(args[1], 1);
+        auto val1 = jerry_object_get_index(args[idx + 1], 0);
+        auto val2 = jerry_object_get_index(args[idx + 1], 1);
+        auto val3 = jerry_object_get_index(args[idx + 2], 0);
+        auto val4 = jerry_object_get_index(args[idx + 2], 1);
 
         Point pt1 = {(float)jerry_value_as_number(val1),  (float)jerry_value_as_number(val2)};
         Point pt2 = {(float)jerry_value_as_number(val3),  (float)jerry_value_as_number(val4)};
         Point ret;
-        if (t <= tMin) ret = pt1;
-        else if (t >= tMax) ret = pt2;
-        else ret = mathLerp(pt1, pt2, t);
+        ret = lerp(pt1, pt2, t);
 
         jerry_value_free(val1);
         jerry_value_free(val2);
@@ -370,10 +474,8 @@ static jerry_value_t _interp(float t, const jerry_value_t args[], int argsCnt)
 
     //1d
     auto val1 = (float) jerry_value_as_number(args[idx + 1]);
-    if (t <= tMin) jerry_number(val1);
     auto val2 = (float) jerry_value_as_number(args[idx + 2]);
-    if (t >= tMax) jerry_number(val2);
-    return jerry_number(mathLerp(val1, val2, t));
+    return jerry_number(lerp(val1, val2, t));
 }
 
 
@@ -511,19 +613,19 @@ static jerry_value_t _random(const jerry_call_info_t* info, const jerry_value_t
 
 static jerry_value_t _deg2rad(const jerry_call_info_t* info, const jerry_value_t args[], const jerry_length_t argsCnt)
 {
-    return jerry_number(mathDeg2Rad((float)jerry_value_as_number(args[0])));
+    return jerry_number(deg2rad((float)jerry_value_as_number(args[0])));
 }
 
 
 static jerry_value_t _rad2deg(const jerry_call_info_t* info, const jerry_value_t args[], const jerry_length_t argsCnt)
 {
-    return jerry_number(mathRad2Deg((float)jerry_value_as_number(args[0])));
+    return jerry_number(rad2deg((float)jerry_value_as_number(args[0])));
 }
 
 
 static jerry_value_t _effect(const jerry_call_info_t* info, const jerry_value_t args[], const jerry_length_t argsCnt)
 {
-    TVGERR("LOTTIE", "effect is not supported in expressions!");
+    TVGLOG("LOTTIE", "effect is not supported in expressions!");
 
     return jerry_undefined();
 }
@@ -531,7 +633,7 @@ static jerry_value_t _effect(const jerry_call_info_t* info, const jerry_value_t
 
 static jerry_value_t _fromCompToSurface(const jerry_call_info_t* info, const jerry_value_t args[], const jerry_length_t argsCnt)
 {
-    TVGERR("LOTTIE", "fromCompToSurface is not supported in expressions!");
+    TVGLOG("LOTTIE", "fromCompToSurface is not supported in expressions!");
 
     return jerry_undefined();
 }
@@ -539,53 +641,22 @@ static jerry_value_t _fromCompToSurface(const jerry_call_info_t* info, const jer
 
 static jerry_value_t _content(const jerry_call_info_t* info, const jerry_value_t args[], const jerry_length_t argsCnt)
 {
-    auto name = _name(args[0]);
     auto data = static_cast<ExpContent*>(jerry_object_get_native_ptr(info->function, &freeCb));
     auto group = static_cast<LottieGroup*>(data->obj);
-    auto target = group->content((char*)name);
-    free(name);
+    auto target = group->content(_idByName(args[0]));
     if (!target) return jerry_undefined();
 
     //find the a path property(sh) in the group layer?
     switch (target->type) {
-        case LottieObject::Group: {
-            auto group = static_cast<LottieGroup*>(target);
-            auto obj = jerry_function_external(_content);
-
-            //attach a transform
-            for (auto c = group->children.begin(); c < group->children.end(); ++c) {
-                if ((*c)->type == LottieObject::Type::Transform) {
-                    _buildTransform(obj, static_cast<LottieTransform*>(*c));
-                    break;
-                }
-            }
-            auto data2 = (ExpContent*)malloc(sizeof(ExpContent));
-            data2->obj = group;
-            data2->frameNo = data->frameNo;
-            jerry_object_set_native_ptr(obj, &freeCb, data2);
-            jerry_object_set_sz(obj, EXP_CONTENT, obj);
-            return obj;
-        }
+        case LottieObject::Group: return _buildGroup(static_cast<LottieGroup*>(target), data->frameNo);
         case LottieObject::Path: {
             jerry_value_t obj = jerry_object();
             jerry_object_set_native_ptr(obj, nullptr, &static_cast<LottiePath*>(target)->pathset);
             jerry_object_set_sz(obj, "path", obj);
             return obj;
         }
-        case LottieObject::Trimpath: {
-            auto trimpath = static_cast<LottieTrimpath*>(target);
-            jerry_value_t obj = jerry_object();
-            auto start = jerry_number(trimpath->start(data->frameNo));
-            jerry_object_set_sz(obj, "start", start);
-            jerry_value_free(start);
-            auto end = jerry_number(trimpath->end(data->frameNo));
-            jerry_object_set_sz(obj, "end", end);
-            jerry_value_free(end);
-            auto offset = jerry_number(trimpath->offset(data->frameNo));
-            jerry_object_set_sz(obj, "offset", end);
-            jerry_value_free(offset);
-            return obj;
-        }
+        case LottieObject::Polystar: return _buildPolystar(static_cast<LottiePolyStar*>(target), data->frameNo);
+        case LottieObject::Trimpath: return _buildTrimpath(static_cast<LottieTrimpath*>(target), data->frameNo);
         default: break;
     }
     return jerry_undefined();
@@ -594,26 +665,25 @@ static jerry_value_t _content(const jerry_call_info_t* info, const jerry_value_t
 
 static jerry_value_t _layer(const jerry_call_info_t* info, const jerry_value_t args[], const jerry_length_t argsCnt)
 {
-    auto comp = static_cast<LottieComposition*>(jerry_object_get_native_ptr(info->function, nullptr));
+    auto data = static_cast<ExpContent*>(jerry_object_get_native_ptr(info->function, &freeCb));
+    auto comp = static_cast<LottieLayer*>(data->obj);
     LottieLayer* layer;
 
     //layer index
     if (jerry_value_is_number(args[0])) {
         auto idx = (uint16_t)jerry_value_as_int32(args[0]);
-        layer = comp->layer(idx);
+        layer = comp->layerByIdx(idx);
         jerry_value_free(idx);
     //layer name
     } else {
-        auto name = _name(args[0]);
-        layer = comp->layer((char*)name);
-        free(name);
+        layer = comp->layerById(_idByName(args[0]));
     }
 
     if (!layer) return jerry_undefined();
 
     auto obj = jerry_object();
     jerry_object_set_native_ptr(obj, nullptr, layer);
-    _buildLayer(obj, layer, comp);
+    _buildLayer(obj, data->frameNo, layer, comp, data->exp);
 
     return obj;
 }
@@ -633,13 +703,54 @@ static jerry_value_t _nearestKey(const jerry_call_info_t* info, const jerry_valu
     return obj;
 }
 
+static jerry_value_t _property(const jerry_call_info_t* info, const jerry_value_t args[], const jerry_length_t argsCnt)
+{
+    auto data = static_cast<ExpContent*>(jerry_object_get_native_ptr(info->function, &freeCb));
+    auto property = data->obj->property(jerry_value_as_int32(args[0]));
+    if (!property) return jerry_undefined();
+    return _value(data->frameNo, property);
+}
+
+
+static jerry_value_t _propertyGroup(const jerry_call_info_t* info, const jerry_value_t args[], const jerry_length_t argsCnt)
+{
+    auto data = static_cast<ExpContent*>(jerry_object_get_native_ptr(info->function, &freeCb));
+    auto level = jerry_value_as_int32(args[0]);
+
+    //intermediate group
+    if (level == 1) {
+        auto group = jerry_function_external(_property);
+        jerry_object_set_native_ptr(group, &freeCb, _expcontent(data->exp, data->frameNo, data->obj));
+        jerry_object_set_sz(group, "", group);
+        return group;
+    }
+
+    TVGLOG("LOTTIE", "propertyGroup(%d)?", level);
+
+    return jerry_undefined();
+}
+
 
 static jerry_value_t _valueAtTime(const jerry_call_info_t* info, const jerry_value_t args[], const jerry_length_t argsCnt)
 {
     auto exp = static_cast<LottieExpression*>(jerry_object_get_native_ptr(info->function, nullptr));
     auto time = jerry_value_as_number(args[0]);
     auto frameNo = exp->comp->frameAtTime(time);
-    return _value(frameNo, exp);
+    return _value(frameNo, exp->property);
+}
+
+
+static jerry_value_t _velocity(float px, float cx, float py, float cy, float elapsed)
+{
+    float velocity[] = {(cx - px) / elapsed, (cy - py) / elapsed};
+    auto obj = jerry_object();
+    auto val1 = jerry_number(velocity[0]);
+    auto val2 = jerry_number(velocity[1]);
+    jerry_object_set_index(obj, 0, val1);
+    jerry_object_set_index(obj, 1, val2);
+    jerry_value_free(val1);
+    jerry_value_free(val2);
+    return obj;
 }
 
 
@@ -653,37 +764,27 @@ static jerry_value_t _velocityAtTime(const jerry_call_info_t* info, const jerry_
     auto cframe = exp->property->frameNo(key);
     auto elapsed = (cframe - pframe) / (exp->comp->frameRate);
 
-    Point cur, prv;
-
     //compute the velocity
-    switch (exp->type) {
+    switch (exp->property->type) {
         case LottieProperty::Type::Point: {
-            prv = (*static_cast<LottiePoint*>(exp->property))(pframe);
-            cur = (*static_cast<LottiePoint*>(exp->property))(cframe);
-            break;
+            auto prv = (*static_cast<LottiePoint*>(exp->property))(pframe);
+            auto cur = (*static_cast<LottiePoint*>(exp->property))(cframe);
+            return _velocity(prv.x, cur.x, prv.y, cur.y, elapsed);
         }
         case LottieProperty::Type::Position: {
-            prv = (*static_cast<LottiePosition*>(exp->property))(pframe);
-            cur = (*static_cast<LottiePosition*>(exp->property))(cframe);
-            break;
+            auto prv = (*static_cast<LottiePosition*>(exp->property))(pframe);
+            auto cur = (*static_cast<LottiePosition*>(exp->property))(cframe);
+            return _velocity(prv.x, cur.x, prv.y, cur.y, elapsed);
         }
-        default: {
-            TVGERR("LOTTIE", "Non supported type for velocityAtTime?");
-            return jerry_undefined();
+        case LottieProperty::Type::Float: {
+            auto prv = (*static_cast<LottieFloat*>(exp->property))(pframe);
+            auto cur = (*static_cast<LottieFloat*>(exp->property))(cframe);
+            auto velocity = (cur - prv) / elapsed;
+            return jerry_number(velocity);
         }
+        default: TVGLOG("LOTTIE", "Non supported type for velocityAtTime?");
     }
-
-    float velocity[] = {(cur.x - prv.x) / elapsed, (cur.y - prv.y) / elapsed};
-
-    auto obj = jerry_object();
-    auto val1 = jerry_number(velocity[0]);
-    auto val2 = jerry_number(velocity[1]);
-    jerry_object_set_index(obj, 0, val1);
-    jerry_object_set_index(obj, 1, val2);
-    jerry_value_free(val1);
-    jerry_value_free(val2);
-
-    return obj;
+    return jerry_undefined();
 }
 
 
@@ -700,7 +801,7 @@ static jerry_value_t _speedAtTime(const jerry_call_info_t* info, const jerry_val
     Point cur, prv;
 
     //compute the velocity
-    switch (exp->type) {
+    switch (exp->property->type) {
         case LottieProperty::Type::Point: {
             prv = (*static_cast<LottiePoint*>(exp->property))(pframe);
             cur = (*static_cast<LottiePoint*>(exp->property))(cframe);
@@ -712,7 +813,7 @@ static jerry_value_t _speedAtTime(const jerry_call_info_t* info, const jerry_val
             break;
         }
         default: {
-            TVGERR("LOTTIE", "Non supported type for speedAtTime?");
+            TVGLOG("LOTTIE", "Non supported type for speedAtTime?");
             return jerry_undefined();
         }
     }
@@ -733,11 +834,11 @@ static bool _loopOutCommon(LottieExpression* exp, const jerry_value_t args[], co
         else if (!strcmp(name, EXP_PINGPONG)) exp->loop.mode = LottieExpression::LoopMode::OutPingPong;
         else if (!strcmp(name, EXP_OFFSET)) exp->loop.mode = LottieExpression::LoopMode::OutOffset;
         else if (!strcmp(name, EXP_CONTINUE)) exp->loop.mode = LottieExpression::LoopMode::OutContinue;
-        free(name);
+        lv_free(name);
     }
 
-    if (exp->loop.mode != LottieExpression::LoopMode::OutCycle) {
-        TVGERR("hermet", "Not supported loopOut type = %d", exp->loop.mode);
+    if (exp->loop.mode != LottieExpression::LoopMode::OutCycle && exp->loop.mode != LottieExpression::LoopMode::OutPingPong) {
+        TVGLOG("LOTTIE", "Not supported loopOut type = %d", exp->loop.mode);
         return false;
     }
 
@@ -785,11 +886,11 @@ static bool _loopInCommon(LottieExpression* exp, const jerry_value_t args[], con
         else if (!strcmp(name, EXP_PINGPONG)) exp->loop.mode = LottieExpression::LoopMode::InPingPong;
         else if (!strcmp(name, EXP_OFFSET)) exp->loop.mode = LottieExpression::LoopMode::InOffset;
         else if (!strcmp(name, EXP_CONTINUE)) exp->loop.mode = LottieExpression::LoopMode::InContinue;
-        free(name);
+        lv_free(name);
     }
 
-    if (exp->loop.mode != LottieExpression::LoopMode::InCycle) {
-        TVGERR("hermet", "Not supported loopOut type = %d", exp->loop.mode);
+    if (exp->loop.mode != LottieExpression::LoopMode::InCycle && exp->loop.mode != LottieExpression::LoopMode::InPingPong) {
+        TVGLOG("LOTTIE", "Not supported loopIn type = %d", exp->loop.mode);
         return false;
     }
 
@@ -802,9 +903,7 @@ static jerry_value_t _loopIn(const jerry_call_info_t* info, const jerry_value_t
 
     if (!_loopInCommon(exp, args, argsCnt)) return jerry_undefined();
 
-    if (argsCnt > 1) {
-        exp->loop.in = exp->comp->frameAtTime((float)jerry_value_as_int32(args[1]));
-    }
+    if (argsCnt > 1) exp->loop.key = jerry_value_as_int32(args[1]);
 
     auto obj = jerry_object();
     jerry_object_set_native_ptr(obj, nullptr, exp->property);
@@ -834,13 +933,21 @@ static jerry_value_t _key(const jerry_call_info_t* info, const jerry_value_t arg
     auto key = jerry_value_as_int32(args[0]);
     auto frameNo = exp->property->frameNo(key);
     auto time = jerry_number(exp->comp->timeAtFrame(frameNo));
-    auto value = _value(frameNo, exp);
+    auto value = _value(frameNo, exp->property);
 
     auto obj = jerry_object();
     jerry_object_set_sz(obj, EXP_TIME, time);
     jerry_object_set_sz(obj, EXP_INDEX, args[0]);
     jerry_object_set_sz(obj, EXP_VALUE, value);
 
+    //direct access, key[0], key[1]
+    if (exp->property->type == LottieProperty::Type::Float) {
+        jerry_object_set_index(obj, 0, value);
+    } else if (exp->property->type == LottieProperty::Type::Point || exp->property->type == LottieProperty::Type::Position) {
+        jerry_object_set_index(obj, 0, jerry_object_get_index(value, 0));
+        jerry_object_set_index(obj, 1, jerry_object_get_index(value, 1));
+    }
+
     jerry_value_free(time);
     jerry_value_free(value);
 
@@ -848,10 +955,9 @@ static jerry_value_t _key(const jerry_call_info_t* info, const jerry_value_t arg
 }
 
 
-
 static jerry_value_t _createPath(const jerry_call_info_t* info, const jerry_value_t args[], const jerry_length_t argsCnt)
 {
-    //TODO: arg1: points, arg2: inTagents, arg3: outTangents, arg4: isClosed
+    //TODO: arg1: points, arg2: inTangents, arg3: outTangents, arg4: isClosed
     auto arg1 = jerry_value_to_object(args[0]);
     auto pathset = jerry_object_get_native_ptr(arg1, nullptr);
     if (!pathset) {
@@ -872,7 +978,7 @@ static jerry_value_t _uniformPath(const jerry_call_info_t* info, const jerry_val
     auto pathset = static_cast<LottiePathSet*>(jerry_object_get_native_ptr(info->function, nullptr));
 
     /* TODO: ThorVG prebuilds the path data for performance.
-       It acutally need to constructs the Array<Point> for points, inTangents, outTangents and then return here... */
+       It actually need to constructs the Array<Point> for points, inTangents, outTangents and then return here... */
     auto obj = jerry_object();
     jerry_object_set_native_ptr(obj, nullptr, pathset);
     return obj;
@@ -888,7 +994,7 @@ static jerry_value_t _isClosed(const jerry_call_info_t* info, const jerry_value_
 
 static void _buildPath(jerry_value_t context, LottieExpression* exp)
 {
-    //Trick for fast buliding path.
+    //Trick for fast building path.
     auto points = jerry_function_external(_uniformPath);
     jerry_object_set_native_ptr(points, nullptr, exp->property);
     jerry_object_set_sz(context, "points", points);
@@ -914,7 +1020,7 @@ static void _buildPath(jerry_value_t context, LottieExpression* exp)
 
 static void _buildProperty(float frameNo, jerry_value_t context, LottieExpression* exp)
 {
-    auto value = _value(frameNo, exp);
+    auto value = _value(frameNo, exp->property);
     jerry_object_set_sz(context, EXP_VALUE, value);
     jerry_value_free(value);
 
@@ -981,42 +1087,37 @@ static void _buildProperty(float frameNo, jerry_value_t context, LottieExpressio
     jerry_object_set_sz(context, "numKeys", numKeys);
     jerry_value_free(numKeys);
 
-    //propertyGroup(countUp = 1)
+    auto propertyGroup = jerry_function_external(_propertyGroup);
+    jerry_object_set_native_ptr(propertyGroup, &freeCb, _expcontent(exp, frameNo, exp->object));
+    jerry_object_set_sz(context, "propertyGroup", propertyGroup);
+    jerry_value_free(propertyGroup);
+
     //propertyIndex
+
     //name
 
     //content("name"), #look for the named property from a layer
-    auto data = (ExpContent*)malloc(sizeof(ExpContent));
-    data->obj = exp->layer;
-    data->frameNo = frameNo;
-
     auto content = jerry_function_external(_content);
     jerry_object_set_sz(context, EXP_CONTENT, content);
-    jerry_object_set_native_ptr(content, &freeCb, data);
+    jerry_object_set_native_ptr(content, &freeCb, _expcontent(exp, frameNo, exp->layer));
     jerry_value_free(content);
+
+    //expansions per types
+    if (exp->property->type == LottieProperty::Type::PathSet) _buildPath(context, exp);
 }
 
 
 static jerry_value_t _comp(const jerry_call_info_t* info, const jerry_value_t args[], const jerry_length_t argsCnt)
 {
-    auto comp = static_cast<LottieComposition*>(jerry_object_get_native_ptr(info->function, nullptr));
-    LottieLayer* layer;
-
-    auto arg0 = jerry_value_to_string(args[0]);
-    auto len = jerry_string_length(arg0);
-    auto name = (jerry_char_t*)alloca(len * sizeof(jerry_char_t) + 1);
-    jerry_string_to_buffer(arg0, JERRY_ENCODING_UTF8, name, len);
-    name[len] = '\0';
-
-    jerry_value_free(arg0);
-
-    layer = comp->asset((char*)name);
+    auto data = static_cast<ExpContent*>(jerry_object_get_native_ptr(info->function, &freeCb));
+    auto comp = static_cast<LottieLayer*>(data->obj);
+    auto layer = comp->layerById(_idByName(args[0]));
 
     if (!layer) return jerry_undefined();
 
     auto obj = jerry_object();
     jerry_object_set_native_ptr(obj, nullptr, layer);
-    _buildLayer(obj, layer, comp);
+    _buildLayer(obj, data->frameNo, layer, comp, data->exp);
 
     return obj;
 }
@@ -1116,11 +1217,37 @@ static void _buildMath(jerry_value_t context)
 }
 
 
-void LottieExpressions::buildComp(LottieComposition* comp)
+void LottieExpressions::buildGlobal(LottieExpression* exp)
 {
-    jerry_object_set_native_ptr(this->comp, nullptr, comp);
-    jerry_object_set_native_ptr(thisComp, nullptr, comp);
-    jerry_object_set_native_ptr(layer, nullptr, comp);
+    auto index = jerry_number(exp->layer->idx);
+    jerry_object_set_sz(global, EXP_INDEX, index);
+    jerry_value_free(index);
+}
+
+
+void LottieExpressions::buildComp(jerry_value_t context, float frameNo, LottieLayer* comp, LottieExpression* exp)
+{
+    auto data = static_cast<ExpContent*>(jerry_object_get_native_ptr(context, &freeCb));
+    data->exp = exp;
+    data->frameNo = frameNo;
+    data->obj = comp;
+
+    //layer(index) / layer(name) / layer(otherLayer, reIndex)
+    auto layer = jerry_function_external(_layer);
+    jerry_object_set_sz(context, "layer", layer);
+
+    jerry_object_set_native_ptr(layer, &freeCb, _expcontent(exp, frameNo, comp));
+    jerry_value_free(layer);
+
+    auto numLayers = jerry_number(comp->children.count);
+    jerry_object_set_sz(context, "numLayers", numLayers);
+    jerry_value_free(numLayers);
+}
+
+
+void LottieExpressions::buildComp(LottieComposition* comp, float frameNo, LottieExpression* exp)
+{
+    buildComp(this->comp, frameNo, comp->root, exp);
 
     //marker
     //marker.key(index)
@@ -1128,10 +1255,6 @@ void LottieExpressions::buildComp(LottieComposition* comp)
     //marker.nearestKey(t)
     //marker.numKeys
 
-    auto numLayers = jerry_number(comp->root->children.count);
-    jerry_object_set_sz(thisComp, "numLayers", numLayers);
-    jerry_value_free(numLayers);
-
     //activeCamera
 
     auto width = jerry_number(comp->w);
@@ -1170,17 +1293,15 @@ jerry_value_t LottieExpressions::buildGlobal()
 
     //comp(name)
     comp = jerry_function_external(_comp);
+    jerry_object_set_native_ptr(comp, &freeCb, _expcontent(nullptr, 0.0f, nullptr));
     jerry_object_set_sz(global, "comp", comp);
 
     //footage(name)
 
     thisComp = jerry_object();
+    jerry_object_set_native_ptr(thisComp, &freeCb, _expcontent(nullptr, 0.0f, nullptr));
     jerry_object_set_sz(global, "thisComp", thisComp);
 
-    //layer(index) / layer(name) / layer(otherLayer, reIndex)
-    layer = jerry_function_external(_layer);
-    jerry_object_set_sz(thisComp, "layer", layer);
-
     thisLayer = jerry_object();
     jerry_object_set_sz(global, "thisLayer", thisLayer);
 
@@ -1208,23 +1329,36 @@ jerry_value_t LottieExpressions::buildGlobal()
 
 jerry_value_t LottieExpressions::evaluate(float frameNo, LottieExpression* exp)
 {
-    buildComp(exp->comp);
+    if (exp->disabled) return jerry_undefined();
+
+    buildGlobal(exp);
+
+    //main composition
+    buildComp(exp->comp, frameNo, exp);
+
+    //this composition
+    buildComp(thisComp, frameNo, exp->layer->comp, exp);
 
     //update global context values
-    jerry_object_set_native_ptr(thisLayer, nullptr, exp->layer);
-    _buildLayer(thisLayer, exp->layer, exp->comp);
-
-    jerry_object_set_native_ptr(thisProperty, nullptr, exp->property);
     _buildProperty(frameNo, global, exp);
 
-    if (exp->type == LottieProperty::Type::PathSet) _buildPath(thisProperty, exp);
-    if (exp->object->type == LottieObject::Transform) _buildTransform(global, static_cast<LottieTransform*>(exp->object));
+    //this layer
+    jerry_object_set_native_ptr(thisLayer, nullptr, exp->layer);
+    _buildLayer(thisLayer, frameNo, exp->layer, exp->comp->root, exp);
+
+    //this property
+    jerry_object_set_native_ptr(thisProperty, nullptr, exp->property);
+    _buildProperty(frameNo, thisProperty, exp);
+
+    //expansions per object type
+    if (exp->object->type == LottieObject::Transform) _buildTransform(global, frameNo, static_cast<LottieTransform*>(exp->object));
 
     //evaluate the code
     auto eval = jerry_eval((jerry_char_t *) exp->code, strlen(exp->code), JERRY_PARSE_NO_OPTS);
 
     if (jerry_value_is_exception(eval) || jerry_value_is_undefined(eval)) {
-        exp->enabled = false;  // The feature is experimental, it will be forcely turned off if it's incompatible.
+        TVGERR("LOTTIE", "Failed to dispatch the expressions!");
+        exp->disabled = true;
         return jerry_undefined();
     }
 
@@ -1242,7 +1376,6 @@ LottieExpressions::~LottieExpressions()
 {
     jerry_value_free(thisProperty);
     jerry_value_free(thisLayer);
-    jerry_value_free(layer);
     jerry_value_free(thisComp);
     jerry_value_free(comp);
     jerry_value_free(global);
