@@ -1,4 +1,4 @@
-﻿/**
+/**
  * @file lv_style.h
  *
  */
@@ -21,6 +21,7 @@ extern "C" {
 #include "lv_types.h"
 #include "lv_assert.h"
 #include "lv_bidi.h"
+#include "lv_grad.h"
 #include "../layouts/lv_layout.h"
 
 /*********************
@@ -29,24 +30,22 @@ extern "C" {
 
 #define LV_STYLE_SENTINEL_VALUE     0xAABBCCDD
 
-/**
+/*
  * Flags for style behavior
- *
- * The rest of the flags will have _FLAG added to their name in v9.
  */
-#define LV_STYLE_PROP_FLAG_NONE                     (0)
-#define LV_STYLE_PROP_FLAG_INHERITABLE              (1 << 0)  /*Inherited*/
-#define LV_STYLE_PROP_FLAG_EXT_DRAW_UPDATE          (1 << 1)  /*Requires ext. draw size update when changed*/
-#define LV_STYLE_PROP_FLAG_LAYOUT_UPDATE            (1 << 2)  /*Requires layout update when changed*/
-#define LV_STYLE_PROP_FLAG_PARENT_LAYOUT_UPDATE     (1 << 3)  /*Requires layout update on parent when changed*/
-#define LV_STYLE_PROP_FLAG_LAYER_UPDATE             (1 << 4)  /*Affects layer handling*/
-#define LV_STYLE_PROP_FLAG_TRANSFORM                (1 << 5)  /*Affects the object's transformation*/
-#define LV_STYLE_PROP_FLAG_ALL                      (0x3F)    /*Indicating all flags*/
+#define LV_STYLE_PROP_FLAG_NONE                     (0)       /**< No special behavior */
+#define LV_STYLE_PROP_FLAG_INHERITABLE              (1 << 0)  /**< Inherited */
+#define LV_STYLE_PROP_FLAG_EXT_DRAW_UPDATE          (1 << 1)  /**< Requires ext. draw size update when changed */
+#define LV_STYLE_PROP_FLAG_LAYOUT_UPDATE            (1 << 2)  /**< Requires layout update when changed */
+#define LV_STYLE_PROP_FLAG_PARENT_LAYOUT_UPDATE     (1 << 3)  /**< Requires layout update on parent when changed */
+#define LV_STYLE_PROP_FLAG_LAYER_UPDATE             (1 << 4)  /**< Affects layer handling */
+#define LV_STYLE_PROP_FLAG_TRANSFORM                (1 << 5)  /**< Affects the object's transformation */
+#define LV_STYLE_PROP_FLAG_ALL                      (0x3F)    /**< Indicating all flags */
 
-/**
+/*
  * Other constants
  */
-#define LV_SCALE_NONE            256        /*Value for not zooming the image*/
+#define LV_SCALE_NONE            256        /**< Value for not zooming the image */
 LV_EXPORT_CONST_INT(LV_SCALE_NONE);
 
 // *INDENT-OFF*
@@ -54,63 +53,62 @@ LV_EXPORT_CONST_INT(LV_SCALE_NONE);
 #define LV_STYLE_CONST_INIT(var_name, prop_array)                       \
     const lv_style_t var_name = {                                       \
         .sentinel = LV_STYLE_SENTINEL_VALUE,                            \
-        .values_and_props = (void*)prop_array,                                      \
+        .values_and_props = (void*)prop_array,                          \
         .has_group = 0xFFFFFFFF,                                        \
-        .prop_cnt = 255                                               \
+        .prop_cnt = 255                                                 \
     }
 #else
 #define LV_STYLE_CONST_INIT(var_name, prop_array)                       \
     const lv_style_t var_name = {                                       \
-        .values_and_props = prop_array,                                      \
+        .values_and_props = (void*)prop_array,                          \
         .has_group = 0xFFFFFFFF,                                        \
-        .prop_cnt = 255,                                               \
+        .prop_cnt = 255,                                                \
     }
 #endif
 // *INDENT-ON*
 
-#define LV_STYLE_CONST_PROPS_END { .prop_ptr = NULL, .value = { .num = 0 } }
+#define LV_STYLE_CONST_PROPS_END { .prop = LV_STYLE_PROP_INV, .value = { .num = 0 } }
+
+#if LV_GRADIENT_MAX_STOPS < 2
+#error LVGL needs at least 2 stops for gradients. Please increase the LV_GRADIENT_MAX_STOPS
+#endif
+
+#define LV_GRAD_LEFT    LV_PCT(0)
+#define LV_GRAD_RIGHT   LV_PCT(100)
+#define LV_GRAD_TOP     LV_PCT(0)
+#define LV_GRAD_BOTTOM  LV_PCT(100)
+#define LV_GRAD_CENTER  LV_PCT(50)
 
 /**********************
  *      TYPEDEFS
  **********************/
 
 /**
- * Possible options how to blend opaque drawings
+ * Possible options for blending opaque drawings
  */
-enum _lv_blend_mode_t {
+typedef enum {
     LV_BLEND_MODE_NORMAL,     /**< Simply mix according to the opacity value*/
     LV_BLEND_MODE_ADDITIVE,   /**< Add the respective color channels*/
     LV_BLEND_MODE_SUBTRACTIVE,/**< Subtract the foreground from the background*/
     LV_BLEND_MODE_MULTIPLY,   /**< Multiply the foreground and background*/
-};
-
-#ifdef DOXYGEN
-typedef _lv_blend_mode_t lv_blend_mode_t;
-#else
-typedef uint8_t lv_blend_mode_t;
-#endif /*DOXYGEN*/
+    LV_BLEND_MODE_DIFFERENCE, /**< Absolute difference between foreground and background*/
+} lv_blend_mode_t;
 
 /**
  * Some options to apply decorations on texts.
  * 'OR'ed values can be used.
  */
-enum _lv_text_decor_t {
+typedef enum {
     LV_TEXT_DECOR_NONE          = 0x00,
     LV_TEXT_DECOR_UNDERLINE     = 0x01,
     LV_TEXT_DECOR_STRIKETHROUGH = 0x02,
-};
-
-#ifdef DOXYGEN
-typedef _lv_text_decor_t lv_text_decor_t;
-#else
-typedef uint8_t lv_text_decor_t;
-#endif /*DOXYGEN*/
+} lv_text_decor_t;
 
 /**
  * Selects on which sides border should be drawn
  * 'OR'ed values can be used.
  */
-enum _lv_border_side_t {
+typedef enum {
     LV_BORDER_SIDE_NONE     = 0x00,
     LV_BORDER_SIDE_BOTTOM   = 0x01,
     LV_BORDER_SIDE_TOP      = 0x02,
@@ -118,88 +116,7 @@ enum _lv_border_side_t {
     LV_BORDER_SIDE_RIGHT    = 0x08,
     LV_BORDER_SIDE_FULL     = 0x0F,
     LV_BORDER_SIDE_INTERNAL = 0x10, /**< FOR matrix-like objects (e.g. Button matrix)*/
-};
-
-#ifdef DOXYGEN
-typedef _lv_border_side_t lv_border_side_t;
-#else
-typedef uint8_t lv_border_side_t;
-#endif /*DOXYGEN*/
-
-/**
- * The direction of the gradient.
- */
-enum _lv_grad_dir_t {
-    LV_GRAD_DIR_NONE,       /**< No gradient (the `grad_color` property is ignored)*/
-    LV_GRAD_DIR_VER,        /**< Simple vertical (top to bottom) gradient*/
-    LV_GRAD_DIR_HOR,        /**< Simple horizontal (left to right) gradient*/
-    LV_GRAD_DIR_LINEAR,     /**< Linear gradient defined by start and end points. Can be at any angle.*/
-    LV_GRAD_DIR_RADIAL,     /**< Radial gradient defined by start and end circles*/
-    LV_GRAD_DIR_CONICAL,    /**< Conical gradient defined by center point, start and end angles*/
-};
-
-/**
- * Gradient behavior outside the defined range.
-*/
-enum _lv_grad_extend_t {
-    LV_GRAD_EXTEND_PAD,     /**< Repeat the same color*/
-    LV_GRAD_EXTEND_REPEAT,  /**< Repeat the pattern*/
-    LV_GRAD_EXTEND_REFLECT, /**< Repeat the pattern mirrored*/
-};
-
-#ifdef DOXYGEN
-typedef _lv_grad_dir_t lv_grad_dir_t;
-typedef _lv_grad_type_t lv_grad_type_t;
-typedef _lv_grad_extend_t lv_grad_extend_t;
-#else
-typedef uint8_t lv_grad_dir_t;
-typedef uint8_t lv_grad_type_t;
-typedef uint8_t lv_grad_extend_t;
-#endif /*DOXYGEN*/
-
-/** A gradient stop definition.
- *  This matches a color and a position in a virtual 0-255 scale.
- */
-typedef struct {
-    lv_color_t color;   /**< The stop color */
-    lv_opa_t   opa;     /**< The opacity of the color*/
-    uint8_t    frac;    /**< The stop position in 1/255 unit */
-} lv_gradient_stop_t;
-
-/** A descriptor of a gradient. */
-typedef struct {
-    lv_gradient_stop_t   stops[LV_GRADIENT_MAX_STOPS];  /**< A gradient stop array */
-    uint8_t              stops_count;                   /**< The number of used stops in the array */
-    lv_grad_dir_t        dir : 3;                       /**< The gradient direction.
-                                                         * Any of LV_GRAD_DIR_NONE, LV_GRAD_DIR_VER, LV_GRAD_DIR_HOR,
-                                                           LV_GRAD_TYPE_LINEAR, LV_GRAD_TYPE_RADIAL, LV_GRAD_TYPE_CONICAL */
-    lv_grad_extend_t     extend : 2;                    /**< Behaviour outside the defined range.
-                                                         * LV_GRAD_EXTEND_NONE, LV_GRAD_EXTEND_PAD, LV_GRAD_EXTEND_REPEAT, LV_GRAD_EXTEND_REFLECT */
-#if LV_USE_DRAW_SW_COMPLEX_GRADIENTS
-    union {
-        /*Linear gradient parameters*/
-        struct {
-            lv_point_t  start;                          /**< Linear gradient vector start point */
-            lv_point_t  end;                            /**< Linear gradient vector end point */
-        } linear;
-        /*Radial gradient parameters*/
-        struct {
-            lv_point_t  focal;                          /**< Center of the focal (starting) circle in local coordinates */
-            /* (can be the same as the ending circle to create concentric circles) */
-            lv_point_t  focal_extent;                   /**< Point on the circle (can be the same as the center) */
-            lv_point_t  end;                            /**< Center of the ending circle in local coordinates */
-            lv_point_t  end_extent;                     /**< Point on the circle determining the radius of the gradient */
-        } radial;
-        /*Conical gradient parameters*/
-        struct {
-            lv_point_t  center;                         /**< Conical gradient center point */
-            int16_t     start_angle;                    /**< Start angle 0..3600 */
-            int16_t     end_angle;                      /**< End angle 0..3600 */
-        } conical;
-    } params;
-    void * state;
-#endif
-} lv_grad_dsc_t;
+} lv_border_side_t;
 
 /**
  * A common type to handle all the property types in the same way.
@@ -215,7 +132,7 @@ typedef union {
  *
  * Props are split into groups of 16. When adding a new prop to a group, ensure it does not overflow into the next one.
  */
-enum _lv_style_prop_t {
+enum _lv_style_id_t {
     LV_STYLE_PROP_INV               = 0,
 
     /*Group 0*/
@@ -233,6 +150,8 @@ enum _lv_style_prop_t {
     LV_STYLE_ALIGN                  = 10,
 
     LV_STYLE_RADIUS                 = 12,
+    LV_STYLE_RADIAL_OFFSET          = 13,
+    LV_STYLE_PAD_RADIAL             = 14,
 
     /*Group 1*/
     LV_STYLE_PAD_TOP                = 16,
@@ -319,74 +238,64 @@ enum _lv_style_prop_t {
     LV_STYLE_TEXT_LINE_SPACE        = 92,
     LV_STYLE_TEXT_DECOR             = 93,
     LV_STYLE_TEXT_ALIGN             = 94,
+    LV_STYLE_TEXT_OUTLINE_STROKE_WIDTH = 95,
+    LV_STYLE_TEXT_OUTLINE_STROKE_OPA   = 96,
+    LV_STYLE_TEXT_OUTLINE_STROKE_COLOR = 97,
 
-    LV_STYLE_OPA                    = 95,
-    LV_STYLE_OPA_LAYERED            = 96,
-    LV_STYLE_COLOR_FILTER_DSC       = 97,
-    LV_STYLE_COLOR_FILTER_OPA       = 98,
-    LV_STYLE_ANIM                   = 99,
-    LV_STYLE_ANIM_DURATION          = 100,
-    LV_STYLE_TRANSITION             = 102,
-    LV_STYLE_BLEND_MODE             = 103,
-    LV_STYLE_TRANSFORM_WIDTH        = 104,
-    LV_STYLE_TRANSFORM_HEIGHT       = 105,
-    LV_STYLE_TRANSLATE_X            = 106,
-    LV_STYLE_TRANSLATE_Y            = 107,
-    LV_STYLE_TRANSFORM_SCALE_X      = 108,
-    LV_STYLE_TRANSFORM_SCALE_Y      = 109,
-    LV_STYLE_TRANSFORM_ROTATION     = 110,
-    LV_STYLE_TRANSFORM_PIVOT_X      = 111,
-    LV_STYLE_TRANSFORM_PIVOT_Y      = 112,
-    LV_STYLE_TRANSFORM_SKEW_X       = 113,
-    LV_STYLE_TRANSFORM_SKEW_Y       = 114,
-    LV_STYLE_BITMAP_MASK_SRC        = 115,
-    LV_STYLE_ROTARY_SENSITIVITY     = 116,
+    LV_STYLE_OPA                    = 98,
+    LV_STYLE_OPA_LAYERED            = 99,
+    LV_STYLE_COLOR_FILTER_DSC       = 100,
+    LV_STYLE_COLOR_FILTER_OPA       = 101,
+    LV_STYLE_ANIM                   = 102,
+    LV_STYLE_ANIM_DURATION          = 103,
+    LV_STYLE_TRANSITION             = 104,
+    LV_STYLE_BLEND_MODE             = 105,
+    LV_STYLE_TRANSFORM_WIDTH        = 106,
+    LV_STYLE_TRANSFORM_HEIGHT       = 107,
+    LV_STYLE_TRANSLATE_X            = 108,
+    LV_STYLE_TRANSLATE_Y            = 109,
+    LV_STYLE_TRANSFORM_SCALE_X      = 110,
+    LV_STYLE_TRANSFORM_SCALE_Y      = 111,
+    LV_STYLE_TRANSFORM_ROTATION     = 112,
+    LV_STYLE_TRANSFORM_PIVOT_X      = 113,
+    LV_STYLE_TRANSFORM_PIVOT_Y      = 114,
+    LV_STYLE_TRANSFORM_SKEW_X       = 115,
+    LV_STYLE_TRANSFORM_SKEW_Y       = 116,
+    LV_STYLE_BITMAP_MASK_SRC        = 117,
+    LV_STYLE_ROTARY_SENSITIVITY     = 118,
+    LV_STYLE_TRANSLATE_RADIAL       = 119,
+    LV_STYLE_RECOLOR                = 120,
+    LV_STYLE_RECOLOR_OPA            = 121,
 
-#if LV_USE_FLEX
-    LV_STYLE_FLEX_FLOW              = 125,
-    LV_STYLE_FLEX_MAIN_PLACE        = 126,
-    LV_STYLE_FLEX_CROSS_PLACE       = 127,
-    LV_STYLE_FLEX_TRACK_PLACE       = 128,
-    LV_STYLE_FLEX_GROW              = 129,
-#endif
+    LV_STYLE_FLEX_FLOW              = 122,
+    LV_STYLE_FLEX_MAIN_PLACE        = 123,
+    LV_STYLE_FLEX_CROSS_PLACE       = 124,
+    LV_STYLE_FLEX_TRACK_PLACE       = 125,
+    LV_STYLE_FLEX_GROW              = 126,
 
-#if LV_USE_GRID
-    LV_STYLE_GRID_COLUMN_ALIGN      = 130,
-    LV_STYLE_GRID_ROW_ALIGN         = 131,
-    LV_STYLE_GRID_ROW_DSC_ARRAY     = 132,
-    LV_STYLE_GRID_COLUMN_DSC_ARRAY  = 133,
-    LV_STYLE_GRID_CELL_COLUMN_POS   = 134,
-    LV_STYLE_GRID_CELL_COLUMN_SPAN  = 135,
-    LV_STYLE_GRID_CELL_X_ALIGN      = 136,
-    LV_STYLE_GRID_CELL_ROW_POS      = 137,
-    LV_STYLE_GRID_CELL_ROW_SPAN     = 138,
-    LV_STYLE_GRID_CELL_Y_ALIGN      = 139,
-#endif
+    LV_STYLE_GRID_COLUMN_ALIGN      = 127,
+    LV_STYLE_GRID_ROW_ALIGN         = 128,
+    LV_STYLE_GRID_ROW_DSC_ARRAY     = 129,
+    LV_STYLE_GRID_COLUMN_DSC_ARRAY  = 130,
+    LV_STYLE_GRID_CELL_COLUMN_POS   = 131,
+    LV_STYLE_GRID_CELL_COLUMN_SPAN  = 132,
+    LV_STYLE_GRID_CELL_X_ALIGN      = 133,
+    LV_STYLE_GRID_CELL_ROW_POS      = 134,
+    LV_STYLE_GRID_CELL_ROW_SPAN     = 135,
+    LV_STYLE_GRID_CELL_Y_ALIGN      = 136,
 
-    _LV_STYLE_LAST_BUILT_IN_PROP     = 140,
+    LV_STYLE_LAST_BUILT_IN_PROP     = 137,
 
-    _LV_STYLE_NUM_BUILT_IN_PROPS     = _LV_STYLE_LAST_BUILT_IN_PROP + 1,
+    LV_STYLE_NUM_BUILT_IN_PROPS     = LV_STYLE_LAST_BUILT_IN_PROP + 1,
 
     LV_STYLE_PROP_ANY                = 0xFF,
-    _LV_STYLE_PROP_CONST             = 0xFF /* magic value for const styles */
+    LV_STYLE_PROP_CONST             = 0xFF /* magic value for const styles */
 };
 
-#ifdef DOXYGEN
-typedef _lv_style_prop_t lv_style_prop_t;
-#else
-typedef uint8_t lv_style_prop_t;
-#endif /*DOXYGEN*/
-
-enum _lv_style_res_t {
+typedef enum {
     LV_STYLE_RES_NOT_FOUND,
     LV_STYLE_RES_FOUND,
-};
-
-#ifdef DOXYGEN
-typedef _lv_style_res_t lv_style_res_t;
-#else
-typedef uint8_t lv_style_res_t;
-#endif /*DOXYGEN*/
+} lv_style_res_t;
 
 /**
  * Descriptor for style transitions
@@ -394,7 +303,7 @@ typedef uint8_t lv_style_res_t;
 typedef struct {
     const lv_style_prop_t * props; /**< An array with the properties to animate.*/
     void * user_data;              /**< A custom user data that will be passed to the animation's user_data */
-    lv_anim_path_cb_t path_xcb;     /**< A path for the animation.*/
+    lv_anim_path_cb_t path_xcb;    /**< A path for the animation.*/
     uint32_t time;                 /**< Duration of the transition in [ms]*/
     uint32_t delay;                /**< Delay before the transition in [ms]*/
 } lv_style_transition_dsc_t;
@@ -403,7 +312,7 @@ typedef struct {
  * Descriptor of a constant style property.
  */
 typedef struct {
-    const lv_style_prop_t * prop_ptr;
+    lv_style_prop_t prop;
     lv_style_value_t value;
 } lv_style_const_prop_t;
 
@@ -441,6 +350,33 @@ void lv_style_init(lv_style_t * style);
  */
 void lv_style_reset(lv_style_t * style);
 
+/**
+ * Copy all properties of a style to an other.
+ * It has the same affect callying the same `lv_set_style_...`
+ * functions on both styles.
+ * It means new memory will be allocated to store the properties in
+ * the destination style.
+ * After the copy the destination style is fully independent of the source
+ * and source can removed without affecting the destination style.
+ * @param dst   the destination to copy into (can not the a constant style)
+ * @param src   the source style to copy from.
+ */
+void lv_style_copy(lv_style_t * dst, const lv_style_t * src);
+
+/**
+ * Copy all properties of a style to an other without resetting the dst style.
+ * It has the same effect as calling the same `lv_set_style_...`
+ * functions on both styles.
+ * It means new memory will be allocated to store the properties in
+ * the destination style.
+ * After the copy the destination style is fully independent of the source
+ * and source can removed without affecting the destination style.
+ * @param dst   the destination to copy into (cannot be a constant style)
+ * @param src   the source style to copy from.
+ */
+void lv_style_merge(lv_style_t * dst, const lv_style_t * src);
+
+
 /**
  * Check if a style is constant
  * @param style     pointer to a style
@@ -452,10 +388,13 @@ static inline bool lv_style_is_const(const lv_style_t * style)
     return false;
 }
 
+
 /**
  * Register a new style property for custom usage
  * @return a new property ID, or LV_STYLE_PROP_INV if there are no more available.
- * @example
+ *
+ * Example:
+ * @code
  * lv_style_prop_t MY_PROP;
  * static inline void lv_style_set_my_prop(lv_style_t * style, lv_color_t value) {
  * lv_style_value_t v = {.color = value}; lv_style_set_prop(style, MY_PROP, v); }
@@ -464,6 +403,7 @@ static inline bool lv_style_is_const(const lv_style_t * style)
  * MY_PROP = lv_style_register_prop();
  * ...
  * lv_style_set_my_prop(&style1, lv_palette_main(LV_PALETTE_RED));
+ * @endcode
  */
 lv_style_prop_t lv_style_register_prop(uint8_t flag);
 
@@ -509,10 +449,13 @@ lv_style_res_t lv_style_get_prop(const lv_style_t * style, lv_style_prop_t prop,
  * @param time      duration of the transition in [ms]
  * @param delay     delay before the transition in [ms]
  * @param user_data any custom data that will be saved in the transition animation and will be available when `path_cb` is called
- * @example
+ *
+ * Example:
+ * @code
  * const static lv_style_prop_t trans_props[] = { LV_STYLE_BG_OPA, LV_STYLE_BG_COLOR, 0 };
- *  static lv_style_transition_dsc_t trans1;
- *  lv_style_transition_dsc_init(&trans1, trans_props, NULL, 300, 0, NULL);
+ * static lv_style_transition_dsc_t trans1;
+ * lv_style_transition_dsc_init(&trans1, trans_props, NULL, 300, 0, NULL);
+ * @endcode
  */
 void lv_style_transition_dsc_init(lv_style_transition_dsc_t * tr, const lv_style_prop_t props[],
                                   lv_anim_path_cb_t path_cb, uint32_t time, uint32_t delay, void * user_data);
@@ -540,8 +483,8 @@ static inline lv_style_res_t lv_style_get_prop_inlined(const lv_style_t * style,
     if(lv_style_is_const(style)) {
         lv_style_const_prop_t * props = (lv_style_const_prop_t *)style->values_and_props;
         uint32_t i;
-        for(i = 0; props[i].prop_ptr; i++) {
-            if(*props[i].prop_ptr == prop) {
+        for(i = 0; props[i].prop != LV_STYLE_PROP_INV; i++) {
+            if(props[i].prop == prop) {
                 *value = props[i].value;
                 return LV_STYLE_RES_FOUND;
             }
@@ -574,7 +517,7 @@ bool lv_style_is_empty(const lv_style_t * style);
  * @param prop a style property
  * @return the group [0..30] 30 means all the custom properties with index > 120
  */
-static inline uint32_t _lv_style_get_prop_group(lv_style_prop_t prop)
+static inline uint32_t lv_style_get_prop_group(lv_style_prop_t prop)
 {
     uint32_t group = prop >> 2;
     if(group > 30) group = 31;    /*The MSB marks all the custom properties*/
@@ -588,7 +531,7 @@ static inline uint32_t _lv_style_get_prop_group(lv_style_prop_t prop)
  * @param prop a style property
  * @return the flags of the property
  */
-uint8_t _lv_style_prop_lookup_flags(lv_style_prop_t prop);
+uint8_t lv_style_prop_lookup_flags(lv_style_prop_t prop);
 
 #include "lv_style_gen.h"
 
@@ -624,6 +567,26 @@ static inline void lv_style_set_pad_gap(lv_style_t * style, int32_t value)
     lv_style_set_pad_column(style, value);
 }
 
+static inline void lv_style_set_margin_hor(lv_style_t * style, int32_t value)
+{
+    lv_style_set_margin_left(style, value);
+    lv_style_set_margin_right(style, value);
+}
+
+static inline void lv_style_set_margin_ver(lv_style_t * style, int32_t value)
+{
+    lv_style_set_margin_top(style, value);
+    lv_style_set_margin_bottom(style, value);
+}
+
+static inline void lv_style_set_margin_all(lv_style_t * style, int32_t value)
+{
+    lv_style_set_margin_left(style, value);
+    lv_style_set_margin_right(style, value);
+    lv_style_set_margin_top(style, value);
+    lv_style_set_margin_bottom(style, value);
+}
+
 static inline void lv_style_set_transform_scale(lv_style_t * style, int32_t value)
 {
     lv_style_set_transform_scale_x(style, value);
@@ -642,7 +605,7 @@ static inline void lv_style_set_transform_scale(lv_style_t * style, int32_t valu
  */
 static inline bool lv_style_prop_has_flag(lv_style_prop_t prop, uint8_t flag)
 {
-    return _lv_style_prop_lookup_flags(prop) & flag;
+    return lv_style_prop_lookup_flags(prop) & flag;
 }
 
 /*************************
