@@ -88,37 +88,48 @@
 /**********************
  *   GLOBAL FUNCTIONS
  **********************/
-
-void lv_text_get_size(lv_point_t * size_res, const char * text, const lv_font_t * font, int32_t letter_space,
-                      int32_t line_space, int32_t max_width, lv_text_flag_t flag)
+void lv_text_attributes_init(lv_text_attributes_t * attributes)
 {
+    lv_memzero(attributes, sizeof(lv_text_attributes_t));
+}
+
+void lv_text_get_size(lv_point_t * size_res, const char * text, const lv_font_t * font,
+                      lv_text_attributes_t * attributes)
+{
+    uint32_t line_start     = 0;
+    uint32_t new_line_start = 0;
+    uint16_t letter_height  = 0;
     size_res->x = 0;
     size_res->y = 0;
 
-    if(text == NULL) return;
-    if(font == NULL) return;
+    LV_ASSERT_NULL(attributes);
+    LV_ASSERT_NULL(font);
+    LV_ASSERT_NULL(text);
 
-    if(flag & LV_TEXT_FLAG_EXPAND) max_width = LV_COORD_MAX;
+    letter_height = lv_font_get_line_height(font);
 
-    uint32_t line_start     = 0;
-    uint32_t new_line_start = 0;
-    uint16_t letter_height = lv_font_get_line_height(font);
+    if(attributes->text_flags & LV_TEXT_FLAG_EXPAND) {
+        attributes->max_width = LV_COORD_MAX;
+    }
 
     /*Calc. the height and longest line*/
     while(text[line_start] != '\0') {
-        new_line_start += lv_text_get_next_line(&text[line_start], font, letter_space, max_width, NULL, flag);
+        new_line_start += lv_text_get_next_line(
+                              &text[line_start], LV_TEXT_LEN_MAX, font, NULL, attributes);
 
-        if((unsigned long)size_res->y + (unsigned long)letter_height + (unsigned long)line_space > LV_MAX_OF(int32_t)) {
+        if((unsigned long)size_res->y +
+           (unsigned long)letter_height + (unsigned long)attributes->line_space > LV_MAX_OF(int32_t)) {
             LV_LOG_WARN("integer overflow while calculating text height");
             return;
         }
         else {
             size_res->y += letter_height;
-            size_res->y += line_space;
+            size_res->y += attributes->line_space;
         }
 
         /*Calculate the longest line*/
-        int32_t act_line_length = lv_text_get_width(&text[line_start], new_line_start - line_start, font, letter_space);
+        int32_t act_line_length = lv_text_get_width(
+                                      &text[line_start], new_line_start - line_start, font, attributes);
 
         size_res->x = LV_MAX(act_line_length, size_res->x);
         line_start  = new_line_start;
@@ -126,14 +137,45 @@ void lv_text_get_size(lv_point_t * size_res, const char * text, const lv_font_t
 
     /*Make the text one line taller if the last character is '\n' or '\r'*/
     if((line_start != 0) && (text[line_start - 1] == '\n' || text[line_start - 1] == '\r')) {
-        size_res->y += letter_height + line_space;
+        size_res->y += letter_height + attributes->line_space;
     }
 
     /*Correction with the last line space or set the height manually if the text is empty*/
     if(size_res->y == 0)
         size_res->y = letter_height;
     else
-        size_res->y -= line_space;
+        size_res->y -= attributes->line_space;
+}
+
+bool lv_text_is_cmd(lv_text_cmd_state_t * state, uint32_t c)
+{
+    bool ret = false;
+
+    if(c == (uint32_t)LV_TXT_COLOR_CMD[0]) {
+        if(*state == LV_TEXT_CMD_STATE_WAIT) { /*Start char*/
+            *state = LV_TEXT_CMD_STATE_PAR;
+            ret = true;
+        }
+        /*Other start char in parameter is escaped cmd. char*/
+        else if(*state == LV_TEXT_CMD_STATE_WAIT) {
+            *state = LV_TEXT_CMD_STATE_WAIT;
+        }
+        /*Command end*/
+        else if(*state == LV_TEXT_CMD_STATE_IN) {
+            *state = LV_TEXT_CMD_STATE_WAIT;
+            ret = true;
+        }
+    }
+
+    /*Skip the color parameter and wait the space after it*/
+    if(*state == LV_TEXT_CMD_STATE_PAR) {
+        if(c == ' ') {
+            *state = LV_TEXT_CMD_STATE_IN; /*After the parameter the text is in the command*/
+        }
+        ret = true;
+    }
+
+    return ret;
 }
 
 /**
@@ -164,12 +206,13 @@ void lv_text_get_size(lv_point_t * size_res, const char * text, const lv_font_t
  * @param max_width max width of the text (break the lines to fit this size). Set COORD_MAX to avoid line breaks
  * @param flags settings for the text from 'txt_flag_type' enum
  * @param[out] word_w_ptr width (in pixels) of the parsed word. May be NULL.
- * @param force Force return the fraction of the word that can fit in the provided space.
+ * @param cmd_state Pointer to a lv_text_cmd_state_t variable which stored the current state of command processing
  * @return the index of the first char of the next word (in byte index not letter index. With UTF-8 they are different)
  */
 static uint32_t lv_text_get_next_word(const char * txt, const lv_font_t * font,
                                       int32_t letter_space, int32_t max_width,
-                                      lv_text_flag_t flag, uint32_t * word_w_ptr, bool force)
+                                      lv_text_flag_t flag, uint32_t * word_w_ptr,
+                                      lv_text_cmd_state_t * cmd_state)
 {
     if(txt == NULL || txt[0] == '\0') return 0;
     if(font == NULL) return 0;
@@ -180,8 +223,8 @@ static uint32_t lv_text_get_next_word(const char * txt, const lv_font_t * font,
     uint32_t letter = 0;      /*Letter at i*/
     uint32_t letter_next = 0; /*Letter at i_next*/
     int32_t letter_w;
-    int32_t cur_w = 0;  /*Pixel Width of transversed string*/
-    uint32_t word_len = 0;   /*Number of characters in the transversed word*/
+    int32_t cur_w = 0;  /*Pixel Width of traversed string*/
+    uint32_t word_len = 0;   /*Number of characters in the traversed word*/
     uint32_t break_index = NO_BREAK_FOUND; /*only used for "long" words*/
     uint32_t break_letter_count = 0; /*Number of characters up to the long word break point*/
 
@@ -193,6 +236,16 @@ static uint32_t lv_text_get_next_word(const char * txt, const lv_font_t * font,
         letter_next = lv_text_encoded_next(txt, &i_next_next);
         word_len++;
 
+        /*Handle the recolor command*/
+        if((flag & LV_TEXT_FLAG_RECOLOR) != 0) {
+            if(lv_text_is_cmd(cmd_state, letter)) {
+                i = i_next;
+                i_next = i_next_next;
+                letter = letter_next;
+                continue;   /*Skip the letter if it is part of a command*/
+            }
+        }
+
         letter_w = lv_font_get_glyph_width(font, letter, letter_next);
         cur_w += letter_w;
 
@@ -204,6 +257,9 @@ static uint32_t lv_text_get_next_word(const char * txt, const lv_font_t * font,
         if(break_index == NO_BREAK_FOUND && (cur_w - letter_space) > max_width) {
             break_index = i;
             break_letter_count = word_len - 1;
+            if(flag & LV_TEXT_FLAG_BREAK_ALL) {
+                break;
+            }
             /*break_index is now pointing at the character that doesn't fit*/
         }
 
@@ -239,14 +295,14 @@ static uint32_t lv_text_get_next_word(const char * txt, const lv_font_t * font,
 #if LV_TXT_LINE_BREAK_LONG_LEN > 0
     /*Word doesn't fit in provided space, but isn't "long"*/
     if(word_len < LV_TXT_LINE_BREAK_LONG_LEN) {
-        if(force) return break_index;
+        if(flag & LV_TEXT_FLAG_BREAK_ALL) return break_index;
         if(word_w_ptr != NULL) *word_w_ptr = 0; /*Return no word*/
         return 0;
     }
 
     /*Word is "long," but insufficient amounts can fit in provided space*/
     if(break_letter_count < LV_TXT_LINE_BREAK_LONG_PRE_MIN_LEN) {
-        if(force) return break_index;
+        if(flag & LV_TEXT_FLAG_BREAK_ALL) return break_index;
         if(word_w_ptr != NULL) *word_w_ptr = 0;
         return 0;
     }
@@ -266,17 +322,17 @@ static uint32_t lv_text_get_next_word(const char * txt, const lv_font_t * font,
     }
     return i;
 #else
-    if(force) return break_index;
+    if(flag & LV_TEXT_FLAG_BREAK_ALL) return break_index;
     if(word_w_ptr != NULL) *word_w_ptr = 0; /*Return no word*/
     (void) break_letter_count;
     return 0;
 #endif
 }
 
-uint32_t lv_text_get_next_line(const char * txt, const lv_font_t * font,
-                               int32_t letter_space, int32_t max_width,
-                               int32_t * used_width, lv_text_flag_t flag)
+uint32_t lv_text_get_next_line(const char * txt, uint32_t len,
+                               const lv_font_t * font, int32_t * used_width, lv_text_attributes_t * attributes)
 {
+
     if(used_width) *used_width = 0;
 
     if(txt == NULL) return 0;
@@ -287,22 +343,34 @@ uint32_t lv_text_get_next_line(const char * txt, const lv_font_t * font,
 
     /*If max_width doesn't matter simply find the new line character
      *without thinking about word wrapping*/
-    if((flag & LV_TEXT_FLAG_EXPAND) || (flag & LV_TEXT_FLAG_FIT)) {
+    if((attributes->text_flags & LV_TEXT_FLAG_EXPAND) ||
+       (attributes->text_flags & LV_TEXT_FLAG_FIT)) {
+
         uint32_t i;
-        for(i = 0; txt[i] != '\n' && txt[i] != '\r' && txt[i] != '\0'; i++) {
+        for(i = 0; i < len && txt[i] != '\n' && txt[i] != '\r' && txt[i] != '\0'; i++) {
             /*Just find the new line chars or string ends by incrementing `i`*/
         }
-        if(txt[i] != '\0') i++;    /*To go beyond `\n`*/
+        if(i < len && txt[i] != '\0') i++;    /*To go beyond `\n`*/
         if(used_width) *used_width = -1;
         return i;
     }
 
-    if(flag & LV_TEXT_FLAG_EXPAND) max_width = LV_COORD_MAX;
+    if(attributes->text_flags & LV_TEXT_FLAG_EXPAND) {
+        attributes->max_width = LV_COORD_MAX;
+    }
+    lv_text_cmd_state_t cmd_state = LV_TEXT_CMD_STATE_WAIT;
+
     uint32_t i = 0;                                        /*Iterating index into txt*/
+    uint32_t max_width = attributes->max_width;
+
+    while(i < len && txt[i] != '\0' && max_width > 0) {
+        lv_text_flag_t word_flag = attributes->text_flags;
+
+        if(i == 0) word_flag |= LV_TEXT_FLAG_BREAK_ALL;
 
-    while(txt[i] != '\0' && max_width > 0) {
         uint32_t word_w = 0;
-        uint32_t advance = lv_text_get_next_word(&txt[i], font, letter_space, max_width, flag, &word_w, i == 0);
+        uint32_t advance = lv_text_get_next_word(&txt[i], font, attributes->letter_space,
+                                                 max_width, word_flag, &word_w, &cmd_state);
         max_width -= word_w;
         line_w += word_w;
 
@@ -318,7 +386,6 @@ uint32_t lv_text_get_next_line(const char * txt, const lv_font_t * font,
             i++;  /*Include the following newline in the current line*/
             break;
         }
-
     }
 
     /*Always step at least one to avoid infinite loops*/
@@ -336,30 +403,40 @@ uint32_t lv_text_get_next_line(const char * txt, const lv_font_t * font,
     return i;
 }
 
-int32_t lv_text_get_width(const char * txt, uint32_t length, const lv_font_t * font, int32_t letter_space)
+int32_t lv_text_get_width(const char * txt, uint32_t length, const lv_font_t * font,
+                          const lv_text_attributes_t * attributes)
 {
     if(txt == NULL) return 0;
     if(font == NULL) return 0;
     if(txt[0] == '\0') return 0;
 
-    uint32_t i                   = 0;
+    uint32_t i                = 0;
     int32_t width             = 0;
+    lv_text_cmd_state_t cmd_state = LV_TEXT_CMD_STATE_WAIT;
 
     if(length != 0) {
-        while(i < length) {
+        while(txt[i] != '\0' && i < length) {
+
             uint32_t letter;
             uint32_t letter_next;
+
             lv_text_encoded_letter_next_2(txt, &letter, &letter_next, &i);
 
+            if((attributes->text_flags & LV_TEXT_FLAG_RECOLOR) != 0) {
+                if(lv_text_is_cmd(&cmd_state, letter) != false) {
+                    continue;
+                }
+            }
+
             int32_t char_width = lv_font_get_glyph_width(font, letter, letter_next);
             if(char_width > 0) {
                 width += char_width;
-                width += letter_space;
+                width += attributes->letter_space;
             }
         }
 
         if(width > 0) {
-            width -= letter_space; /*Trim the last letter space. Important if the text is center
+            width -= attributes->letter_space; /*Trim the last letter space. Important if the text is center
                                       aligned*/
         }
     }
@@ -424,13 +501,13 @@ char * lv_text_set_text_vfmt(const char * fmt, va_list ap)
     lv_vsnprintf(raw_txt, len + 1, fmt, ap);
 
     /*Get the size of the Arabic text and process it*/
-    size_t len_ap = _lv_text_ap_calc_bytes_count(raw_txt);
+    size_t len_ap = lv_text_ap_calc_bytes_count(raw_txt);
     text = lv_malloc(len_ap + 1);
     LV_ASSERT_MALLOC(text);
     if(text == NULL) {
         return NULL;
     }
-    _lv_text_ap_proc(raw_txt, text);
+    lv_text_ap_proc(raw_txt, text);
 
     lv_free(raw_txt);
 #else
@@ -560,6 +637,11 @@ static uint32_t lv_text_utf8_next(const char * txt, uint32_t * i)
     uint32_t i_tmp = 0;
     if(i == NULL) i = &i_tmp;
 
+    /* Ensure the string is not null */
+    if(txt == NULL || txt[*i] == '\0') {
+        return result;
+    }
+
     /*Normal ASCII*/
     if(LV_IS_ASCII(txt[*i])) {
         result = txt[*i];
@@ -712,7 +794,7 @@ static uint32_t lv_text_utf8_get_length(const char * txt)
 /**
  * Give the size of an ISO8859-1 coded character
  * @param str pointer to a character in a string
- * @return length of the UTF-8 character (1,2,3 or 4). O on invalid code
+ * @return length of the ISO8859-1 coded character, will be always 1.
  */
 static uint8_t lv_text_iso8859_1_size(const char * str)
 {
@@ -748,9 +830,9 @@ static uint32_t lv_text_iso8859_1_conv_wc(uint32_t c)
  * Decode an ISO8859-1 character from a string.
  * @param txt pointer to '\0' terminated string
  * @param i start byte index in 'txt' where to start.
- *          After call it will point to the next UTF-8 char in 'txt'.
+ *          After call it will point to the next ISO8859-1 coded char in 'txt'.
  *          NULL to use txt[0] as index
- * @return the decoded Unicode character or 0 on invalid UTF-8 code
+ * @return the decoded ISO8859-1 character.
  */
 static uint32_t lv_text_iso8859_1_next(const char * txt, uint32_t * i)
 {
@@ -764,8 +846,8 @@ static uint32_t lv_text_iso8859_1_next(const char * txt, uint32_t * i)
 /**
  * Get previous ISO8859-1 character form a string.
  * @param txt pointer to '\0' terminated string
- * @param i start byte index in 'txt' where to start. After the call it will point to the previous UTF-8 char in 'txt'.
- * @return the decoded Unicode character or 0 on invalid UTF-8 code
+ * @param i start byte index in 'txt' where to start. After the call it will point to the previous ISO8859-1 coded char in 'txt'.
+ * @return the decoded ISO8859-1 character.
  */
 static uint32_t lv_text_iso8859_1_prev(const char * txt, uint32_t * i)
 {
@@ -779,8 +861,8 @@ static uint32_t lv_text_iso8859_1_prev(const char * txt, uint32_t * i)
 
 /**
  * Convert a character index (in an ISO8859-1 text) to byte index.
- * E.g. in "AÁRT" index of 'R' is 2th char but start at byte 3 because 'Á' is 2 bytes long
- * @param txt a '\0' terminated UTF-8 string
+ * The ISO8859-1 encoding is compatible with ASCII so the indices of characters is the same as the indices of bytes.
+ * @param txt a '\0' terminated char string
  * @param utf8_id character index
  * @return byte index of the 'utf8_id'th letter
  */
@@ -792,8 +874,8 @@ static uint32_t lv_text_iso8859_1_get_byte_id(const char * txt, uint32_t utf8_id
 
 /**
  * Convert a byte index (in an ISO8859-1 text) to character index.
- * E.g. in "AÁRT" index of 'R' is 2th char but start at byte 3 because 'Á' is 2 bytes long
- * @param txt a '\0' terminated UTF-8 string
+ * The ISO8859-1 encoding is compatible with ASCII so the indices of characters is the same as the indices of bytes.
+ * @param txt a '\0' terminated char string
  * @param byte_id byte index
  * @return character index of the letter at 'byte_id'th position
  */
@@ -804,8 +886,8 @@ static uint32_t lv_text_iso8859_1_get_char_id(const char * txt, uint32_t byte_id
 }
 
 /**
- * Get the number of characters (and NOT bytes) in a string. Decode it with UTF-8 if enabled.
- * E.g.: "ÁBC" is 3 characters (but 4 bytes)
+ * Get the number of characters (and NOT bytes) in a string.
+ * The ISO8859-1 encoding is compatible with ASCII so the number of characters is the same as the number of bytes.
  * @param txt a '\0' terminated char string
  * @return number of characters
  */
