@@ -10,7 +10,7 @@
 
 #if LV_USE_THEME_DEFAULT
 
-#include "../lv_theme.h"
+#include "../lv_theme_private.h"
 #include "../../misc/lv_color.h"
 #include "../../core/lv_global.h"
 
@@ -24,7 +24,7 @@ typedef struct _my_theme_t my_theme_t;
 #define theme_def (*(my_theme_t **)(&LV_GLOBAL_DEFAULT()->theme_default))
 
 #define MODE_DARK 1
-#define RADIUS_DEFAULT _LV_DPX_CALC(theme->disp_dpi, theme->disp_size == DISP_LARGE ? 12 : 8)
+#define RADIUS_DEFAULT LV_DPX_CALC(theme->disp_dpi, theme->disp_size == DISP_LARGE ? 12 : 8)
 
 /*SCREEN*/
 #define LIGHT_COLOR_SCR        lv_palette_lighten(LV_PALETTE_GREY, 4)
@@ -37,12 +37,12 @@ typedef struct _my_theme_t my_theme_t;
 #define DARK_COLOR_GREY        lv_color_hex(0x2f3237)
 
 #define TRANSITION_TIME         LV_THEME_DEFAULT_TRANSITION_TIME
-#define BORDER_WIDTH            _LV_DPX_CALC(theme->disp_dpi, 2)
-#define OUTLINE_WIDTH           _LV_DPX_CALC(theme->disp_dpi, 3)
+#define BORDER_WIDTH            LV_DPX_CALC(theme->disp_dpi, 2)
+#define OUTLINE_WIDTH           LV_DPX_CALC(theme->disp_dpi, 3)
 
-#define PAD_DEF     _LV_DPX_CALC(theme->disp_dpi, theme->disp_size == DISP_LARGE ? 24 : theme->disp_size == DISP_MEDIUM ? 20 : 16)
-#define PAD_SMALL   _LV_DPX_CALC(theme->disp_dpi, theme->disp_size == DISP_LARGE ? 14 : theme->disp_size == DISP_MEDIUM ? 12 : 10)
-#define PAD_TINY    _LV_DPX_CALC(theme->disp_dpi, theme->disp_size == DISP_LARGE ? 8 : theme->disp_size == DISP_MEDIUM ? 6 : 2)
+#define PAD_DEF     LV_DPX_CALC(theme->disp_dpi, theme->disp_size == DISP_LARGE ? 24 : theme->disp_size == DISP_MEDIUM ? 20 : 16)
+#define PAD_SMALL   LV_DPX_CALC(theme->disp_dpi, theme->disp_size == DISP_LARGE ? 14 : theme->disp_size == DISP_MEDIUM ? 12 : 10)
+#define PAD_TINY    LV_DPX_CALC(theme->disp_dpi, theme->disp_size == DISP_LARGE ? 8 : theme->disp_size == DISP_MEDIUM ? 6 : 2)
 
 /**********************
  *      TYPEDEFS
@@ -170,9 +170,6 @@ struct _my_theme_t {
     bool inited;
     my_theme_styles_t styles;
 
-    lv_color_filter_dsc_t dark_filter;
-    lv_color_filter_dsc_t grey_filter;
-
 #if LV_THEME_DEFAULT_TRANSITION_TIME
     lv_style_transition_dsc_t trans_delayed;
     lv_style_transition_dsc_t trans_normal;
@@ -182,8 +179,8 @@ struct _my_theme_t {
 /**********************
  *  STATIC PROTOTYPES
  **********************/
-static void theme_apply(lv_theme_t * th, lv_obj_t * obj);
 static void style_init_reset(lv_style_t * style);
+static void theme_apply(lv_theme_t * th, lv_obj_t * obj);
 
 /**********************
  *  STATIC VARIABLES
@@ -197,19 +194,6 @@ static void style_init_reset(lv_style_t * style);
  *   STATIC FUNCTIONS
  **********************/
 
-static lv_color_t dark_color_filter_cb(const lv_color_filter_dsc_t * f, lv_color_t c, lv_opa_t opa)
-{
-    LV_UNUSED(f);
-    return lv_color_darken(c, opa);
-}
-
-static lv_color_t grey_filter_cb(const lv_color_filter_dsc_t * f, lv_color_t color, lv_opa_t opa)
-{
-    LV_UNUSED(f);
-    if(theme_def->base.flags & MODE_DARK) return lv_color_mix(lv_palette_darken(LV_PALETTE_GREY, 2), color, opa);
-    else return lv_color_mix(lv_palette_lighten(LV_PALETTE_GREY, 2), color, opa);
-}
-
 static void style_init(my_theme_t * theme)
 {
 #if TRANSITION_TIME
@@ -219,7 +203,7 @@ static void style_init(my_theme_t * theme)
         LV_STYLE_TRANSLATE_Y, LV_STYLE_TRANSLATE_X,
         LV_STYLE_TRANSFORM_ROTATION,
         LV_STYLE_TRANSFORM_SCALE_X, LV_STYLE_TRANSFORM_SCALE_Y,
-        LV_STYLE_COLOR_FILTER_OPA, LV_STYLE_COLOR_FILTER_DSC,
+        LV_STYLE_RECOLOR_OPA, LV_STYLE_RECOLOR,
         0
     };
 #endif
@@ -247,8 +231,8 @@ static void style_init(my_theme_t * theme)
     lv_style_set_bg_color(&theme->styles.scrollbar, sb_color);
 
     lv_style_set_radius(&theme->styles.scrollbar, LV_RADIUS_CIRCLE);
-    lv_style_set_pad_all(&theme->styles.scrollbar, _LV_DPX_CALC(theme->disp_dpi, 7));
-    lv_style_set_width(&theme->styles.scrollbar,  _LV_DPX_CALC(theme->disp_dpi, 5));
+    lv_style_set_pad_all(&theme->styles.scrollbar, LV_DPX_CALC(theme->disp_dpi, 7));
+    lv_style_set_width(&theme->styles.scrollbar,  LV_DPX_CALC(theme->disp_dpi, 5));
     lv_style_set_bg_opa(&theme->styles.scrollbar,  LV_OPA_40);
 #if TRANSITION_TIME
     lv_style_set_transition(&theme->styles.scrollbar, &theme->trans_normal);
@@ -278,7 +262,7 @@ static void style_init(my_theme_t * theme)
     lv_style_set_pad_row(&theme->styles.card, PAD_SMALL);
     lv_style_set_pad_column(&theme->styles.card, PAD_SMALL);
     lv_style_set_line_color(&theme->styles.card, lv_palette_main(LV_PALETTE_GREY));
-    lv_style_set_line_width(&theme->styles.card, _LV_DPX_CALC(theme->disp_dpi, 1));
+    lv_style_set_line_width(&theme->styles.card, LV_DPX_CALC(theme->disp_dpi, 1));
 
     style_init_reset(&theme->styles.outline_primary);
     lv_style_set_outline_color(&theme->styles.outline_primary, theme->base.color_primary);
@@ -293,31 +277,31 @@ static void style_init(my_theme_t * theme)
 
     style_init_reset(&theme->styles.btn);
     lv_style_set_radius(&theme->styles.btn,
-                        _LV_DPX_CALC(theme->disp_dpi, theme->disp_size == DISP_LARGE ? 16 : theme->disp_size == DISP_MEDIUM ? 12 : 8));
+                        LV_DPX_CALC(theme->disp_dpi, theme->disp_size == DISP_LARGE ? 16 : theme->disp_size == DISP_MEDIUM ? 12 : 8));
     lv_style_set_bg_opa(&theme->styles.btn, LV_OPA_COVER);
     lv_style_set_bg_color(&theme->styles.btn, theme->color_grey);
     if(!(theme->base.flags & MODE_DARK)) {
         lv_style_set_shadow_color(&theme->styles.btn, lv_palette_main(LV_PALETTE_GREY));
         lv_style_set_shadow_width(&theme->styles.btn, LV_DPX(3));
         lv_style_set_shadow_opa(&theme->styles.btn, LV_OPA_50);
-        lv_style_set_shadow_offset_y(&theme->styles.btn, _LV_DPX_CALC(theme->disp_dpi, LV_DPX(4)));
+        lv_style_set_shadow_offset_y(&theme->styles.btn, LV_DPX_CALC(theme->disp_dpi, LV_DPX(4)));
     }
     lv_style_set_text_color(&theme->styles.btn, theme->color_text);
     lv_style_set_pad_hor(&theme->styles.btn, PAD_DEF);
     lv_style_set_pad_ver(&theme->styles.btn, PAD_SMALL);
-    lv_style_set_pad_column(&theme->styles.btn, _LV_DPX_CALC(theme->disp_dpi, 5));
-    lv_style_set_pad_row(&theme->styles.btn, _LV_DPX_CALC(theme->disp_dpi, 5));
-
-    lv_color_filter_dsc_init(&theme->dark_filter, dark_color_filter_cb);
-    lv_color_filter_dsc_init(&theme->grey_filter, grey_filter_cb);
+    lv_style_set_pad_column(&theme->styles.btn, LV_DPX_CALC(theme->disp_dpi, 5));
+    lv_style_set_pad_row(&theme->styles.btn, LV_DPX_CALC(theme->disp_dpi, 5));
 
     style_init_reset(&theme->styles.pressed);
-    lv_style_set_color_filter_dsc(&theme->styles.pressed, &theme->dark_filter);
-    lv_style_set_color_filter_opa(&theme->styles.pressed, 35);
+    lv_style_set_recolor(&theme->styles.pressed, lv_color_black());
+    lv_style_set_recolor_opa(&theme->styles.pressed, 35);
 
     style_init_reset(&theme->styles.disabled);
-    lv_style_set_color_filter_dsc(&theme->styles.disabled, &theme->grey_filter);
-    lv_style_set_color_filter_opa(&theme->styles.disabled, LV_OPA_50);
+    if(theme_def->base.flags & MODE_DARK)
+        lv_style_set_recolor(&theme->styles.disabled, lv_palette_darken(LV_PALETTE_GREY, 2));
+    else
+        lv_style_set_recolor(&theme->styles.disabled, lv_palette_lighten(LV_PALETTE_GREY, 2));
+    lv_style_set_recolor_opa(&theme->styles.disabled, LV_OPA_50);
 
     style_init_reset(&theme->styles.clip_corner);
     lv_style_set_clip_corner(&theme->styles.clip_corner, true);
@@ -333,11 +317,11 @@ static void style_init(my_theme_t * theme)
     lv_style_set_pad_gap(&theme->styles.pad_small, PAD_SMALL);
 
     style_init_reset(&theme->styles.pad_gap);
-    lv_style_set_pad_row(&theme->styles.pad_gap, _LV_DPX_CALC(theme->disp_dpi, 10));
-    lv_style_set_pad_column(&theme->styles.pad_gap, _LV_DPX_CALC(theme->disp_dpi, 10));
+    lv_style_set_pad_row(&theme->styles.pad_gap, LV_DPX_CALC(theme->disp_dpi, 10));
+    lv_style_set_pad_column(&theme->styles.pad_gap, LV_DPX_CALC(theme->disp_dpi, 10));
 
     style_init_reset(&theme->styles.line_space_large);
-    lv_style_set_text_line_space(&theme->styles.line_space_large, _LV_DPX_CALC(theme->disp_dpi, 20));
+    lv_style_set_text_line_space(&theme->styles.line_space_large, LV_DPX_CALC(theme->disp_dpi, 20));
 
     style_init_reset(&theme->styles.text_align_center);
     lv_style_set_text_align(&theme->styles.text_align_center, LV_TEXT_ALIGN_CENTER);
@@ -393,14 +377,14 @@ static void style_init(my_theme_t * theme)
 
 #if LV_THEME_DEFAULT_GROW
     style_init_reset(&theme->styles.grow);
-    lv_style_set_transform_width(&theme->styles.grow, _LV_DPX_CALC(theme->disp_dpi, 3));
-    lv_style_set_transform_height(&theme->styles.grow, _LV_DPX_CALC(theme->disp_dpi, 3));
+    lv_style_set_transform_width(&theme->styles.grow, LV_DPX_CALC(theme->disp_dpi, 3));
+    lv_style_set_transform_height(&theme->styles.grow, LV_DPX_CALC(theme->disp_dpi, 3));
 #endif
 
     style_init_reset(&theme->styles.knob);
     lv_style_set_bg_color(&theme->styles.knob, theme->base.color_primary);
     lv_style_set_bg_opa(&theme->styles.knob, LV_OPA_COVER);
-    lv_style_set_pad_all(&theme->styles.knob, _LV_DPX_CALC(theme->disp_dpi, 6));
+    lv_style_set_pad_all(&theme->styles.knob, LV_DPX_CALC(theme->disp_dpi, 6));
     lv_style_set_radius(&theme->styles.knob, LV_RADIUS_CIRCLE);
 
     style_init_reset(&theme->styles.anim);
@@ -412,7 +396,7 @@ static void style_init(my_theme_t * theme)
 #if LV_USE_ARC
     style_init_reset(&theme->styles.arc_indic);
     lv_style_set_arc_color(&theme->styles.arc_indic, theme->color_grey);
-    lv_style_set_arc_width(&theme->styles.arc_indic, _LV_DPX_CALC(theme->disp_dpi, 15));
+    lv_style_set_arc_width(&theme->styles.arc_indic, LV_DPX_CALC(theme->disp_dpi, 15));
     lv_style_set_arc_rounded(&theme->styles.arc_indic, true);
 
     style_init_reset(&theme->styles.arc_indic_primary);
@@ -425,22 +409,22 @@ static void style_init(my_theme_t * theme)
 #endif
 #if LV_USE_CHECKBOX
     style_init_reset(&theme->styles.cb_marker);
-    lv_style_set_pad_all(&theme->styles.cb_marker, _LV_DPX_CALC(theme->disp_dpi, 3));
+    lv_style_set_pad_all(&theme->styles.cb_marker, LV_DPX_CALC(theme->disp_dpi, 3));
     lv_style_set_border_width(&theme->styles.cb_marker, BORDER_WIDTH);
     lv_style_set_border_color(&theme->styles.cb_marker, theme->base.color_primary);
     lv_style_set_bg_color(&theme->styles.cb_marker, theme->color_card);
     lv_style_set_bg_opa(&theme->styles.cb_marker, LV_OPA_COVER);
     lv_style_set_radius(&theme->styles.cb_marker, RADIUS_DEFAULT / 2);
+    lv_style_set_text_font(&theme->styles.cb_marker, theme->base.font_small);
+    lv_style_set_text_color(&theme->styles.cb_marker, lv_color_white());
 
     style_init_reset(&theme->styles.cb_marker_checked);
     lv_style_set_bg_image_src(&theme->styles.cb_marker_checked, LV_SYMBOL_OK);
-    lv_style_set_text_color(&theme->styles.cb_marker_checked, lv_color_white());
-    lv_style_set_text_font(&theme->styles.cb_marker_checked, theme->base.font_small);
 #endif
 
 #if LV_USE_SWITCH
     style_init_reset(&theme->styles.switch_knob);
-    lv_style_set_pad_all(&theme->styles.switch_knob, - _LV_DPX_CALC(theme->disp_dpi, 4));
+    lv_style_set_pad_all(&theme->styles.switch_knob, - LV_DPX_CALC(theme->disp_dpi, 4));
     lv_style_set_bg_color(&theme->styles.switch_knob, lv_color_white());
 #endif
 
@@ -453,16 +437,16 @@ static void style_init(my_theme_t * theme)
 #if LV_USE_CHART
     style_init_reset(&theme->styles.chart_bg);
     lv_style_set_border_post(&theme->styles.chart_bg, false);
-    lv_style_set_pad_column(&theme->styles.chart_bg, _LV_DPX_CALC(theme->disp_dpi, 10));
+    lv_style_set_pad_column(&theme->styles.chart_bg, LV_DPX_CALC(theme->disp_dpi, 10));
     lv_style_set_line_color(&theme->styles.chart_bg, theme->color_grey);
 
     style_init_reset(&theme->styles.chart_series);
-    lv_style_set_line_width(&theme->styles.chart_series, _LV_DPX_CALC(theme->disp_dpi, 3));
-    lv_style_set_radius(&theme->styles.chart_series, _LV_DPX_CALC(theme->disp_dpi, 3));
+    lv_style_set_line_width(&theme->styles.chart_series, LV_DPX_CALC(theme->disp_dpi, 3));
+    lv_style_set_radius(&theme->styles.chart_series, LV_DPX_CALC(theme->disp_dpi, 3));
 
-    int32_t chart_size = _LV_DPX_CALC(theme->disp_dpi, 8);
+    int32_t chart_size = LV_DPX_CALC(theme->disp_dpi, 8);
     lv_style_set_size(&theme->styles.chart_series, chart_size, chart_size);
-    lv_style_set_pad_column(&theme->styles.chart_series, _LV_DPX_CALC(theme->disp_dpi, 2));
+    lv_style_set_pad_column(&theme->styles.chart_series, LV_DPX_CALC(theme->disp_dpi, 2));
 
     style_init_reset(&theme->styles.chart_indic);
     lv_style_set_radius(&theme->styles.chart_indic, LV_RADIUS_CIRCLE);
@@ -490,7 +474,7 @@ static void style_init(my_theme_t * theme)
     lv_style_set_pad_hor(&theme->styles.menu_cont, PAD_SMALL);
     lv_style_set_pad_ver(&theme->styles.menu_cont, PAD_SMALL);
     lv_style_set_pad_gap(&theme->styles.menu_cont, PAD_SMALL);
-    lv_style_set_border_width(&theme->styles.menu_cont, _LV_DPX_CALC(theme->disp_dpi, 1));
+    lv_style_set_border_width(&theme->styles.menu_cont, LV_DPX_CALC(theme->disp_dpi, 1));
     lv_style_set_border_opa(&theme->styles.menu_cont, LV_OPA_10);
     lv_style_set_border_color(&theme->styles.menu_cont, theme->color_text);
     lv_style_set_border_side(&theme->styles.menu_cont, LV_BORDER_SIDE_NONE);
@@ -498,7 +482,7 @@ static void style_init(my_theme_t * theme)
     style_init_reset(&theme->styles.menu_sidebar_cont);
     lv_style_set_pad_all(&theme->styles.menu_sidebar_cont, 0);
     lv_style_set_pad_gap(&theme->styles.menu_sidebar_cont, 0);
-    lv_style_set_border_width(&theme->styles.menu_sidebar_cont, _LV_DPX_CALC(theme->disp_dpi, 1));
+    lv_style_set_border_width(&theme->styles.menu_sidebar_cont, LV_DPX_CALC(theme->disp_dpi, 1));
     lv_style_set_border_opa(&theme->styles.menu_sidebar_cont, LV_OPA_10);
     lv_style_set_border_color(&theme->styles.menu_sidebar_cont, theme->color_text);
     lv_style_set_border_side(&theme->styles.menu_sidebar_cont, LV_BORDER_SIDE_RIGHT);
@@ -534,7 +518,7 @@ static void style_init(my_theme_t * theme)
 
 #if LV_USE_TABLE
     style_init_reset(&theme->styles.table_cell);
-    lv_style_set_border_width(&theme->styles.table_cell, _LV_DPX_CALC(theme->disp_dpi, 1));
+    lv_style_set_border_width(&theme->styles.table_cell, LV_DPX_CALC(theme->disp_dpi, 1));
     lv_style_set_border_color(&theme->styles.table_cell, theme->color_grey);
     lv_style_set_border_side(&theme->styles.table_cell, LV_BORDER_SIDE_TOP | LV_BORDER_SIDE_BOTTOM);
 #endif
@@ -542,8 +526,8 @@ static void style_init(my_theme_t * theme)
 #if LV_USE_TEXTAREA
     style_init_reset(&theme->styles.ta_cursor);
     lv_style_set_border_color(&theme->styles.ta_cursor, theme->color_text);
-    lv_style_set_border_width(&theme->styles.ta_cursor, _LV_DPX_CALC(theme->disp_dpi, 2));
-    lv_style_set_pad_left(&theme->styles.ta_cursor, - _LV_DPX_CALC(theme->disp_dpi, 1));
+    lv_style_set_border_width(&theme->styles.ta_cursor, LV_DPX_CALC(theme->disp_dpi, 2));
+    lv_style_set_pad_left(&theme->styles.ta_cursor, - LV_DPX_CALC(theme->disp_dpi, 1));
     lv_style_set_border_side(&theme->styles.ta_cursor, LV_BORDER_SIDE_LEFT);
     lv_style_set_anim_duration(&theme->styles.ta_cursor, 400);
 
@@ -559,7 +543,7 @@ static void style_init(my_theme_t * theme)
     lv_style_set_pad_gap(&theme->styles.calendar_btnm_bg, PAD_SMALL / 2);
 
     style_init_reset(&theme->styles.calendar_btnm_day);
-    lv_style_set_border_width(&theme->styles.calendar_btnm_day, _LV_DPX_CALC(theme->disp_dpi, 1));
+    lv_style_set_border_width(&theme->styles.calendar_btnm_day, LV_DPX_CALC(theme->disp_dpi, 1));
     lv_style_set_border_color(&theme->styles.calendar_btnm_day, theme->color_grey);
     lv_style_set_bg_color(&theme->styles.calendar_btnm_day, theme->color_card);
     lv_style_set_bg_opa(&theme->styles.calendar_btnm_day, LV_OPA_20);
@@ -602,7 +586,7 @@ static void style_init(my_theme_t * theme)
     lv_style_set_clip_corner(&theme->styles.list_bg, true);
 
     style_init_reset(&theme->styles.list_btn);
-    lv_style_set_border_width(&theme->styles.list_btn, _LV_DPX_CALC(theme->disp_dpi, 1));
+    lv_style_set_border_width(&theme->styles.list_btn, LV_DPX_CALC(theme->disp_dpi, 1));
     lv_style_set_border_color(&theme->styles.list_btn, theme->color_grey);
     lv_style_set_border_side(&theme->styles.list_btn, LV_BORDER_SIDE_BOTTOM);
     lv_style_set_pad_all(&theme->styles.list_btn, PAD_SMALL);
@@ -618,9 +602,9 @@ static void style_init(my_theme_t * theme)
     lv_style_set_bg_color(&theme->styles.led, lv_color_white());
     lv_style_set_bg_grad_color(&theme->styles.led, lv_palette_main(LV_PALETTE_GREY));
     lv_style_set_radius(&theme->styles.led, LV_RADIUS_CIRCLE);
-    lv_style_set_shadow_width(&theme->styles.led, _LV_DPX_CALC(theme->disp_dpi, 15));
+    lv_style_set_shadow_width(&theme->styles.led, LV_DPX_CALC(theme->disp_dpi, 15));
     lv_style_set_shadow_color(&theme->styles.led, lv_color_white());
-    lv_style_set_shadow_spread(&theme->styles.led, _LV_DPX_CALC(theme->disp_dpi, 5));
+    lv_style_set_shadow_spread(&theme->styles.led, LV_DPX_CALC(theme->disp_dpi, 5));
 #endif
 
 #if LV_USE_SCALE
@@ -629,6 +613,7 @@ static void style_init(my_theme_t * theme)
     lv_style_set_line_width(&theme->styles.scale, LV_DPX(2));
     lv_style_set_arc_color(&theme->styles.scale, theme->color_text);
     lv_style_set_arc_width(&theme->styles.scale, LV_DPX(2));
+    lv_style_set_length(&theme->styles.scale, LV_DPX(6));
 #endif
 }
 
@@ -640,7 +625,7 @@ lv_theme_t * lv_theme_default_init(lv_display_t * disp, lv_color_t color_primary
                                    const lv_font_t * font)
 {
     /*This trick is required only to avoid the garbage collection of
-     *styles' data if LVGL is used in a binding (e.g. Micropython)
+     *styles' data if LVGL is used in a binding (e.g. MicroPython)
      *In a general case styles could be in a simple `static lv_style_t my_style...` variables*/
 
     if(!lv_theme_default_is_inited()) {
@@ -663,10 +648,9 @@ lv_theme_t * lv_theme_default_init(lv_display_t * disp, lv_color_t color_primary
        theme->disp_size == new_size &&
        lv_color_eq(theme->base.color_primary, color_primary) &&
        lv_color_eq(theme->base.color_secondary, color_secondary) &&
-       (theme->base.flags == dark ? MODE_DARK : 0) &&
+       (theme->base.flags == (dark ? MODE_DARK : 0)) &&
        theme->base.font_small == font) {
         return (lv_theme_t *) theme;
-
     }
 
     theme->disp_size = new_size;
@@ -682,13 +666,31 @@ lv_theme_t * lv_theme_default_init(lv_display_t * disp, lv_color_t color_primary
 
     style_init(theme);
 
-    if(disp == NULL || lv_display_get_theme(disp) == (lv_theme_t *)theme) lv_obj_report_style_change(NULL);
+    if(disp == NULL || lv_display_get_theme(disp) == (lv_theme_t *)theme) {
+        lv_obj_report_style_change(NULL);
+    }
 
     theme->inited = true;
 
     return (lv_theme_t *) theme;
 }
 
+bool lv_theme_default_is_inited(void)
+{
+    my_theme_t * theme = theme_def;
+    if(theme == NULL) return false;
+    return theme->inited;
+}
+
+lv_theme_t * lv_theme_default_get(void)
+{
+    if(!lv_theme_default_is_inited()) {
+        return NULL;
+    }
+
+    return (lv_theme_t *)theme_def;
+}
+
 void lv_theme_default_deinit(void)
 {
     my_theme_t * theme = theme_def;
@@ -699,28 +701,15 @@ void lv_theme_default_deinit(void)
             for(i = 0; i < sizeof(my_theme_styles_t) / sizeof(lv_style_t); i++) {
                 lv_style_reset(theme_styles + i);
             }
-
         }
         lv_free(theme_def);
         theme_def = NULL;
     }
 }
 
-lv_theme_t * lv_theme_default_get(void)
-{
-    if(!lv_theme_default_is_inited()) {
-        return NULL;
-    }
-
-    return (lv_theme_t *)theme_def;
-}
-
-bool lv_theme_default_is_inited(void)
-{
-    my_theme_t * theme = theme_def;
-    if(theme == NULL) return false;
-    return theme->inited;
-}
+/**********************
+ *   STATIC FUNCTIONS
+ **********************/
 
 static void theme_apply(lv_theme_t * th, lv_obj_t * obj)
 {
@@ -803,7 +792,6 @@ static void theme_apply(lv_theme_t * th, lv_obj_t * obj)
                 return;
             }
         }
-
 #endif
         lv_obj_add_style(obj, &theme->styles.btn, 0);
         lv_obj_add_style(obj, &theme->styles.bg_color_primary, 0);
@@ -933,11 +921,9 @@ static void theme_apply(lv_theme_t * th, lv_obj_t * obj)
         lv_obj_add_style(obj, &theme->styles.outline_primary, LV_STATE_FOCUS_KEY);
         lv_obj_add_style(obj, &theme->styles.bg_color_primary, LV_PART_INDICATOR | LV_STATE_CHECKED);
         lv_obj_add_style(obj, &theme->styles.circle, LV_PART_INDICATOR);
-        lv_obj_add_style(obj, &theme->styles.disabled, LV_PART_INDICATOR | LV_STATE_DISABLED);
         lv_obj_add_style(obj, &theme->styles.knob, LV_PART_KNOB);
         lv_obj_add_style(obj, &theme->styles.bg_color_white, LV_PART_KNOB);
         lv_obj_add_style(obj, &theme->styles.switch_knob, LV_PART_KNOB);
-        lv_obj_add_style(obj, &theme->styles.disabled, LV_PART_KNOB | LV_STATE_DISABLED);
 
         lv_obj_add_style(obj, &theme->styles.transition_normal, LV_PART_INDICATOR | LV_STATE_CHECKED);
         lv_obj_add_style(obj, &theme->styles.transition_normal, LV_PART_INDICATOR);
@@ -1060,6 +1046,13 @@ static void theme_apply(lv_theme_t * th, lv_obj_t * obj)
         lv_obj_add_style(obj, &theme->styles.bg_color_secondary_muted, LV_PART_ITEMS | LV_STATE_EDITED);
     }
 #endif
+
+#if LV_USE_LABEL && LV_USE_TEXTAREA
+    else if(lv_obj_check_type(obj, &lv_label_class) && lv_obj_check_type(parent, &lv_textarea_class)) {
+        lv_obj_add_style(obj, &theme->styles.bg_color_primary, LV_PART_SELECTED);
+    }
+#endif
+
 #if LV_USE_LIST
     else if(lv_obj_check_type(obj, &lv_list_class)) {
         lv_obj_add_style(obj, &theme->styles.card, 0);
@@ -1079,7 +1072,6 @@ static void theme_apply(lv_theme_t * th, lv_obj_t * obj)
         lv_obj_add_style(obj, &theme->styles.list_item_grow, LV_STATE_FOCUS_KEY);
         lv_obj_add_style(obj, &theme->styles.list_item_grow, LV_STATE_PRESSED);
         lv_obj_add_style(obj, &theme->styles.pressed, LV_STATE_PRESSED);
-
     }
 #endif
 #if LV_USE_MENU
@@ -1158,7 +1150,6 @@ static void theme_apply(lv_theme_t * th, lv_obj_t * obj)
         lv_obj_add_style(obj, &theme->styles.disabled, LV_STATE_DISABLED);
         return;
     }
-
 #endif
 
 #if LV_USE_SPINBOX
@@ -1210,13 +1201,9 @@ static void theme_apply(lv_theme_t * th, lv_obj_t * obj)
 #endif
 }
 
-/**********************
- *   STATIC FUNCTIONS
- **********************/
-
 static void style_init_reset(lv_style_t * style)
 {
-    if(theme_def->inited) {
+    if(lv_theme_default_is_inited()) {
         lv_style_reset(style);
     }
     else {
