@@ -6,16 +6,19 @@
 /*********************
  *      INCLUDES
  *********************/
-#include "lv_table.h"
+#include "lv_table_private.h"
+#include "../../misc/lv_area_private.h"
+#include "../../core/lv_obj_private.h"
+#include "../../core/lv_obj_class_private.h"
 #if LV_USE_TABLE != 0
 
 #include "../../indev/lv_indev.h"
 #include "../../misc/lv_assert.h"
-#include "../../misc/lv_text.h"
+#include "../../misc/lv_text_private.h"
 #include "../../misc/lv_text_ap.h"
 #include "../../misc/lv_math.h"
 #include "../../stdlib/lv_sprintf.h"
-#include "../../draw/lv_draw.h"
+#include "../../draw/lv_draw_private.h"
 #include "../../stdlib/lv_string.h"
 
 /*********************
@@ -63,7 +66,7 @@ const lv_obj_class_t lv_table_class  = {
     .editable = LV_OBJ_CLASS_EDITABLE_TRUE,
     .group_def = LV_OBJ_CLASS_GROUP_DEF_TRUE,
     .instance_size = sizeof(lv_table_t),
-    .name = "table",
+    .name = "lv_table",
 };
 /**********************
  *      MACROS
@@ -166,14 +169,14 @@ void lv_table_set_cell_value_fmt(lv_obj_t * obj, uint32_t row, uint32_t col, con
     lv_vsnprintf(raw_txt, len + 1, fmt, ap2);
 
     /*Get the size of the Arabic text and process it*/
-    size_t len_ap = _lv_text_ap_calc_bytes_count(raw_txt);
+    size_t len_ap = lv_text_ap_calc_bytes_count(raw_txt);
     table->cell_data[cell] = lv_realloc(table->cell_data[cell], sizeof(lv_table_cell_t) + len_ap + 1);
     LV_ASSERT_MALLOC(table->cell_data[cell]);
     if(table->cell_data[cell] == NULL) {
         va_end(ap2);
         return;
     }
-    _lv_text_ap_proc(raw_txt, table->cell_data[cell]->txt);
+    lv_text_ap_proc(raw_txt, table->cell_data[cell]->txt);
 
     lv_free(raw_txt);
 #else
@@ -274,7 +277,7 @@ void lv_table_set_column_count(lv_obj_t * obj, uint32_t col_cnt)
         int32_t i;
         for(i = 0; i < (int32_t)old_col_cnt - (int32_t)col_cnt; i++) {
             uint32_t idx = old_col_start + min_col_cnt + i;
-            if(table->cell_data[idx]->user_data) {
+            if(table->cell_data[idx] && table->cell_data[idx]->user_data) {
                 lv_free(table->cell_data[idx]->user_data);
                 table->cell_data[idx]->user_data = NULL;
             }
@@ -312,7 +315,7 @@ void lv_table_set_column_width(lv_obj_t * obj, uint32_t col_id, int32_t w)
     refr_size_form_row(obj, 0);
 }
 
-void lv_table_add_cell_ctrl(lv_obj_t * obj, uint32_t row, uint32_t col, lv_table_cell_ctrl_t ctrl)
+void lv_table_set_cell_ctrl(lv_obj_t * obj, uint32_t row, uint32_t col, lv_table_cell_ctrl_t ctrl)
 {
     LV_ASSERT_OBJ(obj, MY_CLASS);
 
@@ -335,6 +338,7 @@ void lv_table_add_cell_ctrl(lv_obj_t * obj, uint32_t row, uint32_t col, lv_table
     }
 
     table->cell_data[cell]->ctrl |= ctrl;
+    refr_cell_size(obj, row, col);
 }
 
 void lv_table_clear_cell_ctrl(lv_obj_t * obj, uint32_t row, uint32_t col, lv_table_cell_ctrl_t ctrl)
@@ -518,6 +522,8 @@ static void lv_table_constructor(const lv_obj_class_t * class_p, lv_obj_t * obj)
     table->row_h[0] = LV_DPI_DEF;
     table->cell_data = lv_realloc(table->cell_data, table->row_cnt * table->col_cnt * sizeof(lv_table_cell_t *));
     table->cell_data[0] = NULL;
+    table->row_act = LV_TABLE_CELL_NONE;
+    table->col_act = LV_TABLE_CELL_NONE;
 
     LV_TRACE_OBJ_CREATE("finished");
 }
@@ -671,7 +677,7 @@ static void draw_main(lv_event_t * e)
     lv_table_t * table = (lv_table_t *)obj;
     lv_layer_t * layer = lv_event_get_layer(e);
     lv_area_t clip_area;
-    if(!_lv_area_intersect(&clip_area, &obj->coords, &layer->_clip_area)) return;
+    if(!lv_area_intersect(&clip_area, &obj->coords, &layer->_clip_area)) return;
 
     const lv_area_t clip_area_ori = layer->_clip_area;
     layer->_clip_area = clip_area;
@@ -691,11 +697,13 @@ static void draw_main(lv_event_t * e)
     lv_draw_rect_dsc_t rect_dsc_def;
     lv_draw_rect_dsc_t rect_dsc_act; /*Passed to the event to modify it*/
     lv_draw_rect_dsc_init(&rect_dsc_def);
+    rect_dsc_def.base.layer = layer;
     lv_obj_init_draw_rect_dsc(obj, LV_PART_ITEMS, &rect_dsc_def);
 
     lv_draw_label_dsc_t label_dsc_def;
     lv_draw_label_dsc_t label_dsc_act;  /*Passed to the event to modify it*/
     lv_draw_label_dsc_init(&label_dsc_def);
+    label_dsc_def.base.layer = layer;
     lv_obj_init_draw_label_dsc(obj, LV_PART_ITEMS, &label_dsc_def);
     obj->state = state_ori;
     obj->skip_trans = 0;
@@ -794,7 +802,9 @@ static void draw_main(lv_event_t * e)
                 obj->state = cell_state;
                 obj->skip_trans = 1;
                 lv_draw_rect_dsc_init(&rect_dsc_act);
+                rect_dsc_act.base.layer = layer;
                 lv_draw_label_dsc_init(&label_dsc_act);
+                label_dsc_act.base.layer = layer;
                 lv_obj_init_draw_rect_dsc(obj, LV_PART_ITEMS, &rect_dsc_act);
                 lv_obj_init_draw_label_dsc(obj, LV_PART_ITEMS, &label_dsc_act);
                 obj->state = state_ori;
@@ -813,21 +823,28 @@ static void draw_main(lv_event_t * e)
                 const int32_t cell_right = lv_obj_get_style_pad_right(obj, LV_PART_ITEMS);
                 const int32_t cell_top = lv_obj_get_style_pad_top(obj, LV_PART_ITEMS);
                 const int32_t cell_bottom = lv_obj_get_style_pad_bottom(obj, LV_PART_ITEMS);
-                lv_text_flag_t txt_flags = LV_TEXT_FLAG_NONE;
-                lv_area_t txt_area;
 
+                lv_text_attributes_t attributes = {0};
+                attributes.text_flags = LV_TEXT_FLAG_NONE;
+                attributes.letter_space = label_dsc_act.letter_space;
+                attributes.line_space = label_dsc_act.line_space;
+
+                lv_area_t txt_area;
                 txt_area.x1 = cell_area.x1 + cell_left;
                 txt_area.x2 = cell_area.x2 - cell_right;
                 txt_area.y1 = cell_area.y1 + cell_top;
                 txt_area.y2 = cell_area.y2 - cell_bottom;
 
+                attributes.max_width = lv_area_get_width(&txt_area);
+
                 /*Align the content to the middle if not cropped*/
                 bool crop = ctrl & LV_TABLE_CELL_CTRL_TEXT_CROP;
-                if(crop) txt_flags = LV_TEXT_FLAG_EXPAND;
+                if(crop) {
+                    attributes.text_flags = LV_TEXT_FLAG_EXPAND;
+                    label_dsc_act.flag |= LV_TEXT_FLAG_EXPAND;
+                }
 
-                lv_text_get_size(&txt_size, table->cell_data[cell]->txt, label_dsc_def.font,
-                                 label_dsc_act.letter_space, label_dsc_act.line_space,
-                                 lv_area_get_width(&txt_area), txt_flags);
+                lv_text_get_size(&txt_size, table->cell_data[cell]->txt, label_dsc_def.font, &attributes);
 
                 /*Align the content to the middle if not cropped*/
                 if(!crop) {
@@ -837,7 +854,7 @@ static void draw_main(lv_event_t * e)
 
                 lv_area_t label_clip_area;
                 bool label_mask_ok;
-                label_mask_ok = _lv_area_intersect(&label_clip_area, &clip_area, &cell_area);
+                label_mask_ok = lv_area_intersect(&label_clip_area, &clip_area, &cell_area);
                 if(label_mask_ok) {
                     layer->_clip_area = label_clip_area;
                     label_dsc_act.text = table->cell_data[cell]->txt;
@@ -925,6 +942,11 @@ static int32_t get_row_height(lv_obj_t * obj, uint32_t row_id, const lv_font_t *
     /* Calculate the cell_data index where to start */
     uint32_t row_start = row_id * table->col_cnt;
 
+    lv_text_attributes_t attributes = {0};
+    attributes.letter_space = letter_space;
+    attributes.line_space = line_space;
+    attributes.text_flags = LV_TEXT_FLAG_NONE;
+
     /* Traverse the cells in the row_id row */
     uint32_t cell;
     uint32_t col;
@@ -935,7 +957,7 @@ static int32_t get_row_height(lv_obj_t * obj, uint32_t row_id, const lv_font_t *
             continue;
         }
 
-        int32_t txt_w = table->col_w[col];
+        attributes.max_width = table->col_w[col];
 
         /* Traverse the current row from the first until the penultimate column.
          * Increment the text width if the cell has the LV_TABLE_CELL_CTRL_MERGE_RIGHT control,
@@ -948,7 +970,7 @@ static int32_t get_row_height(lv_obj_t * obj, uint32_t row_id, const lv_font_t *
 
             lv_table_cell_ctrl_t ctrl = (lv_table_cell_ctrl_t) next_cell_data->ctrl;
             if(ctrl & LV_TABLE_CELL_CTRL_MERGE_RIGHT) {
-                txt_w += table->col_w[col + col_merge + 1];
+                attributes.max_width += table->col_w[col + col_merge + 1];
             }
             else {
                 break;
@@ -965,10 +987,9 @@ static int32_t get_row_height(lv_obj_t * obj, uint32_t row_id, const lv_font_t *
         /*Else we have to calculate the height of the cell text*/
         else {
             lv_point_t txt_size;
-            txt_w -= cell_left + cell_right;
+            attributes.max_width -= cell_left + cell_right;
 
-            lv_text_get_size(&txt_size, table->cell_data[cell]->txt, font,
-                             letter_space, line_space, txt_w, LV_TEXT_FLAG_NONE);
+            lv_text_get_size(&txt_size, table->cell_data[cell]->txt, font, &attributes);
 
             h_max = LV_MAX(txt_size.y + cell_top + cell_bottom, h_max);
             /*Skip until one element after the last merged column*/
@@ -995,6 +1016,9 @@ static lv_result_t get_pressed_cell(lv_obj_t * obj, uint32_t * row, uint32_t * c
     lv_indev_get_point(lv_indev_active(), &p);
 
     int32_t tmp;
+    bool is_click_on_valid_column = false;
+    bool is_click_on_valid_row = false;
+
     if(col) {
         int32_t x = p.x + lv_obj_get_scroll_x(obj);
 
@@ -1010,12 +1034,15 @@ static lv_result_t get_pressed_cell(lv_obj_t * obj, uint32_t * row, uint32_t * c
         tmp = 0;
         for(*col = 0; *col < table->col_cnt; (*col)++) {
             tmp += table->col_w[*col];
-            if(x < tmp) break;
+            if(x < tmp) {
+                is_click_on_valid_column = true;
+                break;
+            }
         }
     }
 
     if(row) {
-        int32_t y = p.y + lv_obj_get_scroll_y(obj);;
+        int32_t y = p.y + lv_obj_get_scroll_y(obj);
         y -= obj->coords.y1;
         y -= lv_obj_get_style_pad_top(obj, LV_PART_MAIN);
 
@@ -1024,11 +1051,20 @@ static lv_result_t get_pressed_cell(lv_obj_t * obj, uint32_t * row, uint32_t * c
 
         for(*row = 0; *row < table->row_cnt; (*row)++) {
             tmp += table->row_h[*row];
-            if(y < tmp) break;
+            if(y < tmp) {
+                is_click_on_valid_row = true;
+                break;
+            }
         }
     }
 
-    return LV_RESULT_OK;
+    /* If the click was on valid column AND row then return valid result, return invalid otherwise */
+    lv_result_t result = LV_RESULT_INVALID;
+    if((is_click_on_valid_column) && (is_click_on_valid_row)) {
+        result = LV_RESULT_OK;
+    }
+
+    return result;
 }
 
 /* Returns number of bytes to allocate based on chars configuration */
@@ -1037,7 +1073,7 @@ static size_t get_cell_txt_len(const char * txt)
     size_t retval = 0;
 
 #if LV_USE_ARABIC_PERSIAN_CHARS
-    retval = sizeof(lv_table_cell_t) + _lv_text_ap_calc_bytes_count(txt) + 1;
+    retval = sizeof(lv_table_cell_t) + lv_text_ap_calc_bytes_count(txt) + 1;
 #else
     retval = sizeof(lv_table_cell_t) + lv_strlen(txt) + 1;
 #endif
@@ -1049,7 +1085,7 @@ static size_t get_cell_txt_len(const char * txt)
 static void copy_cell_txt(lv_table_cell_t * dst, const char * txt)
 {
 #if LV_USE_ARABIC_PERSIAN_CHARS
-    _lv_text_ap_proc(txt, dst->txt);
+    lv_text_ap_proc(txt, dst->txt);
 #else
     lv_strcpy(dst->txt, txt);
 #endif
@@ -1064,18 +1100,35 @@ static void get_cell_area(lv_obj_t * obj, uint32_t row, uint32_t col, lv_area_t
     for(c = 0; c < col; c++) {
         area->x1 += table->col_w[c];
     }
+    /* Traverse the current row from the first until the penultimate column.
+     * Increment the offset if the cell has the LV_TABLE_CELL_CTRL_MERGE_RIGHT control,
+     * exit the traversal when the current cell control is not LV_TABLE_CELL_CTRL_MERGE_RIGHT */
+    uint32_t col_merge = 0;
+    int32_t offset = 0;
+    for(col_merge = 0; col_merge + col < table->col_cnt - 1; col_merge++) {
+        lv_table_cell_t * next_cell_data = table->cell_data[row * table->col_cnt + col_merge];
 
+        if(is_cell_empty(next_cell_data)) break;
+
+        lv_table_cell_ctrl_t ctrl = (lv_table_cell_ctrl_t) next_cell_data->ctrl;
+        if(ctrl & LV_TABLE_CELL_CTRL_MERGE_RIGHT) {
+            offset += table->col_w[col + col_merge + 1];
+        }
+        else {
+            break;
+        }
+    }
     bool rtl = lv_obj_get_style_base_dir(obj, LV_PART_MAIN) == LV_BASE_DIR_RTL;
     if(rtl) {
         area->x1 += lv_obj_get_scroll_x(obj);
         int32_t w = lv_obj_get_width(obj);
-        area->x2 = w - area->x1 - lv_obj_get_style_pad_right(obj, 0);
-        area->x1 = area->x2 - table->col_w[col];
+        area->x2 = w - area->x1 - lv_obj_get_style_pad_right(obj, LV_PART_MAIN);
+        area->x1 = area->x2 - (table->col_w[col] + offset);
     }
     else {
         area->x1 -= lv_obj_get_scroll_x(obj);
-        area->x1 += lv_obj_get_style_pad_left(obj, 0);
-        area->x2 = area->x1 + table->col_w[col] - 1;
+        area->x1 += lv_obj_get_style_pad_left(obj, LV_PART_MAIN);
+        area->x2 = area->x1 + (table->col_w[col] + offset) - 1;
     }
 
     uint32_t r;
@@ -1084,7 +1137,7 @@ static void get_cell_area(lv_obj_t * obj, uint32_t row, uint32_t col, lv_area_t
         area->y1 += table->row_h[r];
     }
 
-    area->y1 += lv_obj_get_style_pad_top(obj, 0);
+    area->y1 += lv_obj_get_style_pad_top(obj, LV_PART_MAIN);
     area->y1 -= lv_obj_get_scroll_y(obj);
     area->y2 = area->y1 + table->row_h[row] - 1;
 
