@@ -1,12 +1,14 @@
 /**
- * @file lv_ta.c
+ * @file lv_textarea.c
  *
  */
 
 /*********************
  *      INCLUDES
  *********************/
-#include "lv_textarea.h"
+#include "lv_textarea_private.h"
+#include "../label/lv_label_private.h"
+#include "../../core/lv_obj_class_private.h"
 #if LV_USE_TEXTAREA != 0
 
 #include "../../core/lv_group.h"
@@ -14,7 +16,7 @@
 #include "../../indev/lv_indev.h"
 #include "../../draw/lv_draw.h"
 #include "../../misc/lv_assert.h"
-#include "../../misc/lv_anim.h"
+#include "../../misc/lv_anim_private.h"
 #include "../../misc/lv_text_private.h"
 #include "../../misc/lv_math.h"
 #include "../../stdlib/lv_string.h"
@@ -61,10 +63,91 @@ static void draw_cursor(lv_event_t * e);
 static void auto_hide_characters(lv_obj_t * obj);
 static void auto_hide_characters_cancel(lv_obj_t * obj);
 static inline bool is_valid_but_non_printable_char(const uint32_t letter);
+static void lv_textarea_scroll_to_cusor_pos(lv_obj_t * obj, int32_t pos);
 
 /**********************
  *  STATIC VARIABLES
  **********************/
+#if LV_USE_OBJ_PROPERTY
+static const lv_property_ops_t properties[] = {
+    {
+        .id = LV_PROPERTY_TEXTAREA_TEXT,
+        .setter = lv_textarea_set_text,
+        .getter = lv_textarea_get_text,
+    },
+    {
+        .id = LV_PROPERTY_TEXTAREA_PLACEHOLDER_TEXT,
+        .setter = lv_textarea_set_placeholder_text,
+        .getter = lv_textarea_get_placeholder_text,
+    },
+    {
+        .id = LV_PROPERTY_TEXTAREA_CURSOR_POS,
+        .setter = lv_textarea_set_cursor_pos,
+        .getter = lv_textarea_get_cursor_pos,
+    },
+    {
+        .id = LV_PROPERTY_TEXTAREA_CURSOR_CLICK_POS,
+        .setter = lv_textarea_set_cursor_click_pos,
+        .getter = lv_textarea_get_cursor_click_pos,
+    },
+    {
+        .id = LV_PROPERTY_TEXTAREA_PASSWORD_MODE,
+        .setter = lv_textarea_set_password_mode,
+        .getter = lv_textarea_get_password_mode,
+    },
+    {
+        .id = LV_PROPERTY_TEXTAREA_PASSWORD_BULLET,
+        .setter = lv_textarea_set_password_bullet,
+        .getter = lv_textarea_get_password_bullet,
+    },
+    {
+        .id = LV_PROPERTY_TEXTAREA_ONE_LINE,
+        .setter = lv_textarea_set_one_line,
+        .getter = lv_textarea_get_one_line,
+    },
+    {
+        .id = LV_PROPERTY_TEXTAREA_ACCEPTED_CHARS,
+        .setter = lv_textarea_set_accepted_chars,
+        .getter = lv_textarea_get_accepted_chars,
+    },
+    {
+        .id = LV_PROPERTY_TEXTAREA_MAX_LENGTH,
+        .setter = lv_textarea_set_max_length,
+        .getter = lv_textarea_get_max_length,
+    },
+    {
+        .id = LV_PROPERTY_TEXTAREA_INSERT_REPLACE,
+        .setter = lv_textarea_set_insert_replace,
+        .getter = NULL,
+    },
+    {
+        .id = LV_PROPERTY_TEXTAREA_TEXT_SELECTION,
+        .setter = lv_textarea_set_text_selection,
+        .getter = lv_textarea_get_text_selection,
+    },
+    {
+        .id = LV_PROPERTY_TEXTAREA_PASSWORD_SHOW_TIME,
+        .setter = lv_textarea_set_password_show_time,
+        .getter = lv_textarea_get_password_show_time,
+    },
+    {
+        .id = LV_PROPERTY_TEXTAREA_LABEL,
+        .setter = NULL,
+        .getter = lv_textarea_get_label,
+    },
+    {
+        .id = LV_PROPERTY_TEXTAREA_TEXT_IS_SELECTED,
+        .setter = NULL,
+        .getter = lv_textarea_text_is_selected,
+    },
+    {
+        .id = LV_PROPERTY_TEXTAREA_CURRENT_CHAR,
+        .setter = NULL,
+        .getter = lv_textarea_get_current_char,
+    },
+};
+#endif
+
 const lv_obj_class_t lv_textarea_class = {
     .constructor_cb = lv_textarea_constructor,
     .destructor_cb = lv_textarea_destructor,
@@ -72,9 +155,22 @@ const lv_obj_class_t lv_textarea_class = {
     .group_def = LV_OBJ_CLASS_GROUP_DEF_TRUE,
     .width_def = LV_DPI_DEF * 2,
     .height_def = LV_DPI_DEF,
+    .editable = LV_OBJ_CLASS_EDITABLE_TRUE,
     .instance_size = sizeof(lv_textarea_t),
     .base_class = &lv_obj_class,
-    .name = "textarea",
+    .name = "lv_textarea",
+#if LV_USE_OBJ_PROPERTY
+    .prop_index_start = LV_PROPERTY_TEXTAREA_START,
+    .prop_index_end = LV_PROPERTY_TEXTAREA_END,
+    .properties = properties,
+    .properties_count = sizeof(properties) / sizeof(properties[0]),
+
+#if LV_USE_OBJ_PROPERTY_NAME
+    .property_names = lv_textarea_property_names,
+    .names_count = sizeof(lv_textarea_property_names) / sizeof(lv_property_name_t),
+#endif
+
+#endif
 };
 
 static const char * ta_insert_replace;
@@ -371,38 +467,7 @@ void lv_textarea_set_cursor_pos(lv_obj_t * obj, int32_t pos)
     /*Position the label to make the cursor visible*/
     lv_obj_update_layout(obj);
 
-    lv_point_t cur_pos;
-    const lv_font_t * font = lv_obj_get_style_text_font(obj, LV_PART_MAIN);
-    lv_label_get_letter_pos(ta->label, pos, &cur_pos);
-
-    /*The text area needs to have it's final size to see if the cursor is out of the area or not*/
-
-    /*Check the top*/
-    int32_t font_h = lv_font_get_line_height(font);
-    if(cur_pos.y < lv_obj_get_scroll_top(obj)) {
-        lv_obj_scroll_to_y(obj, cur_pos.y, LV_ANIM_ON);
-    }
-    /*Check the bottom*/
-    int32_t h = lv_obj_get_content_height(obj);
-    if(cur_pos.y + font_h - lv_obj_get_scroll_top(obj) > h) {
-        lv_obj_scroll_to_y(obj, cur_pos.y - h + font_h, LV_ANIM_ON);
-    }
-
-    /*Check the left*/
-    if(cur_pos.x < lv_obj_get_scroll_left(obj)) {
-        lv_obj_scroll_to_x(obj, cur_pos.x, LV_ANIM_ON);
-    }
-    /*Check the right*/
-    int32_t w = lv_obj_get_content_width(obj);
-    if(cur_pos.x + font_h - lv_obj_get_scroll_left(obj) > w) {
-        lv_obj_scroll_to_x(obj, cur_pos.x - w + font_h, LV_ANIM_ON);
-    }
-
-    ta->cursor.valid_x = cur_pos.x;
-
-    start_cursor_blink(obj);
-
-    refr_cursor_area(obj);
+    lv_textarea_scroll_to_cusor_pos(obj, pos);
 }
 
 void lv_textarea_set_cursor_click_pos(lv_obj_t * obj, bool en)
@@ -639,11 +704,12 @@ const char * lv_textarea_get_password_bullet(lv_obj_t * obj)
     if(ta->pwd_bullet) return ta->pwd_bullet;
 
     lv_font_glyph_dsc_t g;
-    const lv_font_t * font = lv_obj_get_style_text_font(obj, LV_PART_MAIN);
 
     /*If the textarea's font has the bullet character use it else fallback to "*"*/
-    if(lv_font_get_glyph_dsc(font, &g, LV_TEXTAREA_PWD_BULLET_UNICODE, 0))
+    const lv_font_t * bullet_font = lv_obj_get_style_text_font(obj, LV_PART_MAIN);
+    if(lv_font_get_glyph_dsc(bullet_font, &g, LV_TEXTAREA_PWD_BULLET_UNICODE, '\0'))
         return LV_SYMBOL_BULLET;
+
     return "*";
 }
 
@@ -853,7 +919,8 @@ static void lv_textarea_constructor(const lv_obj_class_t * class_p, lv_obj_t * o
     ta->label = lv_label_create(obj);
     lv_obj_set_width(ta->label, lv_pct(100));
     lv_label_set_text(ta->label, "");
-    lv_obj_add_event_cb(ta->label, label_event_cb, LV_EVENT_ALL, NULL);
+    lv_obj_add_event_cb(ta->label, label_event_cb, LV_EVENT_STYLE_CHANGED, NULL);
+    lv_obj_add_event_cb(ta->label, label_event_cb, LV_EVENT_SIZE_CHANGED, NULL);
     lv_obj_add_flag(obj, LV_OBJ_FLAG_SCROLL_ON_FOCUS);
     lv_obj_remove_flag(obj, LV_OBJ_FLAG_SCROLL_WITH_ARROW);
 
@@ -932,6 +999,10 @@ static void lv_textarea_event(const lv_obj_class_t * class_p, lv_event_t * e)
     else if(code == LV_EVENT_DRAW_POST) {
         draw_cursor(e);
     }
+    else if(code == LV_EVENT_SIZE_CHANGED) {
+        lv_textarea_t * ta = (lv_textarea_t *)obj;
+        lv_textarea_scroll_to_cusor_pos(obj, ta->cursor.pos);
+    }
 }
 
 static void label_event_cb(lv_event_t * e)
@@ -1065,7 +1136,7 @@ static void start_cursor_blink(lv_obj_t * obj)
         lv_anim_set_var(&a, ta);
         lv_anim_set_exec_cb(&a, cursor_blink_anim_cb);
         lv_anim_set_duration(&a, blink_time);
-        lv_anim_set_playback_duration(&a, blink_time);
+        lv_anim_set_reverse_duration(&a, blink_time);
         lv_anim_set_values(&a, 1, 0);
         lv_anim_set_path_cb(&a, lv_anim_path_step);
         lv_anim_set_repeat_count(&a, LV_ANIM_REPEAT_INFINITE);
@@ -1114,6 +1185,7 @@ static void refr_cursor_area(lv_obj_t * obj)
 
         uint32_t tmp = letter;
         if(is_valid_but_non_printable_char(letter)) {
+            /*If non printable get the letter_w of the space char*/
             tmp = ' ';
         }
         letter_w = lv_font_get_glyph_width(font, tmp, IGNORE_KERNING);
@@ -1128,11 +1200,12 @@ static void refr_cursor_area(lv_obj_t * obj)
     int32_t bottom = lv_obj_get_style_pad_bottom(obj, LV_PART_CURSOR) + border_width;
     int32_t left = lv_obj_get_style_pad_left(obj, LV_PART_CURSOR) + border_width;
     int32_t right = lv_obj_get_style_pad_right(obj, LV_PART_CURSOR) + border_width;
+    int32_t letter_space_w = lv_obj_get_style_text_letter_space(ta->label, LV_PART_MAIN);
 
     lv_area_t cur_area;
-    cur_area.x1 = letter_pos.x - left;
+    cur_area.x1 = letter_pos.x - left - letter_space_w / 2;
     cur_area.y1 = letter_pos.y - top;
-    cur_area.x2 = letter_pos.x + right + letter_w - 1;
+    cur_area.x2 = letter_pos.x + right + letter_w - 1 + (letter_space_w + 1) / 2;
     cur_area.y2 = letter_pos.y + bottom + letter_h - 1;
 
     /*Save the new area*/
@@ -1303,16 +1376,22 @@ static void draw_placeholder(lv_event_t * e)
     if(txt[0] == '\0' && ta->placeholder_txt && ta->placeholder_txt[0] != 0) {
         lv_draw_label_dsc_t ph_dsc;
         lv_draw_label_dsc_init(&ph_dsc);
+        ph_dsc.base.layer = layer;
         lv_obj_init_draw_label_dsc(obj, LV_PART_TEXTAREA_PLACEHOLDER, &ph_dsc);
 
         if(ta->one_line) ph_dsc.flag |= LV_TEXT_FLAG_EXPAND;
 
         int32_t left = lv_obj_get_style_pad_left(obj, LV_PART_MAIN);
+        int32_t right = lv_obj_get_style_pad_right(obj, LV_PART_MAIN);
         int32_t top = lv_obj_get_style_pad_top(obj, LV_PART_MAIN);
+        int32_t bottom = lv_obj_get_style_pad_bottom(obj, LV_PART_MAIN);
         int32_t border_width = lv_obj_get_style_border_width(obj, LV_PART_MAIN);
         lv_area_t ph_coords;
         lv_area_copy(&ph_coords, &obj->coords);
-        lv_area_move(&ph_coords, left + border_width, top + border_width);
+        ph_coords.x1 += left + border_width;
+        ph_coords.x2 -= right + border_width;
+        ph_coords.y1 += top + border_width;
+        ph_coords.y2 -= bottom + border_width;
         ph_dsc.text = ta->placeholder_txt;
         lv_draw_label(layer, &ph_dsc, &ph_coords);
     }
@@ -1329,6 +1408,7 @@ static void draw_cursor(lv_event_t * e)
 
     lv_draw_rect_dsc_t cur_dsc;
     lv_draw_rect_dsc_init(&cur_dsc);
+    cur_dsc.base.layer = layer;
     lv_obj_init_draw_rect_dsc(obj, LV_PART_CURSOR, &cur_dsc);
 
     /*Draw he cursor according to the type*/
@@ -1354,9 +1434,10 @@ static void draw_cursor(lv_event_t * e)
     /*Draw the letter over the cursor only if
      *the cursor has background or the letter has different color than the original.
      *Else the original letter is drawn twice which makes it look bolder*/
-    lv_color_t label_color = lv_obj_get_style_text_color(ta->label, 0);
+    lv_color_t label_color = lv_obj_get_style_text_color(ta->label, LV_PART_MAIN);
     lv_draw_label_dsc_t cur_label_dsc;
     lv_draw_label_dsc_init(&cur_label_dsc);
+    cur_label_dsc.base.layer = layer;
     lv_obj_init_draw_label_dsc(obj, LV_PART_CURSOR, &cur_label_dsc);
     if(cur_dsc.bg_opa > LV_OPA_MIN || !lv_color_eq(cur_label_dsc.color, label_color)) {
         cur_label_dsc.text = letter_buf;
@@ -1399,4 +1480,45 @@ static inline bool is_valid_but_non_printable_char(const uint32_t letter)
     return false;
 }
 
+static void lv_textarea_scroll_to_cusor_pos(lv_obj_t * obj, int32_t pos)
+{
+    lv_textarea_t * ta = (lv_textarea_t *)obj;
+
+    lv_point_t cur_pos;
+    const lv_font_t * font = lv_obj_get_style_text_font(obj, LV_PART_MAIN);
+    lv_label_get_letter_pos(ta->label, pos, &cur_pos);
+
+    /*The text area needs to have it's final size to see if the cursor is out of the area or not*/
+
+    /*Check the top*/
+    int32_t font_h = lv_font_get_line_height(font);
+    if(cur_pos.y < lv_obj_get_scroll_top(obj)) {
+        lv_obj_scroll_to_y(obj, cur_pos.y, LV_ANIM_ON);
+    }
+    /*Check the bottom*/
+    int32_t h = lv_obj_get_content_height(obj);
+    if(cur_pos.y + font_h - lv_obj_get_scroll_top(obj) > h) {
+        lv_obj_scroll_to_y(obj, cur_pos.y - h + font_h, LV_ANIM_ON);
+    }
+
+    /*Check the left*/
+    if(cur_pos.x < lv_obj_get_scroll_left(obj)) {
+        lv_obj_scroll_to_x(obj, cur_pos.x, LV_ANIM_ON);
+    }
+    /*Check the right*/
+    int32_t w = lv_obj_get_content_width(obj);
+    if(cur_pos.x + font_h > w) {
+        lv_obj_scroll_to_x(obj, cur_pos.x - w + font_h, LV_ANIM_ON);
+    }
+    else {
+        lv_obj_scroll_to_x(obj, 0, LV_ANIM_ON);
+    }
+
+    ta->cursor.valid_x = cur_pos.x;
+
+    start_cursor_blink(obj);
+
+    refr_cursor_area(obj);
+}
+
 #endif
