@@ -1,5 +1,6 @@
 #if LV_BUILD_TEST
 #include "../lvgl.h"
+#include "../../lvgl_private.h"
 #include "../demos/lv_demos.h"
 
 #include "unity/unity.h"
@@ -15,7 +16,7 @@ void tearDown(void)
     lv_display_set_color_format(NULL, LV_COLOR_FORMAT_XRGB8888);
 }
 
-void test_render_to_rgb565(void)
+void test_render_to_argb8888(void)
 {
     lv_display_set_color_format(NULL, LV_COLOR_FORMAT_ARGB8888);
 
@@ -23,7 +24,13 @@ void test_render_to_rgb565(void)
     uint32_t opa;
     for(opa = 0; opa < 2; opa++) {
         uint32_t i;
-        for(i = 0; i < _LV_DEMO_RENDER_SCENE_NUM; i++) {
+        for(i = 0; i < LV_DEMO_RENDER_SCENE_NUM; i++) {
+
+            /*Skip test with transformed indexed images if they are not loaded to RAM*/
+            if(LV_BIN_DECODER_RAM_LOAD == 0 &&
+               (i == LV_DEMO_RENDER_SCENE_IMAGE_NORMAL_3 ||
+                i == LV_DEMO_RENDER_SCENE_IMAGE_RECOLOR_3)) continue;
+
             lv_demo_render(i, opa_values[opa]);
 
             char buf[128];
