@@ -1,5 +1,6 @@
 #if LV_BUILD_TEST
 #include "../lvgl.h"
+#include "../../lvgl_private.h"
 
 #include "unity/unity.h"
 
@@ -18,7 +19,7 @@ void tearDown(void)
     lv_obj_clean(active_screen);
 }
 
-void test_barcode_normal(void)
+void test_qrcode_normal(void)
 {
     lv_color_t bg_color = lv_palette_lighten(LV_PALETTE_LIGHT_BLUE, 5);
     lv_color_t fg_color = lv_palette_darken(LV_PALETTE_BLUE, 4);
@@ -41,6 +42,27 @@ void test_barcode_normal(void)
     TEST_ASSERT_EQUAL_SCREENSHOT("libs/qrcode_1.png");
 }
 
+void test_qrcode_quiet_zone(void)
+{
+    lv_color_t bg_color = lv_palette_lighten(LV_PALETTE_LIGHT_BLUE, 5);
+    lv_color_t fg_color = lv_palette_darken(LV_PALETTE_BLUE, 4);
+
+    lv_obj_t * qr = lv_qrcode_create(active_screen);
+    TEST_ASSERT_NOT_NULL(qr);
+    lv_qrcode_set_size(qr, 150);
+    lv_qrcode_set_dark_color(qr, fg_color);
+    lv_qrcode_set_light_color(qr, bg_color);
+    lv_qrcode_set_quiet_zone(qr, true);
+
+    /*Set data*/
+    const char * data = "https://lvgl.io";
+    lv_result_t res = lv_qrcode_update(qr, data, strlen(data));
+    TEST_ASSERT_EQUAL(res, LV_RESULT_OK);
+    lv_obj_center(qr);
+
+    TEST_ASSERT_EQUAL_SCREENSHOT("libs/qrcode_2.png");
+}
+
 #else
 
 void setUp(void)
@@ -51,7 +73,7 @@ void tearDown(void)
 {
 }
 
-void test_barcode_normal(void)
+void test_qrcode_normal(void)
 {
 }
 
