@@ -1,12 +1,10 @@
 #if LV_BUILD_TEST
 #include "../lvgl.h"
+#include "../../lvgl_private.h"
 #include "../demos/lv_demos.h"
 
 #include "unity/unity.h"
 
-#include "lv_test_helpers.h"
-#include "lv_test_indev.h"
-
 static void loop_through_stress_test(void)
 {
 #if LV_USE_DEMO_STRESS
