@@ -1,8 +1,7 @@
 #if LV_BUILD_TEST
 #include "../lvgl.h"
-
+#include "../../lvgl_private.h"
 #include "unity/unity.h"
-#include "lv_test_indev.h"
 #include <string.h>
 
 void setUp(void)
@@ -15,6 +14,7 @@ void tearDown(void)
     /* Function run after every test */
     lv_obj_clean(lv_screen_active());
 }
+
 void test_dropdown_create_delete(void)
 {
     lv_dropdown_create(lv_screen_active());
@@ -164,7 +164,7 @@ void test_dropdown_keypad(void)
     lv_obj_clean(lv_screen_active());
 
     lv_group_t * g = lv_group_create();
-    lv_indev_set_group(lv_test_keypad_indev, g);
+    lv_indev_set_group(lv_test_indev_get_indev(LV_INDEV_TYPE_KEYPAD), g);
 
     lv_obj_t * dd1 = lv_dropdown_create(lv_screen_active());
     lv_obj_set_pos(dd1, 20, 20);
@@ -257,7 +257,7 @@ void test_dropdown_keypad(void)
     TEST_ASSERT_FALSE(lv_dropdown_is_open(dd1));
     TEST_ASSERT_NOT_NULL(lv_dropdown_get_list(dd2));
 
-    lv_indev_set_group(lv_test_keypad_indev, NULL);
+    lv_indev_set_group(lv_test_indev_get_indev(LV_INDEV_TYPE_KEYPAD), NULL);
     lv_group_delete(g);
 }
 
@@ -266,7 +266,7 @@ void test_dropdown_encoder(void)
     lv_obj_clean(lv_screen_active());
 
     lv_group_t * g = lv_group_create();
-    lv_indev_set_group(lv_test_encoder_indev, g);
+    lv_indev_set_group(lv_test_indev_get_indev(LV_INDEV_TYPE_ENCODER), g);
 
     lv_obj_t * dd1 = lv_dropdown_create(lv_screen_active());
     lv_obj_set_pos(dd1, 20, 20);
@@ -301,9 +301,9 @@ void test_dropdown_encoder(void)
     lv_test_encoder_click();
     lv_test_encoder_turn(2);
     lv_test_encoder_press();
-    lv_test_indev_wait(1000);  //Long press
+    lv_test_wait(1000);  //Long press
     lv_test_encoder_release();
-    lv_test_indev_wait(50);
+    lv_test_wait(50);
     TEST_ASSERT_FALSE(lv_dropdown_is_open(dd1));
     TEST_ASSERT_EQUAL(4, lv_dropdown_get_selected(dd1));
     TEST_ASSERT_EQUAL(2, event_cnt);
@@ -313,7 +313,7 @@ void test_dropdown_encoder(void)
     TEST_ASSERT_FALSE(lv_dropdown_is_open(dd1));
     TEST_ASSERT_TRUE(lv_dropdown_is_open(dd2));
 
-    lv_indev_set_group(lv_test_encoder_indev, NULL);
+    lv_indev_set_group(lv_test_indev_get_indev(LV_INDEV_TYPE_ENCODER), NULL);
     lv_group_delete(g);
 }
 
@@ -380,6 +380,7 @@ void test_dropdown_render_2(void)
     lv_dropdown_open(dd2);
 
     lv_obj_t * dd3 = lv_dropdown_create(lv_screen_active());
+    lv_obj_set_height(dd3, 70);
     lv_dropdown_set_text(dd3, "Limit Down");
     lv_dropdown_set_options(dd3, "1aaaaaaaaaaaaaaaa\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n13\n14\n15");
     lv_obj_align(dd3, LV_ALIGN_LEFT_MID, 300, -10);
@@ -451,4 +452,93 @@ void test_dropdown_get_options_should_check_lengths(void)
     TEST_ASSERT_EQUAL_INT(-1, lv_dropdown_get_option_index(dd, "Option "));
 }
 
+void test_dropdown_text_align(void)
+{
+    lv_obj_set_flex_flow(lv_screen_active(), LV_FLEX_FLOW_COLUMN_WRAP);
+    lv_obj_t * dd;
+    dd = lv_dropdown_create(lv_screen_active());
+    lv_obj_set_style_text_align(dd, LV_TEXT_ALIGN_LEFT, LV_PART_MAIN);
+    dd = lv_dropdown_create(lv_screen_active());
+    lv_obj_set_style_text_align(dd, LV_TEXT_ALIGN_RIGHT, LV_PART_MAIN);
+    dd = lv_dropdown_create(lv_screen_active());
+    lv_obj_set_style_text_align(dd, LV_TEXT_ALIGN_CENTER, LV_PART_MAIN);
+
+    dd = lv_dropdown_create(lv_screen_active());
+    lv_obj_set_style_text_align(dd, LV_TEXT_ALIGN_LEFT, LV_PART_MAIN);
+    lv_obj_set_style_base_dir(dd, LV_BASE_DIR_RTL, 0);
+    dd = lv_dropdown_create(lv_screen_active());
+    lv_obj_set_style_text_align(dd, LV_TEXT_ALIGN_RIGHT, LV_PART_MAIN);
+    lv_obj_set_style_base_dir(dd, LV_BASE_DIR_RTL, 0);
+    dd = lv_dropdown_create(lv_screen_active());
+    lv_obj_set_style_text_align(dd, LV_TEXT_ALIGN_CENTER, LV_PART_MAIN);
+    lv_obj_set_style_base_dir(dd, LV_BASE_DIR_RTL, 0);
+
+    dd = lv_dropdown_create(lv_screen_active());
+    lv_obj_set_style_text_align(dd, LV_TEXT_ALIGN_LEFT, LV_PART_MAIN);
+    lv_dropdown_set_symbol(dd, NULL);
+    dd = lv_dropdown_create(lv_screen_active());
+    lv_obj_set_style_text_align(dd, LV_TEXT_ALIGN_RIGHT, LV_PART_MAIN);
+    lv_dropdown_set_symbol(dd, NULL);
+    dd = lv_dropdown_create(lv_screen_active());
+    lv_obj_set_style_text_align(dd, LV_TEXT_ALIGN_CENTER, LV_PART_MAIN);
+    lv_dropdown_set_symbol(dd, NULL);
+
+    TEST_ASSERT_EQUAL_SCREENSHOT("widgets/dropdown_3.png");
+}
+
+void test_dropdown_properties(void)
+{
+#if LV_USE_OBJ_PROPERTY
+    lv_obj_t * obj = lv_dropdown_create(lv_screen_active());
+    lv_property_t prop = { };
+
+    prop.id = LV_PROPERTY_DROPDOWN_TEXT;
+    prop.ptr = "Hello";
+    TEST_ASSERT_TRUE(lv_obj_set_property(obj, &prop) == LV_RESULT_OK);
+    TEST_ASSERT_EQUAL_STRING("Hello", lv_dropdown_get_text(obj));
+    TEST_ASSERT_EQUAL_STRING("Hello", lv_obj_get_property(obj, LV_PROPERTY_DROPDOWN_TEXT).ptr);
+
+    prop.id = LV_PROPERTY_DROPDOWN_OPTIONS;
+    prop.ptr = "Option 1\nOption 2";
+    TEST_ASSERT_TRUE(lv_obj_set_property(obj, &prop) == LV_RESULT_OK);
+    TEST_ASSERT_EQUAL_STRING("Option 1\nOption 2", lv_dropdown_get_options(obj));
+    TEST_ASSERT_EQUAL_STRING("Option 1\nOption 2", lv_obj_get_property(obj, LV_PROPERTY_DROPDOWN_OPTIONS).ptr);
+
+    TEST_ASSERT_EQUAL(2, lv_dropdown_get_option_count(obj));
+    TEST_ASSERT_EQUAL(2, lv_obj_get_property(obj, LV_PROPERTY_DROPDOWN_OPTION_COUNT).num);
+
+    prop.id = LV_PROPERTY_DROPDOWN_SELECTED;
+    prop.num = 1;
+    TEST_ASSERT_TRUE(lv_obj_set_property(obj, &prop) == LV_RESULT_OK);
+    TEST_ASSERT_EQUAL_INT(1, lv_dropdown_get_selected(obj));
+    TEST_ASSERT_EQUAL_INT(1, lv_obj_get_property(obj, LV_PROPERTY_DROPDOWN_SELECTED).num);
+
+    prop.id = LV_PROPERTY_DROPDOWN_DIR;
+    prop.num = LV_DIR_LEFT;
+    TEST_ASSERT_TRUE(lv_obj_set_property(obj, &prop) == LV_RESULT_OK);
+    TEST_ASSERT_EQUAL_INT(LV_DIR_LEFT, lv_dropdown_get_dir(obj));
+    TEST_ASSERT_EQUAL_INT(LV_DIR_LEFT, lv_obj_get_property(obj, LV_PROPERTY_DROPDOWN_DIR).num);
+
+    prop.id = LV_PROPERTY_DROPDOWN_SYMBOL;
+    prop.ptr = LV_SYMBOL_DOWN;
+    TEST_ASSERT_TRUE(lv_obj_set_property(obj, &prop) == LV_RESULT_OK);
+    TEST_ASSERT_EQUAL_STRING(LV_SYMBOL_DOWN, lv_dropdown_get_symbol(obj));
+    TEST_ASSERT_EQUAL_STRING(LV_SYMBOL_DOWN, lv_obj_get_property(obj, LV_PROPERTY_DROPDOWN_SYMBOL).ptr);
+
+    prop.id = LV_PROPERTY_DROPDOWN_SELECTED_HIGHLIGHT;
+    prop.num = true;
+    TEST_ASSERT_TRUE(lv_obj_set_property(obj, &prop) == LV_RESULT_OK);
+    TEST_ASSERT_TRUE(lv_dropdown_get_selected_highlight(obj));
+    TEST_ASSERT_TRUE(lv_obj_get_property(obj, LV_PROPERTY_DROPDOWN_SELECTED_HIGHLIGHT).num);
+
+    lv_dropdown_open(obj);
+    TEST_ASSERT_TRUE(lv_dropdown_is_open(obj));
+    TEST_ASSERT_TRUE(lv_obj_get_property(obj, LV_PROPERTY_DROPDOWN_IS_OPEN).enable);
+    lv_dropdown_close(obj);
+    TEST_ASSERT_FALSE(lv_dropdown_is_open(obj));
+    TEST_ASSERT_FALSE(lv_obj_get_property(obj, LV_PROPERTY_DROPDOWN_IS_OPEN).enable);
+#endif
+}
+
+
 #endif
