@@ -1,5 +1,6 @@
 #if LV_BUILD_TEST
 #include "../lvgl.h"
+#include "../../lvgl_private.h"
 
 #include "unity/unity.h"
 
@@ -31,28 +32,16 @@ void test_scale_render_example_1(void)
     lv_obj_set_style_length(scale, 10, LV_PART_INDICATOR);
     lv_scale_set_range(scale, 10, 40);
 
-    TEST_ASSERT_EQUAL_SCREENSHOT("scale_1.png");
+    TEST_ASSERT_EQUAL_SCREENSHOT("widgets/scale_1.png");
+
+    /* test no major ticks */
+    lv_scale_set_major_tick_every(scale, 0);
+    TEST_ASSERT_EQUAL_SCREENSHOT("widgets/scale_6.png");
 }
 
-/* An vertical scale with section and custom styling */
+/* An vertical scale and a horizontal scale with section and custom styling */
 void test_scale_render_example_2(void)
 {
-    lv_obj_t * scale = lv_scale_create(lv_screen_active());
-    lv_obj_set_size(scale, 60, 200);
-    lv_scale_set_label_show(scale, true);
-    lv_scale_set_mode(scale, LV_SCALE_MODE_VERTICAL_RIGHT);
-    lv_obj_center(scale);
-
-    lv_scale_set_total_tick_count(scale, 21);
-    lv_scale_set_major_tick_every(scale, 5);
-
-    lv_obj_set_style_length(scale, 5, LV_PART_ITEMS);
-    lv_obj_set_style_length(scale, 10, LV_PART_INDICATOR);
-    lv_scale_set_range(scale, 0, 100);
-
-    static const char * custom_labels[] = {"0 °C", "25 °C", "50 °C", "75 °C", "100 °C", NULL};
-    lv_scale_set_text_src(scale, custom_labels);
-
     static lv_style_t indicator_style;
     lv_style_init(&indicator_style);
 
@@ -64,25 +53,21 @@ void test_scale_render_example_2(void)
     lv_style_set_line_color(&indicator_style, lv_palette_darken(LV_PALETTE_BLUE, 3));
     lv_style_set_width(&indicator_style, 10U);      /*Tick length*/
     lv_style_set_line_width(&indicator_style, 2U);  /*Tick width*/
-    lv_obj_add_style(scale, &indicator_style, LV_PART_INDICATOR);
 
     static lv_style_t minor_ticks_style;
     lv_style_init(&minor_ticks_style);
     lv_style_set_line_color(&minor_ticks_style, lv_palette_lighten(LV_PALETTE_BLUE, 2));
     lv_style_set_width(&minor_ticks_style, 5U);         /*Tick length*/
     lv_style_set_line_width(&minor_ticks_style, 2U);    /*Tick width*/
-    lv_obj_add_style(scale, &minor_ticks_style, LV_PART_ITEMS);
 
     static lv_style_t main_line_style;
     lv_style_init(&main_line_style);
     /* Main line properties */
     lv_style_set_line_color(&main_line_style, lv_palette_darken(LV_PALETTE_BLUE, 3));
     lv_style_set_line_width(&main_line_style, 2U); // Tick width
-    lv_obj_add_style(scale, &main_line_style, LV_PART_MAIN);
 
-    /* Add a section */
-    static lv_style_t section_minor_tick_style;
     static lv_style_t section_label_style;
+    static lv_style_t section_minor_tick_style;
     static lv_style_t section_main_line_style;
 
     lv_style_init(&section_label_style);
@@ -103,20 +88,60 @@ void test_scale_render_example_2(void)
     lv_style_set_line_color(&section_main_line_style, lv_palette_darken(LV_PALETTE_RED, 3));
     lv_style_set_line_width(&section_main_line_style, 4U); /*Tick width*/
 
-    /* Configure section styles */
-    lv_scale_section_t * section = lv_scale_add_section(scale);
-    lv_scale_section_set_range(section, 75, 100);
-    lv_scale_section_set_style(section, LV_PART_INDICATOR, &section_label_style);
-    lv_scale_section_set_style(section, LV_PART_ITEMS, &section_minor_tick_style);
-    lv_scale_section_set_style(section, LV_PART_MAIN, &section_main_line_style);
+    uint32_t i;
+    for(i = 0; i < 2; i++) {
+        lv_obj_t * scale = lv_scale_create(lv_screen_active());
+        lv_scale_set_label_show(scale, true);
 
-    lv_obj_set_style_bg_color(scale, lv_palette_main(LV_PALETTE_BLUE_GREY), 0);
-    lv_obj_set_style_bg_opa(scale, LV_OPA_50, 0);
-    lv_obj_set_style_pad_left(scale, 8, 0);
-    lv_obj_set_style_radius(scale, 8, 0);
-    lv_obj_set_style_pad_ver(scale, 20, 0);
+        if(i == 0) {
+            lv_obj_set_size(scale, 60, 200);
+            lv_scale_set_mode(scale, LV_SCALE_MODE_VERTICAL_RIGHT);
+            lv_obj_align(scale, LV_ALIGN_CENTER, -100, 0);
 
-    TEST_ASSERT_EQUAL_SCREENSHOT("scale_2.png");
+        }
+        else {
+            lv_obj_set_size(scale, 200, 60);
+            lv_scale_set_mode(scale, LV_SCALE_MODE_HORIZONTAL_TOP);
+            lv_obj_align(scale, LV_ALIGN_CENTER, 100, 0);
+        }
+
+        lv_scale_set_total_tick_count(scale, 21);
+        lv_scale_set_major_tick_every(scale, 5);
+
+        lv_obj_set_style_length(scale, 5, LV_PART_ITEMS);
+        lv_obj_set_style_length(scale, 10, LV_PART_INDICATOR);
+        lv_scale_set_range(scale, 0, 100);
+
+        static const char * custom_labels[] = {"0 °C", "25 °C", "50 °C", "75 °C", "100 °C", NULL};
+        lv_scale_set_text_src(scale, custom_labels);
+        /* Configure scale styles */
+        lv_obj_add_style(scale, &indicator_style, LV_PART_INDICATOR);
+        lv_obj_add_style(scale, &minor_ticks_style, LV_PART_ITEMS);
+        lv_obj_add_style(scale, &main_line_style, LV_PART_MAIN);
+
+        lv_scale_section_t * section = lv_scale_add_section(scale);
+        /* Configure section styles */
+        lv_scale_set_section_range(scale, section, 75, 100);
+        lv_scale_set_section_style_indicator(scale, section, &section_label_style);
+        lv_scale_set_section_style_items(scale, section, &section_minor_tick_style);
+        lv_scale_set_section_style_main(scale, section, &section_main_line_style);
+
+        lv_obj_set_style_bg_color(scale, lv_palette_main(LV_PALETTE_BLUE_GREY), 0);
+        lv_obj_set_style_bg_opa(scale, LV_OPA_50, 0);
+        lv_obj_set_style_radius(scale, 8, 0);
+
+        if(i == 0) {
+            lv_obj_set_style_pad_left(scale, 8, 0);
+            lv_obj_set_style_pad_ver(scale, 20, 0);
+
+        }
+        else {
+            lv_obj_set_style_pad_bottom(scale, 8, 0);
+            lv_obj_set_style_pad_hor(scale, 20, 0);
+        }
+    }
+
+    TEST_ASSERT_EQUAL_SCREENSHOT("widgets/scale_2.png");
 }
 
 /* A simple round scale */
@@ -139,7 +164,7 @@ void test_scale_render_example_3(void)
     lv_obj_set_style_length(scale, 10, LV_PART_INDICATOR);
     lv_scale_set_range(scale, 10, 40);
 
-    TEST_ASSERT_EQUAL_SCREENSHOT("scale_3.png");
+    TEST_ASSERT_EQUAL_SCREENSHOT("widgets/scale_3.png");
 }
 
 /* A round scale with section and custom styling */
@@ -213,12 +238,104 @@ void test_scale_render_example_4(void)
 
     /* Configure section styles */
     lv_scale_section_t * section = lv_scale_add_section(scale);
-    lv_scale_section_set_range(section, 75, 100);
-    lv_scale_section_set_style(section, LV_PART_INDICATOR, &section_label_style);
-    lv_scale_section_set_style(section, LV_PART_ITEMS, &section_minor_tick_style);
-    lv_scale_section_set_style(section, LV_PART_MAIN, &section_main_line_style);
+    lv_scale_set_section_range(scale, section, 75, 100);
+    lv_scale_set_section_style_indicator(scale, section, &section_label_style);
+    lv_scale_set_section_style_items(scale, section, &section_minor_tick_style);
+    lv_scale_set_section_style_main(scale, section, &section_main_line_style);
 
-    TEST_ASSERT_EQUAL_SCREENSHOT("scale_4.png");
+    TEST_ASSERT_EQUAL_SCREENSHOT("widgets/scale_4.png");
+}
+
+static void draw_event_cb(lv_event_t * e)
+{
+    lv_obj_t * obj = lv_event_get_target(e);
+    lv_draw_task_t * draw_task = lv_event_get_draw_task(e);
+    lv_draw_dsc_base_t * base_dsc = lv_draw_task_get_draw_dsc(draw_task);
+    lv_draw_label_dsc_t * label_draw_dsc = lv_draw_task_get_label_dsc(draw_task);
+    lv_draw_line_dsc_t * line_draw_dsc = lv_draw_task_get_line_dsc(draw_task);
+    if(label_draw_dsc) {
+        if(base_dsc->part == LV_PART_INDICATOR) {
+            const lv_color_t color_idx[7] = {
+                lv_palette_main(LV_PALETTE_RED),
+                lv_palette_main(LV_PALETTE_ORANGE),
+                lv_palette_main(LV_PALETTE_YELLOW),
+                lv_palette_main(LV_PALETTE_GREEN),
+                lv_palette_main(LV_PALETTE_CYAN),
+                lv_palette_main(LV_PALETTE_BLUE),
+                lv_palette_main(LV_PALETTE_PURPLE),
+            };
+            uint8_t major_tick = lv_scale_get_major_tick_every(obj);
+            label_draw_dsc->color = color_idx[base_dsc->id1 / major_tick];
+
+            /*Free the previously allocated text if needed*/
+            if(label_draw_dsc->text_local) lv_free((void *)label_draw_dsc->text);
+
+            /*Malloc the text and set text_local as 1 to make LVGL automatically free the text.
+             * (Local texts are malloc'd internally by LVGL. Mimic this behavior here too)*/
+            char tmp_buffer[20] = {0}; /* Big enough buffer */
+            lv_snprintf(tmp_buffer, sizeof(tmp_buffer), "%.1f", base_dsc->id2 * 1.0f);
+            label_draw_dsc->text = lv_strdup(tmp_buffer);
+            label_draw_dsc->text_local = 1;
+
+            lv_point_t size;
+            lv_text_attributes_t attributes = {0};
+            attributes.letter_space = 0;
+            attributes.line_space = 0;
+            attributes.max_width = 1000;
+            attributes.text_flags = LV_TEXT_FLAG_NONE;
+
+            lv_text_get_size(&size, label_draw_dsc->text, label_draw_dsc->font, &attributes);
+            int32_t new_w = size.x;
+            int32_t old_w = lv_area_get_width(&draw_task->area);
+
+            /* Distribute the new size equally on both sides */
+            draw_task->area.x1 -= (new_w - old_w) / 2;
+            draw_task->area.x2 += ((new_w - old_w) + 1) / 2;  /* +1 for rounding */
+
+        }
+    }
+    else if(line_draw_dsc) {
+        if(base_dsc->part == LV_PART_INDICATOR || base_dsc->part == LV_PART_ITEMS) {
+            const lv_color_t color_idx[7] = {
+                lv_palette_main(LV_PALETTE_RED),
+                lv_palette_main(LV_PALETTE_ORANGE),
+                lv_palette_main(LV_PALETTE_YELLOW),
+                lv_palette_main(LV_PALETTE_GREEN),
+                lv_palette_main(LV_PALETTE_CYAN),
+                lv_palette_main(LV_PALETTE_BLUE),
+                lv_palette_main(LV_PALETTE_PURPLE),
+            };
+            uint32_t tick_idx = base_dsc->id1;
+            uint32_t tick_value = base_dsc->id2;
+
+            line_draw_dsc->color = color_idx[tick_idx % 7];
+
+            int32_t expected_tick_value = lv_map(tick_idx, 0, 31 - 1, 10, 40);
+            TEST_ASSERT_EQUAL(tick_value, expected_tick_value);
+        }
+    }
+}
+
+void test_scale_render_example_7(void)
+{
+    lv_obj_t * scale = lv_scale_create(lv_screen_active());
+    lv_obj_set_size(scale, lv_pct(80), 100);
+    lv_scale_set_mode(scale, LV_SCALE_MODE_HORIZONTAL_BOTTOM);
+    lv_obj_center(scale);
+
+    lv_scale_set_label_show(scale, true);
+
+    lv_scale_set_total_tick_count(scale, 31);
+    lv_scale_set_major_tick_every(scale, 5);
+
+    lv_obj_set_style_length(scale, 5, LV_PART_ITEMS);
+    lv_obj_set_style_length(scale, 10, LV_PART_INDICATOR);
+    lv_scale_set_range(scale, 10, 40);
+
+    lv_obj_add_event_cb(scale, draw_event_cb, LV_EVENT_DRAW_TASK_ADDED, NULL);
+    lv_obj_add_flag(scale, LV_OBJ_FLAG_SEND_DRAW_TASK_EVENTS);
+
+    TEST_ASSERT_EQUAL_SCREENSHOT("widgets/scale_5.png");
 }
 
 void test_scale_set_style(void)
@@ -235,16 +352,16 @@ void test_scale_set_style(void)
 
     /* Configure section styles */
     lv_scale_section_t * section = lv_scale_add_section(scale);
-    lv_scale_section_set_range(section, 75, 100);
+    lv_scale_set_section_range(scale, section, 75, 100);
 
-    lv_scale_section_set_style(section, LV_PART_MAIN, &section_main_line_style);
+    lv_scale_set_section_style_main(scale, section, &section_main_line_style);
     TEST_ASSERT_NOT_NULL(section->main_style);
     TEST_ASSERT_NULL(section->indicator_style);
     TEST_ASSERT_NULL(section->items_style);
 
     TEST_ASSERT_EQUAL(section->main_style, &section_main_line_style);
 
-    lv_scale_section_set_style(section, LV_PART_INDICATOR, &section_label_style);
+    lv_scale_set_section_style_indicator(scale, section, &section_label_style);
     TEST_ASSERT_NOT_NULL(section->main_style);
     TEST_ASSERT_NOT_NULL(section->indicator_style);
     TEST_ASSERT_NULL(section->items_style);
@@ -252,7 +369,7 @@ void test_scale_set_style(void)
     TEST_ASSERT_EQUAL(section->main_style, &section_main_line_style);
     TEST_ASSERT_EQUAL(section->indicator_style, &section_label_style);
 
-    lv_scale_section_set_style(section, LV_PART_ITEMS, &section_minor_tick_style);
+    lv_scale_set_section_style_items(scale, section, &section_minor_tick_style);
     TEST_ASSERT_NOT_NULL(section->main_style);
     TEST_ASSERT_NOT_NULL(section->indicator_style);
     TEST_ASSERT_NOT_NULL(section->items_style);
@@ -260,22 +377,6 @@ void test_scale_set_style(void)
     TEST_ASSERT_EQUAL(section->main_style, &section_main_line_style);
     TEST_ASSERT_EQUAL(section->indicator_style, &section_label_style);
     TEST_ASSERT_EQUAL(section->items_style, &section_minor_tick_style);
-
-    /* Invalid part */
-    lv_scale_section_set_style(section, LV_PART_CURSOR, &section_minor_tick_style);
-    TEST_ASSERT_NOT_NULL(section->main_style);
-    TEST_ASSERT_NOT_NULL(section->indicator_style);
-    TEST_ASSERT_NOT_NULL(section->items_style);
-
-    TEST_ASSERT_EQUAL(section->main_style, &section_main_line_style);
-    TEST_ASSERT_EQUAL(section->indicator_style, &section_label_style);
-    TEST_ASSERT_EQUAL(section->items_style, &section_minor_tick_style);
-
-    /* NULL section */
-    lv_scale_section_t * null_section = NULL;
-
-    lv_scale_section_set_range(null_section, 75, 100);
-    lv_scale_section_set_style(null_section, LV_PART_MAIN, &section_main_line_style);
 }
 
 /* The scale internally counts the number of custom labels until it finds the NULL sentinel */
@@ -352,6 +453,44 @@ void test_scale_angle_range(void)
     TEST_ASSERT_EQUAL(angle_range, lv_scale_get_angle_range(scale));
 }
 
+void test_scale_rotation(void)
+{
+    lv_obj_t * scale = lv_scale_create(lv_screen_active());
+
+    lv_scale_set_rotation(scale, 135);
+    TEST_ASSERT_EQUAL(lv_scale_get_rotation(scale), 135);
+
+    lv_scale_set_rotation(scale, 375); /* 15 */
+    TEST_ASSERT_EQUAL(lv_scale_get_rotation(scale), 15);
+
+    lv_scale_set_rotation(scale, 540); /* 180 */
+    TEST_ASSERT_EQUAL(lv_scale_get_rotation(scale), 180);
+
+    lv_scale_set_rotation(scale, 1085); /* 5 */
+    TEST_ASSERT_EQUAL(lv_scale_get_rotation(scale), 5);
+
+    lv_scale_set_rotation(scale, -90); /* 270 */
+    TEST_ASSERT_EQUAL(lv_scale_get_rotation(scale), 270);
+
+    lv_scale_set_rotation(scale, -270); /* 90 */
+    TEST_ASSERT_EQUAL(lv_scale_get_rotation(scale), 90);
+
+    lv_scale_set_rotation(scale, -355); /* 5 */
+    TEST_ASSERT_EQUAL(lv_scale_get_rotation(scale), 5);
+
+    lv_scale_set_rotation(scale, -370); /* 350 */
+    TEST_ASSERT_EQUAL(lv_scale_get_rotation(scale), 350);
+
+    lv_scale_set_rotation(scale, -405); /* 315 */
+    TEST_ASSERT_EQUAL(lv_scale_get_rotation(scale), 315);
+
+    lv_scale_set_rotation(scale, -450); /* 270 */
+    TEST_ASSERT_EQUAL(lv_scale_get_rotation(scale), 270);
+
+    lv_scale_set_rotation(scale, -1075); /* 5 */
+    TEST_ASSERT_EQUAL(lv_scale_get_rotation(scale), 5);
+}
+
 void test_scale_range(void)
 {
     lv_obj_t * scale = lv_scale_create(lv_screen_active());
@@ -371,7 +510,7 @@ void test_scale_set_line_needle_value(void)
 
     lv_obj_t * line = lv_line_create(scale);
 
-    /* test the scale alocating the array */
+    /* test the scale allocating the array */
     lv_scale_set_line_needle_value(scale, line, 50, 35);
     TEST_ASSERT_EQUAL_UINT32(2, lv_line_get_point_count(line));
     const lv_point_precise_t * allocated_points_array = lv_line_get_points(line);
