@@ -1,26 +1,27 @@
 
 #if defined(LV_LVGL_H_INCLUDE_SIMPLE)
 #include "lvgl.h"
+#elif defined(LV_LVGL_H_INCLUDE_SYSTEM)
+#include <lvgl.h>
 #elif defined(LV_BUILD_TEST)
 #include "../lvgl.h"
 #else
 #include "lvgl/lvgl.h"
 #endif
 
-
 #ifndef LV_ATTRIBUTE_MEM_ALIGN
 #define LV_ATTRIBUTE_MEM_ALIGN
 #endif
 
-#ifndef LV_ATTRIBUTE_IMG_DUST
-#define LV_ATTRIBUTE_IMG_DUST
+#ifndef LV_ATTRIBUTE_TEST_A8_RLE_ALIGN1
+#define LV_ATTRIBUTE_TEST_A8_RLE_ALIGN1
 #endif
 
 static const
-LV_ATTRIBUTE_MEM_ALIGN LV_ATTRIBUTE_LARGE_CONST LV_ATTRIBUTE_IMG_DUST
+LV_ATTRIBUTE_MEM_ALIGN LV_ATTRIBUTE_LARGE_CONST LV_ATTRIBUTE_TEST_A8_RLE_ALIGN1
 uint8_t test_A8_RLE_align1_map[] = {
 
-    0x01,0x00,0x00,0x00,0x8a,0x0c,0x00,0x00,0xc1,0x12,0x00,0x00,0x47,0xff,0x8b,0x00,
+    0x01,0x00,0x00,0x00,0x89,0x0c,0x00,0x00,0xc0,0x12,0x00,0x00,0x47,0xff,0x8b,0x00,
     0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0x44,0x00,0x8c,0xff,0x00,0x00,
     0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0x44,0x00,0x9e,0xff,0x00,0x00,0x00,
     0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
@@ -220,19 +221,23 @@ uint8_t test_A8_RLE_align1_map[] = {
     0x00,0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0x44,0x00,0x8c,
     0xff,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0x44,0x00,0x8c,0xff,
     0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0x44,0x00,0x8b,0xff,0x00,
-    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x46,0xff,0x8a,0x00,0x00,0x00,0x00,
-    0x00,0x00,0x00,0x00,0x00,0x00,
+    0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,0x46,0xff,0x89,0x00,0x00,0x00,0x00,
+    0x00,0x00,0x00,0x00,0x00,
 
 };
 
-const lv_img_dsc_t test_A8_RLE_align1 = {
-  .header.magic = LV_IMAGE_HEADER_MAGIC,
-  .header.cf = LV_COLOR_FORMAT_A8,
-  .header.flags = 0 | LV_IMAGE_FLAGS_COMPRESSED,
-  .header.w = 71,
-  .header.h = 60,
-  .header.stride = 80,
-  .data_size = 3222,
+const lv_image_dsc_t test_A8_RLE_align1 = {
+  .header = {
+    .magic = LV_IMAGE_HEADER_MAGIC,
+    .cf = LV_COLOR_FORMAT_A8,
+    .flags = 0 | LV_IMAGE_FLAGS_COMPRESSED,
+    .w = 71,
+    .h = 60,
+    .stride = 80,
+    .reserved_2 = 0,
+  },
+  .data_size = sizeof(test_A8_RLE_align1_map),
   .data = test_A8_RLE_align1_map,
+  .reserved = NULL,
 };
 
