@@ -1,26 +1,27 @@
 
 #if defined(LV_LVGL_H_INCLUDE_SIMPLE)
 #include "lvgl.h"
+#elif defined(LV_LVGL_H_INCLUDE_SYSTEM)
+#include <lvgl.h>
 #elif defined(LV_BUILD_TEST)
 #include "../lvgl.h"
 #else
 #include "lvgl/lvgl.h"
 #endif
 
-
 #ifndef LV_ATTRIBUTE_MEM_ALIGN
 #define LV_ATTRIBUTE_MEM_ALIGN
 #endif
 
-#ifndef LV_ATTRIBUTE_IMG_DUST
-#define LV_ATTRIBUTE_IMG_DUST
+#ifndef LV_ATTRIBUTE_TEST_I8_RLE_ALIGN1
+#define LV_ATTRIBUTE_TEST_I8_RLE_ALIGN1
 #endif
 
 static const
-LV_ATTRIBUTE_MEM_ALIGN LV_ATTRIBUTE_LARGE_CONST LV_ATTRIBUTE_IMG_DUST
+LV_ATTRIBUTE_MEM_ALIGN LV_ATTRIBUTE_LARGE_CONST LV_ATTRIBUTE_TEST_I8_RLE_ALIGN1
 uint8_t test_I8_RLE_align1_map[] = {
 
-    0x01,0x00,0x00,0x00,0x32,0x12,0x00,0x00,0xc1,0x16,0x00,0x00,0xff,0x4c,0x70,0x47,
+    0x01,0x00,0x00,0x00,0x31,0x12,0x00,0x00,0xc0,0x16,0x00,0x00,0xff,0x4c,0x70,0x47,
     0x00,0x00,0xff,0xff,0xa5,0x00,0xff,0xff,0x02,0x00,0xff,0xff,0x58,0x00,0xff,0xff,
     0x7a,0x00,0xff,0xff,0xe1,0x00,0xff,0xff,0xb6,0x00,0xff,0xff,0x1c,0x00,0xff,0xff,
     0x69,0x00,0xff,0xff,0x2d,0x00,0xff,0xff,0xf2,0x00,0xff,0xff,0xd0,0x00,0xff,0xff,
@@ -311,18 +312,22 @@ uint8_t test_I8_RLE_align1_map[] = {
     0x00,0x00,0x00,0x9d,0x00,0x44,0x00,0x8c,0x9d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
     0x00,0x00,0x9d,0x00,0x44,0x00,0x8c,0x9d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
     0x00,0x9d,0x00,0x44,0x00,0x8b,0x9d,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
-    0x9d,0x46,0x9d,0x8a,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+    0x9d,0x46,0x9d,0x89,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
 
 };
 
-const lv_img_dsc_t test_I8_RLE_align1 = {
-  .header.magic = LV_IMAGE_HEADER_MAGIC,
-  .header.cf = LV_COLOR_FORMAT_I8,
-  .header.flags = 0 | LV_IMAGE_FLAGS_COMPRESSED,
-  .header.w = 71,
-  .header.h = 60,
-  .header.stride = 80,
-  .data_size = 4670,
+const lv_image_dsc_t test_I8_RLE_align1 = {
+  .header = {
+    .magic = LV_IMAGE_HEADER_MAGIC,
+    .cf = LV_COLOR_FORMAT_I8,
+    .flags = 0 | LV_IMAGE_FLAGS_COMPRESSED,
+    .w = 71,
+    .h = 60,
+    .stride = 80,
+    .reserved_2 = 0,
+  },
+  .data_size = sizeof(test_I8_RLE_align1_map),
   .data = test_I8_RLE_align1_map,
+  .reserved = NULL,
 };
 
