@@ -1,26 +1,27 @@
 
 #if defined(LV_LVGL_H_INCLUDE_SIMPLE)
 #include "lvgl.h"
+#elif defined(LV_LVGL_H_INCLUDE_SYSTEM)
+#include <lvgl.h>
 #elif defined(LV_BUILD_TEST)
 #include "../lvgl.h"
 #else
 #include "lvgl/lvgl.h"
 #endif
 
-
 #ifndef LV_ATTRIBUTE_MEM_ALIGN
 #define LV_ATTRIBUTE_MEM_ALIGN
 #endif
 
-#ifndef LV_ATTRIBUTE_IMG_DUST
-#define LV_ATTRIBUTE_IMG_DUST
+#ifndef LV_ATTRIBUTE_TEST_RGB565_RLE_ALIGN1
+#define LV_ATTRIBUTE_TEST_RGB565_RLE_ALIGN1
 #endif
 
 static const
-LV_ATTRIBUTE_MEM_ALIGN LV_ATTRIBUTE_LARGE_CONST LV_ATTRIBUTE_IMG_DUST
+LV_ATTRIBUTE_MEM_ALIGN LV_ATTRIBUTE_LARGE_CONST LV_ATTRIBUTE_TEST_RGB565_RLE_ALIGN1
 uint8_t test_RGB565_RLE_align1_map[] = {
 
-    0x01,0x00,0x00,0x00,0x2c,0x18,0x00,0x00,0xc2,0x21,0x00,0x00,0x47,0xe0,0x07,0x83,
+    0x01,0x00,0x00,0x00,0x2a,0x18,0x00,0x00,0xc0,0x21,0x00,0x00,0x47,0xe0,0x07,0x83,
     0x00,0x00,0xe0,0x07,0xff,0xff,0x44,0xff,0xff,0x84,0xe0,0x07,0x00,0x00,0xe0,0x07,
     0xff,0xff,0x44,0xff,0xff,0x8e,0xe0,0x07,0x00,0x00,0xe0,0x07,0xff,0xff,0xff,0xff,
     0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
@@ -407,18 +408,22 @@ uint8_t test_RGB565_RLE_align1_map[] = {
     0xff,0xff,0xff,0xff,0xff,0xff,0xe0,0x07,0x00,0x00,0xe0,0x07,0xff,0xff,0x44,0xff,
     0xff,0x84,0xe0,0x07,0x00,0x00,0xe0,0x07,0xff,0xff,0x44,0xff,0xff,0x84,0xe0,0x07,
     0x00,0x00,0xe0,0x07,0xff,0xff,0x44,0xff,0xff,0x83,0xe0,0x07,0x00,0x00,0xe0,0x07,
-    0x46,0xe0,0x07,0x82,0x00,0x00,0x00,0x00,
+    0x46,0xe0,0x07,0x81,0x00,0x00,
 
 };
 
-const lv_img_dsc_t test_RGB565_RLE_align1 = {
-  .header.magic = LV_IMAGE_HEADER_MAGIC,
-  .header.cf = LV_COLOR_FORMAT_RGB565,
-  .header.flags = 0 | LV_IMAGE_FLAGS_COMPRESSED,
-  .header.w = 71,
-  .header.h = 60,
-  .header.stride = 144,
-  .data_size = 6200,
+const lv_image_dsc_t test_RGB565_RLE_align1 = {
+  .header = {
+    .magic = LV_IMAGE_HEADER_MAGIC,
+    .cf = LV_COLOR_FORMAT_RGB565,
+    .flags = 0 | LV_IMAGE_FLAGS_COMPRESSED,
+    .w = 71,
+    .h = 60,
+    .stride = 144,
+    .reserved_2 = 0,
+  },
+  .data_size = sizeof(test_RGB565_RLE_align1_map),
   .data = test_RGB565_RLE_align1_map,
+  .reserved = NULL,
 };
 
