@@ -1,26 +1,27 @@
 
 #if defined(LV_LVGL_H_INCLUDE_SIMPLE)
 #include "lvgl.h"
+#elif defined(LV_LVGL_H_INCLUDE_SYSTEM)
+#include <lvgl.h>
 #elif defined(LV_BUILD_TEST)
 #include "../lvgl.h"
 #else
 #include "lvgl/lvgl.h"
 #endif
 
-
 #ifndef LV_ATTRIBUTE_MEM_ALIGN
 #define LV_ATTRIBUTE_MEM_ALIGN
 #endif
 
-#ifndef LV_ATTRIBUTE_IMG_DUST
-#define LV_ATTRIBUTE_IMG_DUST
+#ifndef LV_ATTRIBUTE_TEST_ARGB8888_RLE_ALIGN64
+#define LV_ATTRIBUTE_TEST_ARGB8888_RLE_ALIGN64
 #endif
 
 static const
-LV_ATTRIBUTE_MEM_ALIGN LV_ATTRIBUTE_LARGE_CONST LV_ATTRIBUTE_IMG_DUST
+LV_ATTRIBUTE_MEM_ALIGN LV_ATTRIBUTE_LARGE_CONST LV_ATTRIBUTE_TEST_ARGB8888_RLE_ALIGN64
 uint8_t test_ARGB8888_RLE_align64_map[] = {
 
-    0x01,0x00,0x00,0x00,0x34,0x30,0x00,0x00,0x84,0x43,0x00,0x00,0x47,0x00,0xff,0x00,
+    0x01,0x00,0x00,0x00,0x30,0x30,0x00,0x00,0x80,0x43,0x00,0x00,0x47,0x00,0xff,0x00,
     0xff,0x83,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0xff,0x00,0x00,0x00,0x00,0x44,0x00,
     0x00,0x00,0x00,0x84,0x00,0xff,0x00,0xff,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0xff,
     0x00,0x00,0x00,0x00,0x44,0x00,0x00,0x00,0x00,0x96,0x00,0xff,0x00,0xff,0x00,0x00,
@@ -791,18 +792,22 @@ uint8_t test_ARGB8888_RLE_align64_map[] = {
     0x00,0x00,0x00,0xff,0x00,0xff,0x00,0x00,0x00,0x00,0x44,0x00,0x00,0x00,0x00,0x84,
     0x00,0xff,0x00,0xff,0x00,0x00,0x00,0x00,0x00,0xff,0x00,0xff,0x00,0x00,0x00,0x00,
     0x44,0x00,0x00,0x00,0x00,0x83,0x00,0xff,0x00,0xff,0x00,0x00,0x00,0x00,0x00,0xff,
-    0x00,0xff,0x46,0x00,0xff,0x00,0xff,0x82,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+    0x00,0xff,0x46,0x00,0xff,0x00,0xff,0x81,0x00,0x00,0x00,0x00,
 
 };
 
-const lv_img_dsc_t test_ARGB8888_RLE_align64 = {
-  .header.magic = LV_IMAGE_HEADER_MAGIC,
-  .header.cf = LV_COLOR_FORMAT_ARGB8888,
-  .header.flags = 0 | LV_IMAGE_FLAGS_COMPRESSED,
-  .header.w = 71,
-  .header.h = 60,
-  .header.stride = 288,
-  .data_size = 12352,
+const lv_image_dsc_t test_ARGB8888_RLE_align64 = {
+  .header = {
+    .magic = LV_IMAGE_HEADER_MAGIC,
+    .cf = LV_COLOR_FORMAT_ARGB8888,
+    .flags = 0 | LV_IMAGE_FLAGS_COMPRESSED,
+    .w = 71,
+    .h = 60,
+    .stride = 288,
+    .reserved_2 = 0,
+  },
+  .data_size = sizeof(test_ARGB8888_RLE_align64_map),
   .data = test_ARGB8888_RLE_align64_map,
+  .reserved = NULL,
 };
 
