@@ -1,26 +1,27 @@
 
 #if defined(LV_LVGL_H_INCLUDE_SIMPLE)
 #include "lvgl.h"
+#elif defined(LV_LVGL_H_INCLUDE_SYSTEM)
+#include <lvgl.h>
 #elif defined(LV_BUILD_TEST)
 #include "../lvgl.h"
 #else
 #include "lvgl/lvgl.h"
 #endif
 
-
 #ifndef LV_ATTRIBUTE_MEM_ALIGN
 #define LV_ATTRIBUTE_MEM_ALIGN
 #endif
 
-#ifndef LV_ATTRIBUTE_IMG_DUST
-#define LV_ATTRIBUTE_IMG_DUST
+#ifndef LV_ATTRIBUTE_TEST_RGB565A8_RLE_ALIGN64
+#define LV_ATTRIBUTE_TEST_RGB565A8_RLE_ALIGN64
 #endif
 
 static const
-LV_ATTRIBUTE_MEM_ALIGN LV_ATTRIBUTE_LARGE_CONST LV_ATTRIBUTE_IMG_DUST
+LV_ATTRIBUTE_MEM_ALIGN LV_ATTRIBUTE_LARGE_CONST LV_ATTRIBUTE_TEST_RGB565A8_RLE_ALIGN64
 uint8_t test_RGB565A8_RLE_align64_map[] = {
 
-    0x01,0x00,0x00,0x00,0x3e,0x1c,0x00,0x00,0xa2,0x32,0x00,0x00,0x47,0xe0,0x07,0x83,
+    0x01,0x00,0x00,0x00,0x3c,0x1c,0x00,0x00,0xa0,0x32,0x00,0x00,0x47,0xe0,0x07,0x83,
     0x00,0x00,0xe0,0x07,0x00,0x00,0x44,0x00,0x00,0x84,0xe0,0x07,0x00,0x00,0xe0,0x07,
     0x00,0x00,0x44,0x00,0x00,0x96,0xe0,0x07,0x00,0x00,0xe0,0x07,0x00,0x00,0x00,0x00,
     0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
@@ -472,18 +473,22 @@ uint8_t test_RGB565A8_RLE_align64_map[] = {
     0x60,0x57,0x50,0x47,0x3f,0x36,0x2d,0x24,0x1c,0x13,0x0a,0x02,0x00,0x00,0x00,0x00,
     0xff,0x00,0xff,0x00,0x00,0x00,0x21,0x00,0x00,0x83,0xff,0x00,0xff,0x00,0x00,0x00,
     0x21,0x00,0x00,0x83,0xff,0x00,0xff,0x00,0x00,0x00,0x21,0x00,0x00,0x82,0xff,0x00,
-    0xff,0xff,0x22,0xff,0xff,0x82,0xff,0x00,0x00,0x00,
+    0xff,0xff,0x22,0xff,0xff,0x81,0xff,0x00,
 
 };
 
-const lv_img_dsc_t test_RGB565A8_RLE_align64 = {
-  .header.magic = LV_IMAGE_HEADER_MAGIC,
-  .header.cf = LV_COLOR_FORMAT_RGB565A8,
-  .header.flags = 0 | LV_IMAGE_FLAGS_COMPRESSED,
-  .header.w = 71,
-  .header.h = 60,
-  .header.stride = 144,
-  .data_size = 7242,
+const lv_image_dsc_t test_RGB565A8_RLE_align64 = {
+  .header = {
+    .magic = LV_IMAGE_HEADER_MAGIC,
+    .cf = LV_COLOR_FORMAT_RGB565A8,
+    .flags = 0 | LV_IMAGE_FLAGS_COMPRESSED,
+    .w = 71,
+    .h = 60,
+    .stride = 144,
+    .reserved_2 = 0,
+  },
+  .data_size = sizeof(test_RGB565A8_RLE_align64_map),
   .data = test_RGB565A8_RLE_align64_map,
+  .reserved = NULL,
 };
 
