@@ -7,28 +7,23 @@
 #ifndef UL_COMMAND_LINE_H
 #define UL_COMMAND_LINE_H
 
+#include "../shared/cli_common.h"
+
 #include <stdbool.h>
 
 /**
  * Options parsed from command line arguments
  */
 typedef struct {
+    /* Common cli options */
+    bbx_cli_common_opts common;
     /* Number of config files */
     int num_config_files;
     /* Paths of config file */
     const char **config_files;
-    /* Horizontal display resolution */
-    int hor_res;
-    /* Vertical display resolution */
-    int ver_res;
-    /* Horizontal display offset */
-    int x_offset;
-    /* Vertical display offset */
-    int y_offset;
-    /* DPI */
-    int dpi;
-    /* Verbose mode. If true, provide more detailed logging output on STDERR. */
-    bool verbose;
+    /* Message for a user */
+    const char *message;
+    bool newline;
 } ul_cli_opts;
 
 /**
